unit KernelUtilities;

// For info on use, e-mail PII                         
//                      or PII                 
//
// Requires the RPCBroker development components for Delphi
// Requires the Fileman components for Delphi
//
// Originally built for the CAPRI application
//
// Thanks for your help, Joel and Wally!  :-)

interface

uses Fmcmpnts,
  Fmcntrls,
  dialogs,
  Trpcb,
  classes,
  sysutils;

function kernel_AssignMenuToSecondary(xBroker: TRPCBroker; xDUZ: string; xMenu: string): string;
function kernel_AssignMenuToPrimary(xBroker: TRPCBroker; xDUZ: string; xMenu: string): string;

implementation

// function Kernel_AssignMenuToPrimary(xBroker: TRPCBroker; xDUZ: String; xMenu: String):string;
//
// Checks if user's primary menu matches the passed in menu.
// If not, the menu is changed to the new menu.
//
// Params:
//      xBroker = A TRPCBroker component already instantiated in the main application
//      xDUZ = User's DUZ # (IEN in NEW PERSON file) to which the menu option should be assigned
//      xMenu = The exact name of the menu being assigned - .01 field of file 19 (OPTION file)
//
// Result:  1^Success message, if any
//          0^Error/Info message/No changes made message

function Kernel_AssignMenuToPrimary(xBroker: TRPCBroker; xDUZ: string; xMenu: string): string;
var
  xFMFinder: TFMFinder;
  xFMGets: TFMGets;
  xFMFiler: TFMFiler;
  Options: TStringList;
  OptionIEN: string;
  x: integer;
  FoundFlag: Integer;
begin
  xMenu := uppercase(xMenu);

  // Check if the broker is connected.  If not, don't continue.
  if xBroker.Connected = False then begin
    Result := '0^The RPCBroker does not have an active connection.';
    exit;
  end;

  // Search for a match on the specified option.  Get the IEN.
  // Use the Finder for speed
  Options := TStringList.Create;
  xFMFinder := TFMFinder.Create(xBroker);
  xFMFinder.FileNumber := '19';
  xFMFinder.RPCBroker := xBroker;
  xFMFinder.Value := xMenu;
  xFMFinder.GetFinderList(Options);
  FoundFlag := -1;
  if Options.Count > 0 then
    for x := 0 to Options.Count - 1 do
      if Options[x] = xMenu then begin
        FoundFlag := x;
        OptionIEN := xFMFinder.Results[x];
      end;
  xFMFinder.Free;
  Options.Free;
  if FoundFlag = -1 then begin
    Result := '0^An exact match for option ' + xMenu + ' wasn''t found on this system.';
    exit;
  end;
  // OptionIEN now contains the IEN in the option file for the specified menu option in the params

  // Check the user's primary menu in the NEW PERSON file.
  xFMGets := TFMGets.Create(xBroker);
  xFMGets.FileNumber := '200';
  xFMGets.FieldNumbers.Add('201');
  xFMGets.IENS := xDuz;
  xFMGets.RPCBroker := xBroker;
  xFMGets.GetData;

  // If no primary menu, set the primary menu
  try
    if xFMGets.GetField('201').FMDBInternal = '' then begin
      xFMFiler := TFMFiler.Create(xBroker);
      xFMFiler.RPCBroker := xBroker;
      xFMFiler.AddFDA('200', xDUZ, '201', OptionIEN);
      if not xFMFiler.Update then begin
        xFMFiler.Free;
        Result := '0^There was a problem setting the specified user''s primary menu to the specified option.';
        exit;
      end;
      xFMFiler.Free;
  end except begin
      xFMGets.Free;
      Result := '0^There was a problem reading the primary menu for the specified user DUZ.';
      exit;
    end;
  end;

  // Check to see if the option passed into the function is already the user's primary menu
  if xFMGets.GetField('201').FMDBInternal = OptionIEN then begin
    xFMGets.Free;
    Result := '1^The option being assigned is already the user''s primary menu.';
    exit;
  end;
  xFMGets.Free;

  // Overwrite primary menu with the new menu being assigned.
  xFMFiler := TFMFiler.Create(xBroker);
  xFMFiler.RPCBroker := xBroker;
  xFMFiler.AddFDA('200', xDUZ, '201', OptionIEN);
  if xFMFiler.AnythingToFile then
    if xFMFiler.DataProblemCheck then begin
      xFMFiler.Free;
      Result := '0^There was a problem setting the specified user''s primary menu to the specified option.';
      exit;
    end
    else
      if not xFMFiler.Update then begin
        xFMFiler.Free;
        Result := '0^There was a problem setting the specified user''s primary menu to the specified option.';
        exit;
      end;
  xFMFiler.Free;

  Result := '1^The primary menu matches the specified option.';
end;

// function Kernel_AssignMenuToSecondary(xBroker: TRPCBroker; xDUZ: String; xMenu: String):string;
//
// Checks if a user has a primary menu.  If not, mailman is assigned.
// If the specified option is the user's primary menu, the function exits with a result of 1^
// Checks if a user has the specified menu as a secondary.  If not, it's assigned
//
// Params:
//      xBroker = A TRPCBroker component already instantiated in the main application
//      xDUZ = User's DUZ # (IEN in NEW PERSON file) to which the menu option should be assigned
//      xMenu = The exact name of the menu being assigned - .01 field of file 19 (OPTION file)
//
// Result:  1^Success message, if any
//          0^Error message

function Kernel_AssignMenuToSecondary(xBroker: TRPCBroker; xDUZ: string; xMenu: string): string;
var
  xFMFinder: TFMFinder;
  xFMGets: TFMGets;
  xFMFiler: TFMFiler;
  xFMLister: TFMLister;
  Options: TStringList;
  OptionIEN: string;
  MailManIEN: string;
  x: integer;
  FoundFlag: integer;
begin
  xMenu := uppercase(xMenu);

  // Check if the broker is connected.  If not, don't continue.
  if xBroker.Connected = False then begin
    Result := '0^The RPCBroker does not have an active connection.';
    exit;
  end;

  // Search for a match on the specified option.  Get the IEN.
  // Use the Finder for speed
  Options := TStringList.Create;
  xFMFinder := TFMFinder.Create(xBroker);
  xFMFinder.FileNumber := '19';
  xFMFinder.RPCBroker := xBroker;
  xFMFinder.Value := xMenu;
  xFMFinder.GetFinderList(Options);
  FoundFlag := -1;
  if Options.Count > 0 then
    for x := 0 to Options.Count - 1 do
      if Options[x] = xMenu then begin
        FoundFlag := x;
        OptionIEN := xFMFinder.Results[x];
      end;
  xFMFinder.Free;
  Options.Free;
  if FoundFlag = -1 then begin
    Result := '0^An exact match for option ' + xMenu + ' wasn''t found on this system.';
    exit;
  end;
  // OptionIEN now contains the IEN in the option file for the specified menu option in the params

  // Grab the MailMan Menu Option IEN - XMUSER
  // Use the Finder for speed
  Options := TStringList.Create;
  xFMFinder := TFMFinder.Create(xBroker);
  xFMFinder.FileNumber := '19';
  xFMFinder.RPCBroker := xBroker;
  xFMFinder.Value := 'XUSERTOOLS';
  xFMFinder.GetFinderList(Options);
  FoundFlag := -1;
  if Options.Count > 0 then
    for x := 0 to Options.Count - 1 do
      if Options[x] = 'XUSERTOOLS' then begin
        FoundFlag := x;
        MailManIEN := xFMFinder.Results[x];
      end;
  xFMFinder.Free;
  Options.Free;
  if FoundFlag = -1 then begin
    Result := '0^An exact match for option XUSERTOOLS wasn''t found on this system.';
    exit;
  end;
  // MailmanIEN now contains the MailMan IEN in the option file.

  // Check the user's primary menu in the NEW PERSON file.
  // If blank, stuff in the Mailman IEN
  xFMGets := TFMGets.Create(xBroker);
  xFMGets.FileNumber := '200';
  xFMGets.FieldNumbers.Add('201');
  xFMGets.IENS := xDuz;
  xFMGets.RPCBroker := xBroker;
  xFMGets.GetData;

  //Showmessage(xFMGets.GetField('201').FMDBExternal);
  // If no primary menu, set the primary menu to XUSERTOOLS
  try
    if xFMGets.GetField('201').FMDBInternal = '' then begin
      xFMFiler := TFMFiler.Create(xBroker);
      xFMFiler.RPCBroker := xBroker;
      xFMFiler.AddFDA('200', xDUZ, '201', MailManIEN);
      if not xFMFiler.Update then begin
        xFMFiler.Free;
        Result := '0^There was a problem setting the specified user''s primary menu to XUSERTOOLS.';
        exit;
      end;
      xFMFiler.Free;
  end except begin
      xFMGets.Free;
      Result := '0^There was a problem reading the primary menu for the specified user DUZ.';
      exit;
    end;
  end;
  // Check to see if the secondary option passed into the function is the user's primary menu
  if xFMGets.GetField('201').FMDBInternal = OptionIEN then begin
    xFMGets.Free;
    Result := '1^The secondary option being assigned is already the user''s primary menu.';
    exit;
  end;
  xFMGets.Free;

  // Now load the secondary menu options for the user and check if the user already has
  // the option to be added.
  xFMLister := TFMLister.Create(xBroker);
  xFMLister.FileNumber := '200.03';
  xFMLister.IENS := ',' + xDuz + ',';
  xFMLister.RPCBroker := xBroker;
  Options := TStringList.Create;
  xFMLister.GetList(Options);
  FoundFlag := -1;
  if Options.Count > -1 then
    for x := 0 to Options.Count - 1 do
      if Options[x] = xMenu then begin
        // User already has the secondary option specified
        FoundFlag := x;
      end;
  xFMLister.Free;
  Options.Free;
  if FoundFlag = -1 then begin
    // User doesn't have the secondary option yet, so add it
    xFMFiler := TFMFiler.Create(xBroker);
    xFMFiler.RPCBroker := xBroker;
    xFMFiler.AddFDA('200.03', '+1,' + xDUZ + ',', '.01', OptionIEN);
    if xFMFiler.AnythingToFile then
      if xFMFiler.DataProblemCheck then begin
        xFMFiler.Free;
        Result := '0^There was a problem adding the specified option to the user''s secondary menu.';
        exit;
      end
      else
        if not xFMFiler.Update then begin
          xFMFiler.Free;
          Result := '0^There was a problem adding the specified option to the user''s secondary menu.';
          exit;
        end;
    xFMFiler.Free;
  end;

  Result := '1^The secondary option being assigned is available to the user on their secondary menu.';
end;

end.
