{
CodeCR565 - Unmerging the PDF file that is sent to Virtual VA.  Virtual VA will
still receive a merged PDF, but the DBQs sent to VLER/DAS will now have XML and
the PDF associated with that XML only.  This unit supports the unmerging of the
PDF files.

Some of the functions in this unit are copied from other areas of the code -
specifically preparing and sending a PDF to Virtual VA.  Because of time and 
testing constraints, the functions were copied here rather than altering
anything related to Virtual VA.  

JRL, July 2014
}

unit SplitExamInfo;

interface

uses Windows, SysUtils, Classes, Dialogs, DateUtils, CommCtrl, Types, Graphics,
     Controls, StrUtils, RpRender, RpRenderPDF, RpDefine, RpBase, RpSystem,
     ShellAPI, ComCtrls, Forms, DialogsCAPRI, CAPRISupport;

type 

  TvlerVet = record
    FullName: String;
    SSN: String;
  end;

  TvlerDoc = record  //Virtual VA web service parameters
    OutputFilePath: String;
    OutputFileName: String;
  end;
  
  TvlerReport = record   //Report generation parameters
    ReportText: TStringList;
    FooterDesc: String;
    FontSize: Integer;
    FontName: String;
    SystemName: String;
    Division: String;
    PrintDate: String;
    Patient: TvlerVet;
    vlerDoc: TvlerDoc;
    hdrTitle : String;
    hdrDate : String;
    hdrAuthor : String;
    hdrCosigner : String;
    hdrStatus : String;
    hdrUrgency : String;
    DateOfNote : String;
  end;

TSplitExam = Class
  CONST
    PDF_EXTENT = '.PDF';
    TXT_EXTENT = '.TXT';
  private
    OutputFilePath, OutputFileName : string;
    // tempFolder: array[0..MAX_PATH] of Char;             // CodeCR496 - LMS - 2014-07-14 consolidate/refactor all GetTempDir into one location, so all can benefit from Win 7 specific handling.
    RaveSystem: TRvSystem;
    RaveRenderPDF: TRvRenderPDF;
    procedure RavePrint(Sender: TObject);
    procedure RavePrintFooter(Sender: TObject);
    procedure RavePrintHeader(Sender: TObject);
    procedure RaveBeforePrint(Sender: TObject);
    procedure PrintLine(ReportComponent: TBaseReport; ReportLine: String);
    function GetFileSize(const FileName: String): LongInt;
    function ReplaceTabs(LineWithTabs: String; tabSize: Integer): String;
    function CreateXmitFile(var vlerReport: TvlerReport): LongInt;
    function GetServerDateTime(): String;
    procedure CompressPDF(UncompressedFilename, CompressedFilename: String);
    function PadString(S: string; Ch: Char; Len: Integer): string;
  public           
    vlerReport : TvlerReport;
  published
//    constructor Create; Overload;
//    Destructor  Destroy; override; 
    function ProcessTextToPDF (eSignature, eTitle, eSignatureDateTime, examAuthor, examCosigner, title, DateOfNote : String) : Boolean;
end;

implementation

uses Main
     , trpcb
     , untMiscMthds
     ;


//------------------------------------------------------------------------------
// CONSTRUCTOR
//------------------------------------------------------------------------------
//constructor TSplitExam.Create;
//begin
//   inherited;
//
//end;
//------------------------------------------------------------------------------
// DESTRUCTOR
//------------------------------------------------------------------------------
//destructor TSplitExam.Destroy;
//begin
//  inherited;                                     
//end;


//------------------------------------------------------------------------------
// ProcessTextToPDF
//------------------------------------------------------------------------------
function TSplitExam.ProcessTextToPDF (eSignature, eTitle, eSignatureDateTime, examAuthor, examCosigner, title, DateOfNote : String) : Boolean;
var
  x : Integer;
  len : Integer;
  
begin
  //GetTempPath(MAX_PATH, @tempFolder);
  vlerReport.FontName := 'Courier New';
  vlerReport.FontSize := 10;
  vlerReport.Patient.FullName := patientname;
  vlerReport.Patient.SSN := patientssn;
  vlerReport.PrintDate := GetServerDateTime(); 
  VlerReport.SystemName := Uppercase(frmMain.RPCBroker1.ANUStrServer);
  if CallRPC(frmMain.RPCBroker1, 'XWB GET VARIABLE VALUE', ['DUZ("2")'], nil, True) then
     vlerReport.Division := frmMain.RPCBroker1.Results[0];
  vlerReport.FooterDesc := 'CLIN DOC: Progress Note';
  vlerReport.ReportText := TStringList.Create;
  vlerReport.ReportText.Clear;
  vlerReport.hdrTitle := title;
  vlerReport.hdrDate := eSignatureDateTime;
  vlerReport.hdrAuthor := examAuthor;
  vlerReport.hdrCosigner := examCosigner;
  vlerReport.hdrUrgency := ' '; // where do we get this?
  vlerReport.DateOfNote := DateOfNote;

  for x := 0 to frmMain.NumExamTextForPDFs - 1 do
  begin
    //vlerReport.vlerDoc.OutputFilePath := StrPas(tempFolder);  // ??? should this have a CAPRI subfolder?
    vlerReport.vlerDoc.OutputFilePath := GetTempDir;  // DONE: // CodeCR496 - LMS - 2014-07-14 consolidate/refactor all GetTempDir into one location, so all can benefit from Win 7 specific handling.

    vlerReport.vlerDoc.OutputFileName := 'Exam' + IntToStr(x) + PDF_EXTENT;
    vlerReport.ReportText.Clear;
    frmMain.ExamTextForPDFs[x].Position := 0;
    vlerReport.ReportText.LoadFromStream(frmMain.ExamTextForPDFs[x]);  // Get Exam Text
    // add in signature here - then go to the first and add header info
    vlerReport.ReportText.Append('/es/ ' + eSignature);
    vlerReport.ReportText.Append(eTitle);
    vlerReport.ReportText.Append('Signed: ' + eSignatureDateTime);
    vlerReport.ReportText.Insert(0,'     URGENCY: ' + PadString(vlerReport.hdrUrgency,' ',26) + ' STATUS: COMPLETED');   // ????? WHERE TO GET URGENCY?
    vlerReport.ReportText.Insert(0,'      AUTHOR: ' + PadString(vlerReport.hdrAuthor,' ',19) + '  EXP COSIGNER: ' + vlerReport.hdrCosigner);
    vlerReport.ReportText.Insert(0,'DATE OF NOTE: ' + vlerReport.DateOfNote + '  ENTRY DATE: ' + vlerReport.hdrDate);
    vlerReport.ReportText.Insert(0,' LOCAL TITLE: ' + title); 
    
    RaveSystem := TRvSystem.Create(nil);
    with RaveSystem do
    begin
      TitleSetup := 'Output Options';
      TitleStatus := 'Report Status';
      TitlePreview := 'Report Preview';
      SystemSetups := [ssAllowCollate, ssAllowDuplex, ssAllowDestPreview, ssAllowDestPrinter, ssAllowDestFile, ssAllowPrinterSetup, ssAllowPreviewSetup];
      SystemOptions := [soAllowPrintFromPreview, soAllowSaveFromPreview, soPreviewModal];
      SystemFiler.StatusFormat := 'Generating page %p';
      SystemPreview.ZoomFactor := 100.000000000000000000;
      SystemPrinter.MarginBottom := 0.500000000000000000;
      SystemPrinter.MarginLeft := 0.500000000000000000;
      SystemPrinter.MarginRight := 0.500000000000000000;
      SystemPrinter.MarginTop := 0.500000000000000000;
      SystemPrinter.ScaleX := 100.000000000000000000;
      SystemPrinter.ScaleY := 100.000000000000000000;
      SystemPrinter.StatusFormat := 'Printing page %p';
      SystemPrinter.Title := 'Rave Report';
      SystemPrinter.UnitsFactor := 1.000000000000000000;
    end;

    RaveRenderPDF := TRvRenderPDF.Create(nil);
    with RaveRenderPDF do
    begin
      DisplayName := 'Adobe Acrobat (PDF)';
      FileExtension := '*.pdf';
      EmbedFonts := True;
      ImageQuality := 90;
      MetafileDPI := 300;
      FontEncoding := feWinAnsiEncoding;
      DocInfo.Creator := 'Rave Reports (http://www.nevrona.com/rave)';
      DocInfo.Producer := 'Nevrona Designs';
      BufferDocument := True;
      DisableHyperlinks := False;
    end;

    // Make sure that PDF is compressed before it is updated into memory!!!  CodeCR565 JRL 7/13/14
    if (CreateXmitFile(vlerReport) > 0) then  //File ready to transmit
       Result := TRUE    // success
    else
       Result := FALSE;  // failure

    // update the memory with the updated report
    frmMain.ExamTextForPDFs[x].LoadFromFile(vlerReport.vlerDoc.OutputFilePath + 'c' + vlerReport.vlerDoc.OutputFileName);
  end;       
  
  FreeAndNil(RaveSystem);
  FreeAndNil(RaveRenderPDF);
  FreeAndNil(vlerReport.ReportText);

end;

{============================================================================
 CreateXmitFile
 This function uses Rave Reports to format and create the PDF output file that
 will be transmitted to VLER DAS.  The existing PDF is then compressed and
 linearized prior to sending to VLER DAS.

 Input:  vlerReport: TvlerReport - report object
 Output: function result - returns True on success;
                           otherwise returns False.
 ============================================================================}
function TSplitExam.CreateXmitFile(var vlerReport: TvlerReport): LongInt;
const
  FILE_EXTENT = '.PDF';
begin
// Make sure that PDF is compressed before it is updated into memory!!!  CodeCR565 JRL 7/13/14
// Do we need a flag to delete or not delete individual PDFs?

  RaveSystem.OnPrint := RavePrint;
  RaveSystem.OnPrintHeader := RavePrintHeader;
  RaveSystem.OnPrintFooter := RavePrintFooter;
  RaveSystem.DefaultDest := rdFile;
  RaveSystem.DoNativeOutput := false;
  RaveSystem.RenderObject := RaveRenderPDF;
  RaveSystem.OutputFileName := vlerReport.vlerDoc.OutputFilePath + vlerReport.vlerDoc.OutputFileName;   
  RaveSystem.SystemSetups := RaveSystem.SystemSetups - [ssAllowSetup];
  RaveSystem.Execute;
  if FileExists(RaveSystem.OutputFileName) then  
  begin
    // make a compressed version of the PDF here to send to VLER/DAS - filename has a 'c' added at the start  
    try
      CompressPDF(RaveSystem.OutputFileName,vlerReport.vlerDoc.OutputFilePath + 'c' + vlerReport.vlerDoc.OutputFileName); 
    except                                                                                  
      on E:Exception do                                                                     
        begin // Copy uncompressed file to compressed filename so it can be sent if there was an error compressing
          if FileExists(vlerReport.vlerDoc.OutputFilePath + 'c' + vlerReport.vlerDoc.OutputFileName) then
             DeleteFile(pchar(vlerReport.vlerDoc.OutputFilePath + 'c' + vlerReport.vlerDoc.OutputFileName));
          if FileExists(RaveSystem.OutputFileName) then
             MoveFile(pchar(RaveSystem.OutputFileName), pchar(vlerReport.vlerDoc.OutputFilePath + 'c' + vlerReport.vlerDoc.OutputFileName));
        end; // compress except
    end; // compress try
    Result := GetFileSize(vlerReport.vlerDoc.OutputFilePath + vlerReport.vlerDoc.OutputFileName);  // Return file size                         
  end
  else
  begin
    Result := 0;
  end;

end;


{============================================================================
 CompressPDF
 This function uses an external tool, QPDF, to compress the Rave Report 
 output PDF file to a compressed and linearized format.  This file will be
 transmitted to the Virtual VA.  QPDF must finish compressing the file before
 exiting from this routine to avoid file errors in subsequent code.

 Parameters:  
    Input:  Uncompressed PDF filename 
    Output: Compressed PDF filename 

 Required files for running QPDF:
    qpdf.exe
    qpdf13.dll
    libgcc_s_dw2-1.dll
    libstdc++6.dll

 Created routine:  CodeCR559 3/13/14 JRL
 ============================================================================}
procedure TSplitExam.CompressPDF(UncompressedFilename, CompressedFilename: String);
var
  SEInfo: TShellExecuteInfo;
  ExitCode: DWORD;
  ExecuteFile, ParamString, StartInString : string;
begin
  ParamString := '--linearize "' + UncompressedFilename + '" "' + CompressedFilename + '"';
  ExecuteFile:='qpdf.exe';
  FillChar(SEInfo, SizeOf(SEInfo), 0) ;
  SEInfo.cbSize := SizeOf(TShellExecuteInfo) ;
  with SEInfo do 
  begin
    fMask := SEE_MASK_NOCLOSEPROCESS;
    Wnd := Application.Handle;
    lpFile := PChar(ExecuteFile) ;
    lpParameters := PChar(ParamString) ; 
    StartInString := ExtractFilePath(Application.ExeName);      
    lpDirectory := PChar(StartInString);
    nShow := SW_HIDE;
  end;

  if ShellExecuteEx(@SEInfo) then 
  begin
    repeat
      Application.ProcessMessages;
      GetExitCodeProcess(SEInfo.hProcess, ExitCode) ;
    until (ExitCode <> STILL_ACTIVE) or
    Application.Terminated;
    end;
end;




{==================================================================
 RavePrint
 This procedure is the Rave Reports OnPrint handler that breaks the
 report text into pages.
 ==================================================================}
procedure TSplitExam.RavePrint(Sender: TObject);
var
  I: Integer;
  currentLineHeight: Double;
  footerLineHeight: Double;
begin
  with Sender as TBaseReport do begin
    footerLineHeight := 0.2;
    SetFont(vlerReport.FontName, vlerReport.FontSize);
    for I := 0 to vlerReport.ReportText.Count - 1 do
    begin
      PrintLine(TBaseReport(Sender), vlerReport.ReportText[I]);
      currentLineHeight := LineHeight;
      LineHeight := footerLineHeight;
      if (LinesLeft < 5) then  //need 4 lines for footer
        NewPage;
      LineHeight := currentLineHeight;
    end;
  end;
end;


{===================================================================
 RavePrintFooter
 This procedure is the Rave Reports OnPRintFooter handler that
 formats the report footer and occurs after each NewPage call.
 ===================================================================}
procedure TSplitExam.RavePrintFooter(Sender: TObject);
var
  X, Y: double;
  SavedPen, LinePen: TPen;
begin
  with Sender as TBaseReport do begin
    SaveFont(1);
    SetFont('Arial', 10);
    SectionBottom := SectionBottom - (3 * LineHeight);
    GoToFooter;

    //Draw separator line
    SavedPen := Canvas.Pen;
    LinePen := CreatePen(clBlack, psSolid, 10, pmBlack);
    try
      Canvas.Pen := LinePen;
      X := XD2I(CursorXPos);
      Y := YD2I(CursorYPos);
      MoveTo(X, Y);
      LineTo(SectionRight, Y);
    finally
      Canvas.Pen := SavedPen;
      FreeAndNil(LinePen);
    end;

    //footer line 1
    SectionBottom := SectionBottom + LineHeight;
    PrintFooter(vlerReport.FooterDesc, pjLeft);
    PrintFooter('Page: ' + IntToStr(CurrentPage), pjRight);

    //footer line 2
    SectionBottom := SectionBottom + LineHeight;
    PrintFooter(vlerReport.Patient.FullName + '  SSN#' + vlerReport.Patient.SSN, pjLeft);
    PrintFooter('Printed on: ' + vlerReport.PrintDate, pjRight);

    //footer line 3
    SectionBottom := SectionBottom + LineHeight;
    PrintFooter('System: ' +  vlerReport.SystemName, pjLeft);
    PrintFooter('Division: ' + vlerReport.Division, pjRight);

    RestoreFont(1);
  end;
end;

{=====================================================================
 RavePrintHeader
 This procedure is the placeholder for the Rave Reports OnPrintHeader
 handler.  Not used at this time.
 =====================================================================}
procedure TSplitExam.RavePrintHeader(Sender: TObject);
begin
  //Not currently used - placeholder only.
end;



{============================================================================
 PrintLine
 This procedure provides a wrapper around the TBaseReport PrintLn procedure
 to prevent Tab (#9) characters from crashing the report writer.
 ============================================================================}
procedure TSplitExam.PrintLine(ReportComponent: TBaseReport;
  ReportLine: String);
var
  cleanLine, trimmedLine: String;
  printWidth : Double;
  maxChars: Integer;
begin
  if Pos(#9, ReportLine) > 0 then
    cleanLine := ReplaceTabs(ReportLine, 6)
  else
    cleanLine := ReportLine;

  {Some reports erroneously return long strings of periods '.' that
   are intended as a keepalive for interactive terminal sessions.
   These strings cause errors when they are longer than 255 characters.
   The following code attempts to prevent lines of periods from printing and
   also wraps the remaining valid text that is longer than the available
   page print width.}
  printWidth := ReportComponent.PageWidth - ReportComponent.MarginLeft - ReportComponent.MarginRight;
  if ReportComponent.TextWidth(cleanLine) > printWidth then
  begin
    maxChars := Round(Length(cleanLine)/ReportComponent.TextWidth(cleanLine) * printWidth);
    while (Length(cleanLine) > maxChars) do
    begin
      trimmedLine := Copy(cleanLine, 0, maxChars);
      if trimmedLine <> StringOfChar('.', Length(trimmedLine)) then  //only print if not periods
        ReportComponent.PrintLn(trimmedLine);

      cleanLine := Copy(cleanLine, maxChars + 1, Length(cleanLine));
      if cleanLine = StringOfChar('.', Length(cleanLine)) then   //clear "periods only" line
        cleanLine := '';
    end;
  end;

  ReportComponent.Println(cleanLine);

end;

{==========================================================================
 ReplaceTabs
 This utility function supports the PrintLine procedure and replaces all
 tab characters with spaces in a given string based on the given tab size.
 ==========================================================================}
function TSplitExam.ReplaceTabs(LineWithTabs: String;
  tabSize: Integer): String;
var
  I, padLength : Integer;
  LineWithSpaces : String;
begin
  LineWithSpaces := '';
  for I := 1 to Length(LineWithTabs) do
  begin
    if LineWithTabs[i] = #9 then
    begin
      padLength := tabSize - (Length(LineWithSpaces) MOD tabSize);
      LineWithSpaces := LineWithSpaces + StringOfChar(' ', padLength);
    end
    else
      LineWithSpaces := LineWithSpaces + LineWithTabs[i];
  end;
  Result := LineWithSpaces;
end;
  
{=======================================================================
 RaveBeforePrint
 This procedure is the Rave Reports OnBeforePrint handler that provides
 a setup location for properties prior to the OnPrint event.
 =======================================================================}
procedure TSplitExam.RaveBeforePrint(Sender: TObject);
begin
  with Sender as TBaseReport do
  begin
    LineHeight := 0.147;  //just to match existing QuickReports spacing
  end;
end;

{=========================================================================
 GetFileSize
 This function determines the size of a given file and returns the size
 in bytes.
 =========================================================================}
function TSplitExam.GetFileSize(const FileName: String): LongInt;
var
  aFile: File of Byte;
begin
  AssignFile(aFile, FileName);
  Reset(aFile);
  try
    Result := FileSize(aFile);
  finally
    Close(aFile);
  end;
end;

{========================================================================
 GetServerDateTime
 This function retrieves the current date/time from the connected
 remote system in "HH:MM:SS am/pm" format. Refactored from
 actPrintFileExecute for reuse.

 Input: none
 Output:  returns formatted date/time on success;
          otherwise, returns empty string
 ========================================================================}
 function TSplitExam.GetServerDateTime(): String;
 begin
   frmMain.RPCBroker1.RemoteProcedure := 'DVBAB DATETIME';
   try
     frmMain.RPCBroker1.Call;
   except
     on EBrokerError do
     begin
       FormatDateTime('mmm dd, yyyy hh:mm:ss', Date);
     end;
   end;
   if (frmMain.RPCBroker1.Results.Count > 0) then
     Result := frmMain.RPCBroker1.Results[0]
   else
     Result := '';
 end;

{==================================================================
 RightPad
 This procedure right pads header strings with spaces.
 ==================================================================}
function TSplitExam.PadString(S: string; Ch: Char; Len: Integer): string;
var
  RestLen: Integer;
begin
  Result  := S;
  RestLen := Len - Length(s);
  if RestLen < 1 then Exit;
  Result := S + StringOfChar(Ch, RestLen);
//  Result := StringOfChar(Ch, RestLen) + S;
end;

  
end.
