unit VlerDasIdHTTP;

interface

uses
  SysUtils, Classes
  , IdBaseComponent
  , IdComponent
  , IdTCPConnection
  , IdTCPClient
  , IdHTTP
  , IdObjs
  , IdMultipartFormData
  , IdCookieManager
  , IdSSL, IdSSLOpenSSL
  , IdURI
  , IdGlobal
  , CCOWRPCBrokerCAPRI
  , CAPRISupport
  ;

type
  TIdHTTPVler = class(TComponent)
  private
    FManagedIdHTTP: TIdHTTP;
    FBroker: TCCOWRPCBrokerCAPRI;
    FIdSSLIOHandlerSocketOpenSSL: TIdSSLIOHandlerSocketOpenSSL;

    Class function GetVlerDasTokenInt: String;
    function GetVlerDasToken: String;
    //Class function GetVlerDasTokenWTS(ABroker: TCCOWRPCBrokerCAPRI): String;       // todo: lms May add support for midnight hour by using timestamps. Proxy is already coded for this.

    procedure SetVlerDasToken(Value : String);
    Class procedure SetVlerDasTokenInt(Value : String; ABroker: TCCOWRPCBrokerCAPRI);

    function GetManagedIdHTTP: TIdHTTP;
    procedure SetManagedIdHTTP(const Value: TIdHTTP);
    procedure EnsureManagedIdHTTPReady;
    function GetRequest: TIdHTTPRequest;
    procedure SetRequest(const Value: TIdHTTPRequest);
    function GetCookieManager: TIdCookieManager;
    procedure SetCookieManager(const Value: TIdCookieManager);
    function GetResponse: TIdHTTPResponse;
    procedure EnsureAuthTokenReady;
    function TokenExpired : boolean;
  protected
    property VlerDasToken: string read GetVlerDasToken write SetVlerDasToken;
  public
    {Begin TIdHTTP section}
    {End TIdHTTP section}

    property ManagedIdHTTP : TIdHTTP read GetManagedIdHTTP write SetManagedIdHTTP;

    {Begin TIdHTTP section}
        property Request : TIdHTTPRequest read GetRequest write SetRequest;
        property Response: TIdHTTPResponse read GetResponse;
        property CookieManager : TIdCookieManager read GetCookieManager write SetCookieManager;
    {End TIdHTTP section}

    {Begin TIdHTTP section}
        procedure Get(AURL: string; AResponseContent: TIdStream); overload;
        procedure Get(AURL: string; AResponseContent: TIdStream; AIgnoreReplies: array of SmallInt);
         overload;
        function Get(AURL: string): string; overload;
        function Get(AURL: string; AIgnoreReplies: array of SmallInt): string; overload;

        procedure Head(AURL: string);

        function Post(AURL: string; ASource: TIdStrings): string; overload;
        function Post(AURL: string; ASource: TIdStream): string; overload;
        function Post(AURL: string; ASource: TIdStream; AIgnoreReplies: array of SmallInt): string; overload;
        function Post(AURL: string; ASource: TIdMultiPartFormDataStream): string; overload;
        procedure Post(AURL: string; ASource: TIdMultiPartFormDataStream; AResponseContent: TIdStream); overload;
        procedure Post(AURL: string; ASource: TIdStrings; AResponseContent: TIdStream); overload;

        {Post data provided by a stream, this is for submitting data to a server}
        procedure Post(AURL: string; ASource, AResponseContent: TIdStream); overload;
        procedure Post(AURL: string; ASource, AResponseContent: TIdStream; AIgnoreReplies: array of SmallInt); overload;
        procedure Post(AURL: string; ASource: TIdMultiPartFormDataStream; AResponseContent: TIdStream; AIgnoreReplies: array of SmallInt); overload;  // DONE: LMS - Added new call because in its absence due to overloading the call was going to a different method altogether.

        procedure Trace(AURL: string; AResponseContent: TIdStream); overload;
        function Trace(AURL: string): string; overload;

        function Put(AURL: string; ASource: TIdStream): string; overload;
        procedure Put(AURL: string; ASource, AResponseContent: TIdStream); overload;
        procedure Put(AURL: string; ASource, AResponseContent: TIdStream; AIgnoreReplies: array of SmallInt); overload;

    {End TIdHTTP section}

    procedure InitVLERBaseRequestHeader;  // was called SetupHTTPSrvHeader  // SetupHTTPServerHeader;  // TODO could wrap it into GET/POST method overrides.
    //constructor Create(AOwner: TComponent); overload; virtual;
    constructor Create(AOwner: TComponent; ABroker: TCCOWRPCBrokerCAPRI); reintroduce; overload; virtual; // specifically Application owner to ensure no memory leak at end of app. Could be changed in the future to drop down to TComponent but conflict with base class constructors that are overridden will need to be checked because IdHTTP is doing some specific things I've not looked into in their constructors.

    class function GetVlerDasURL: string;
    class procedure SetVlerDasURL(Value: string);
    class function ChangeVlerHostInURL(VlerURL: string): string;
    class function URLParamsEncode(const ASrc: string): string; static;
  end;

function Base64Encode(theText : string): string;   // DONE: CodeCR540 - LMS - utility function.. moved out of class.

implementation

uses
  IdCoderMIME
  , StrUtils
  , DateUtils
  , untBrkrMthds
  , untMiscMthds
  ;

const
  HTTPHeaderAcceptXML = 'application/xml';
  HTTPHeaderContentTypeText = 'text/xml';
  HTTPContentEncoding = 'utf-8';
  HTTPHeaderCapriURLVer = 'CapriURLVersion: 2.0';  // DONE: CodeCR540 - LMS - For backward (and forward) compatibility, especially when in production we'll be running multiple versions of GUI through one proxy server. The proxy will handle translation of each call based on URL version.
  HTTPHeaderCapriAuthToken = 'authToken:';

  TokenMaxLifeInMinutes = 10;

var U_VlerDasToken: String;      // a common token, last used with one broker. Will have to be modified to handle multiple brokers if needed.
    U_VlerDasTokenTimeStamp: TDateTime;
    U_VlerDasTokenBroker: TCCOWRPCBrokerCAPRI;

    U_VlerDasURL : String;   // one instance per app. main.pas sets it. This class simply uses the value, never changes it.
   // CodeCR566 LMS 2014-07-21  Moving from main form field. Needs to be available App wide in all units that need VLER access
    ProxyHostLength : integer;
    ProxyIdURI : TIdURI;
    ProxyIdURIProtocol, ProxyIdURIHost, ProxyIdURIPort, ProxyIdURIPath, ProxyIdURIDocument : string;
    ProxyIdURIHostToCompare : string;

{===============================================================================
  Base64Encode
    Base64Encode is a function that encodes an input string into base64.  The
      TIdEncoderMime results are not limited to 76 char lines.  Testing against
      the WS indicates that this is required.  As a result, the lines are
      terminated at 76 characters

  Parameters
    theText:
      A string representing the data to be encoded

  Return values
    Base64Encode returns a string representing the encoded result

  Modification History:
    patch 186 - Method created 7/2013 -MER
 ==============================================================================}
function Base64Encode(theText : string): string;
var
  lEncoder: TIdEncoderMime;
  lNdx:     integer;
  lString1: string;
  lString2: string;
begin
  lString2 := '';
  lEncoder := TIdEncoderMime.Create(nil);
  try
    lString1 := lEncoder.Encode(theText);
    // For the encoding to be accepted by the web service operation we must
    // force the line length to 76 characters terminating each line with x0D x0A
    for lNdx := 0 to (Length(lString1) - 1) do begin
      if lNdx MOD 76 = 0 then
        lString2 := lString2 + #13#10;
      lString2 := lString2 + lString1[lNdx + 1];
    end;
    Result := lString2;
  finally
    lEncoder.Free;
  end
end;

function TIdHTTPVler.GetCookieManager: TIdCookieManager;
begin
  Result := ManagedIdHTTP.CookieManager;
end;

function TIdHTTPVler.GetManagedIdHTTP: TIdHTTP;
begin
  if not assigned(FManagedIdHTTP) then
  begin
    EnsureManagedIdHTTPReady;
  end;
  Result := FManagedIdHTTP;
end;

function TIdHTTPVler.GetRequest: TIdHTTPRequest;
begin
  Result := ManagedIdHTTP.Request;
end;

function TIdHTTPVler.GetResponse: TIdHTTPResponse;
begin
  result := ManagedIdHTTP.Response;
end;

procedure TIdHTTPVler.Head(AURL: string);
begin
  ManagedIdHTTP.Head(ChangeVlerHostInURL(AURL));
end;

function TIdHTTPVler.Get(AURL: string): string;
begin
  // Check if AURL needs URLEncoding and do it.
  // Check if it needs the custom token header and set it.
  // DONE: LMS handle situations where server name is returned in the result set given by VLER and translate it to our Proxy servers.
  Result := ManagedIdHTTP.Get(ChangeVlerHostInURL(AURL){$IFDEF CAPRIDEVELOPER},[502]{$ENDIF});
end;

procedure TIdHTTPVler.Get(AURL: string; AResponseContent: TIdStream; AIgnoreReplies: array of SmallInt);
begin
  ManagedIdHTTP.Get(ChangeVlerHostInURL(AURL), AResponseContent, AIgnoreReplies);
end;

procedure TIdHTTPVler.Get(AURL: string; AResponseContent: TIdStream);
begin
  ManagedIdHTTP.Get(ChangeVlerHostInURL(AURL), AResponseContent);
end;

function TIdHTTPVler.Get(AURL: string; AIgnoreReplies: array of SmallInt): string;
begin
  Result := ManagedIdHTTP.Get(ChangeVlerHostInURL(AURL), AIgnoreReplies);
end;

function TIdHTTPVler.Post(AURL: string; ASource: TIdMultiPartFormDataStream): string;
begin
  Result := ManagedIdHTTP.Post(ChangeVlerHostInURL(AURL), ASource);
end;

function TIdHTTPVler.Post(AURL: string; ASource: TIdStream): string;
begin
  Result := ManagedIdHTTP.Post(ChangeVlerHostInURL(AURL), ASource);
end;

function TIdHTTPVler.Post(AURL: string; ASource: TIdStream; AIgnoreReplies: array of SmallInt): string;
begin
  Result := ManagedIdHTTP.Post(ChangeVlerHostInURL(AURL), ASource, AIgnoreReplies);
end;

function TIdHTTPVler.Post(AURL: string; ASource: TIdStrings): string;
begin
  Result := ManagedIdHTTP.Post(ChangeVlerHostInURL(AURL), ASource);
end;

procedure TIdHTTPVler.Post(AURL: string; ASource, AResponseContent: TIdStream);
begin
  ManagedIdHTTP.Post(ChangeVlerHostInURL(AURL), ASource, AResponseContent);
end;

procedure TIdHTTPVler.Post(AURL: string; ASource, AResponseContent: TIdStream; AIgnoreReplies: array of SmallInt);
begin
  ManagedIdHTTP.Post(ChangeVlerHostInURL(AURL), ASource, AResponseContent, AIgnoreReplies);
end;

procedure TIdHTTPVler.Post(AURL: string; ASource: TIdStrings; AResponseContent: TIdStream);
begin
  ManagedIdHTTP.Post(ChangeVlerHostInURL(AURL), ASource, AResponseContent);
end;

procedure TIdHTTPVler.Post(AURL: string; ASource: TIdMultiPartFormDataStream; AResponseContent: TIdStream; AIgnoreReplies: array of SmallInt);
begin
  ManagedIdHTTP.Post(ChangeVlerHostInURL(AURL), ASource, AResponseContent, AIgnoreReplies);
end;

procedure TIdHTTPVler.Post(AURL: string; ASource: TIdMultiPartFormDataStream; AResponseContent: TIdStream);
begin
  ManagedIdHTTP.Post(ChangeVlerHostInURL(AURL), ASource, AResponseContent);
end;

function TIdHTTPVler.Trace(AURL: string): string;
begin
  Result:= ManagedIdHTTP.Trace(ChangeVlerHostInURL(AURL));
end;

procedure TIdHTTPVler.Trace(AURL: string; AResponseContent: TIdStream);
begin
  ManagedIdHTTP.Trace(ChangeVlerHostInURL(AURL), AResponseContent);
end;

function TIdHTTPVler.Put(AURL: string; ASource: TIdStream): string;
begin
  Result := ManagedIdHTTP.Put(ChangeVlerHostInURL(AURL), ASource);
end;

procedure TIdHTTPVler.Put(AURL: string; ASource, AResponseContent: TIdStream);
begin
  ManagedIdHTTP.Put(ChangeVlerHostInURL(AURL), ASource, AResponseContent);
end;

procedure TIdHTTPVler.Put(AURL: string; ASource, AResponseContent: TIdStream; AIgnoreReplies: array of SmallInt);
begin
  ManagedIdHTTP.Put(ChangeVlerHostInURL(AURL), ASource, AResponseContent, AIgnoreReplies);
end;

{constructor TIdHTTPvler.Create(AOwner: TComponent); overload; virtual;
begin

end;}

constructor TIdHTTPvler.Create(AOwner: TComponent; ABroker: TCCOWRPCBrokerCAPRI);
begin
  inherited Create(AOwner);
  FBroker := ABroker;

  EnsureAuthTokenReady; // 'XUSBSE958-470149_6^500^9560';


  EnsureManagedIdHTTPReady;

  {
  with FManagedIdHTTP do begin
  FManagedIdHTTP.AllowCookies := True       ;
  FManagedIdHTTP.ProxyParams.BasicAuthentication := False;
  FManagedIdHTTP.ProxyParams.ProxyPort := 0;
  FManagedIdHTTP.Request.ContentLength := -1 ;
  FManagedIdHTTP.Request.Accept := 'text/html, */*';
  FManagedIdHTTP.Request.BasicAuthentication := False;
  FManagedIdHTTP.Request.UserAgent := 'Mozilla/3.0 (compatible; Indy Library) - CAPRI-IdHTTPvler';
  FManagedIdHTTP.HTTPOptions := [hoForceEncodeParams]                          ;
  end; // Left = 296
  }

end;

procedure TIdHTTPvler.InitVLERBaseRequestHeader;
begin
  with ManagedIdHTTP do begin
    ReadTimeout := 40000; // 40 seconds
    ConnectTimeout := 10000; // 10 seconds
    {when testing through proxy then uncomment this and set the server and proxy port
      ManagedIdHTTP.ProxyParams.ProxyServer := 'localhost';
      ManagedIdHTTP.ProxyParams.ProxyPort := 8080;
      ManagedIdHTTP.ProxyParams.BasicAuthentication := False;
    }
    with ManagedIdHTTP.Request do begin
      Clear;
      Accept := HTTPHeaderAcceptXML;  // 'application/xml';  we have to explicitly state what we accept, otherwise many responses are JSON and we don't accept that type yet.
      ContentType := HTTPHeaderContentTypeText;
      CustomHeaders.Clear;
      CustomHeaders.Add(HTTPHeaderCapriAuthToken + VlerDasToken);  // TODO -cNextRelease: wait LMS Instead of VlerDasToken alone, it will be replaced by a method call where the timestamp is taken into account.
      CustomHeaders.Add(HTTPHeaderCapriURLVer);  // DONE: LMS This is the solution for backward compatibility of Proxy server. All future versions that need any different handling should pass a newer version number and handle it in the proxy.
      ContentLength := -1 ;
    end;
  end;
end;

procedure TIdHTTPVler.EnsureManagedIdHTTPReady;
begin
  if not assigned(FManagedIdHTTP) then
  begin
    FManagedIdHTTP := TIdHTTP.Create(Self);
    // DONE: LMS move SSL support to common path. FIdSSLIOHandler := TIdSSLIOHandlerSocketOpenSSL.create(FManagedIdHTTP,ABroker);
    FIdSSLIOHandlerSocketOpenSSL := TIdSSLIOHandlerSocketOpenSSL.create(FManagedIdHTTP);
    // start of values from VlerDasClien.dfm prior to removing components.
    FIdSSLIOHandlerSocketOpenSSL.MaxLineAction := maException;
    FIdSSLIOHandlerSocketOpenSSL.Port := 0;
    FIdSSLIOHandlerSocketOpenSSL.DefaultPort := 0;
    FIdSSLIOHandlerSocketOpenSSL.SSLOptions.Method := sslvSSLv23;
    FIdSSLIOHandlerSocketOpenSSL.SSLOptions.Mode := sslmUnassigned;
    FIdSSLIOHandlerSocketOpenSSL.SSLOptions.VerifyMode := [];
    FIdSSLIOHandlerSocketOpenSSL.SSLOptions.VerifyDepth := 0;
    FManagedIdHTTP.IOHandler := FIdSSLIOHandlerSocketOpenSSL;
  end;

  FManagedIdHTTP.Request.UserAgent := 'Mozilla/3.0 (compatible; Indy Library) - CAPRI-TIdHTTPvler';
  FManagedIdHTTP.AllowCookies := True       ;

  InitVLERBaseRequestHeader;
  {with FManagedIdHTTP.Request do begin
    ContentLength := -1 ;
  end;}
end;

procedure TIdHTTPVler.SetCookieManager(const Value: TIdCookieManager);
begin
  ManagedIdHTTP.CookieManager := Value;
end;

procedure TIdHTTPVler.SetManagedIdHTTP(const Value: TIdHTTP);
begin
  if assigned(FManagedIdHTTP) then
    if (FManagedIdHTTP <> Value) then
      FManagedIdHTTP.Free;
  FManagedIdHTTP := Value;    // Class users have to provide a fully setup, object, i.e. SSL handler and any other owned objects and properties like Request must be all setup before setting this value
end;

procedure TIdHTTPVler.SetRequest(const Value: TIdHTTPRequest);
begin
  ManagedIdHTTP.Request := Value;
end;

// TODO -cNextRelease: wait LMS - Return Token with timestamp
function TIdHTTPVler.GetVlerDasToken: String;
begin
  EnsureAuthTokenReady;
  result := GetVlerDasTokenInt;
end;

Class function TIdHTTPVler.GetVlerDasTokenInt: String;
begin
  result := U_VlerDasToken;
end;

{
Class function TIdHTTPVler.GetVlerDasTokenWTS(ABroker: TCCOWRPCBrokerCAPRI): String;
begin  // TODO: LMS idea for future.. not finished
  result := U_VlerDasToken + '^' + GetVistATimestampUTC(ABroker);
end;
}

procedure TIdHTTPVler.SetVlerDasToken(Value: String);
begin
  SetVlerDasTokenInt(Value, FBroker);
end;

Class procedure TIdHTTPVler.SetVlerDasTokenInt(Value: String; ABroker: TCCOWRPCBrokerCAPRI);
begin
  U_VlerDasToken := Value;
  U_VlerDasTokenTimeStamp := Now;  // todo -cNextRelease: wait LMS idea for future.. Here GetVistATimestampUTC(ABroker) and note difference from current machine clock.
        // TODO -cNextRelease: LMS - if this class sends token as "token^2014-06-10T15:30:00+07:00", where the suffix is a value for "expire token by this time", then the Proxy stores it in a temporary hash table of valid tokens and doesn't keep hitting the VistA servers (CLAIMS and back to originating station) for the full - llooooong delayed - validation.
        //   todo -cNextRelease: lms - nice to have.. when application closes, either at finalization or at broker change, send Proxy and expiration notice for the old token.
  U_VlerDasTokenBroker := ABroker;
end;

class function TIdHTTPVler.GetVlerDasURL: string;
begin
  Result := U_VlerDasURL;
end;

class procedure TIdHTTPVler.SetVlerDasURL(Value: string);
begin
  value := trim(value);
  if value = '' then begin
    if assigned(ProxyIdURI) then
      FreeAndNil(ProxyIdURI);
    ProxyIdURIProtocol := '';
    ProxyIdURIHost := '';
    ProxyIdURIPort := '';
    ProxyIdURIPath := '';
    ProxyIdURIDocument := '';
    ProxyHostLength := 0;
    ProxyIdURIHostToCompare := '';
  end else begin
    {
    if copy(Value,length(Value),1) = '/' then
      Value := copy(Value,0,length(Value)-1);
    U_VlerDasURL := Value;
    }

    if assigned(ProxyIdURI) then
      FreeAndNil(ProxyIdURI);

    ProxyIdURI := TIdURI.Create(Value);
    ProxyIdURIProtocol := ProxyIdURI.Protocol;
    ProxyIdURIHost := ProxyIdURI.Host;
    ProxyIdURIPort := ProxyIdURI.Port;
    ProxyIdURIPath := ProxyIdURI.Path;
    ProxyIdURIDocument := ProxyIdURI.Document;
    if ((ProxyIdURIPath = '/')
      and (ProxyIdURIDocument = ''))
        then ProxyIdURIPath := '';

    ProxyHostLength := length(ProxyIdURIProtocol) + 3 + length(ProxyIdURIHost);
    if length(ProxyIdURIPort) > 0 then
      ProxyHostLength := ProxyHostLength + 1 + length(ProxyIdURIPort);
    if length(ProxyIdURIPath) > 0 then
      ProxyHostLength := ProxyHostLength + length(ProxyIdURIPath);
    if length(ProxyIdURIDocument) > 0 then
      ProxyHostLength := ProxyHostLength + length(ProxyIdURIDocument);

    ProxyIdURIHostToCompare := copy(ProxyIdURI.GetFullURI([ofBookmark]),1,ProxyHostLength);  // This is normally the same as U_VlerDasURL but can be different in future scenarios.
    U_VlerDasURL := ProxyIdURIHostToCompare;
  end;
end;

class function TIdHTTPVler.ChangeVlerHostInURL(VlerURL: string): string;
// LMS - Optimized this function for efficiency when The URL passed is has the same host as the current value of VlerDasURL.
var
  VLERIdURI : TIdURI;
begin
  if copy(VlerURL,1,ProxyHostLength) = ProxyIdURIHostToCompare then
    Result := VlerURL         // shortcut when the host is same.
  else begin
    VLERIdURI := TIdURI.Create(VlerURL);
    try
      VLERIdURI.Protocol := ProxyIdURIProtocol;
      VLERIdURI.Host := ProxyIdURIHost;
      VLERIdURI.Port := ProxyIdURIPort;
      VLERIdURI.Path := ProxyIdURIPath + ProxyIdURIDocument + VLERIdURI.Path;
      //VLERIdURI.Document := ProxyIdURIDocument;  // we don't touch the document or bookmarks or parameters if specified in the URL.
      //Result := VLERIdURI.GetFullURI([ofAuthInfo, ofBookmark]); Auth info not used in VLER
      Result := VLERIdURI.GetFullURI([ofBookmark]);
    finally
      VLERIdURI.Free;
    end;
  end;
end;

class function TIdHTTPVler.URLParamsEncode(const ASrc: string): string;
var
  aURI : TIdURI;
begin
    aURI := TIdURI.Create(ASrc);
    try
      //Path := PathEncode(Path);          // VLER Has issues with having all of the URL encoded.
      //Document := PathEncode(Document);
      aURI.Params := aURI.ParamsEncode(aURI.Params);
      Result := aURI.URI;
    finally
      aURI.Free;
    end;
end;

procedure TIdHTTPVler.EnsureAuthTokenReady;
begin
  if (GetVlerDasTokenInt() = '') or TokenExpired() then begin
    VlerDasToken := GetRemoteAuthenticationToken(FBroker);
  end;
end;

function TIdHTTPVler.TokenExpired: boolean;
begin
  Result := (abs(MinutesBetween(now, U_VlerDasTokenTimeStamp)) > TokenMaxLifeInMinutes)  // Over 10 minutes
end;

initialization
  // VlerDasURL is not assigned yet.. so all optimization moved to the set method.

finalization
  if assigned(ProxyIdURI) then
    FreeAndNil(ProxyIdURI);

end.
