unit VocRehabAssnConsults;
//------------------------------------------------------------------------------
// CodeCR347 - JRL June, 2012 - Added unit/form
// Vocational Rehab Form 28-8861
//             
// This form allows the assignment of consults by the Vocational Rehab 
// VHA Coordinator to Vocation Rehab.  This form can only be accessed if the 
// user has a security key of DVBA CAPRI VHA_COORDINATOR.  This form is called 
// from the Request for Medical Services form (VocRehabMedicalRequest.pas).
//------------------------------------------------------------------------------

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, IniFiles, VocRehabClass, ComCtrls,
  dialogsCAPRI, CAPRISupport, Diaccess, Fmcmpnts;

const
  NO_CONSULTS = 'No consults found.';
type
  TfrmVRAssignConsults = class(TForm)
    Panel2: TPanel;
    lblSelectConsult: TLabel;
    lbAvailableConsults: TListBox;
    btnSubmit: TBitBtn;
    btnCancel: TBitBtn;
    FMFilerVocRehab: TFMFiler;
    procedure btnCancelClick(Sender: TObject);
    procedure btnSubmitClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure lbAvailableConsultsClick(Sender: TObject);
  private
    { Private declarations }
    procedure PopulateConsultsList;
    procedure ConsultsRPC;
  public
    { Public declarations }
  end;

var
  frmVRAssignConsults: TfrmVRAssignConsults;

implementation

uses Main, VocRehabMedicalRequest, Trpcb, CCOWRPCBROKERCAPRI;

{$R *.dfm}


//------------------------------------------------------------------------------
// Form Show
//------------------------------------------------------------------------------
procedure TfrmVRAssignConsults.FormShow(Sender: TObject);
begin
  PopulateConsultsList;
end;

//------------------------------------------------------------------------------
// Consult listbox clicked
//------------------------------------------------------------------------------
procedure TfrmVRAssignConsults.lbAvailableConsultsClick(Sender: TObject);
begin
  if lbAvailableConsults.Items[lbAvailableConsults.ItemIndex] <> NO_CONSULTS then
     btnSubmit.Enabled := TRUE;
end;

//------------------------------------------------------------------------------
// Button Submit Click
//------------------------------------------------------------------------------
// Update: 
//    Consult Pointer, File #396.914, Field #.01
//    Request Status, File #396.9, Field # 13, "P" for Pending
//------------------------------------------------------------------------------
procedure TfrmVRAssignConsults.btnSubmitClick(Sender: TObject); 
var
  ConsultIEN : string;
begin
  if (lbAvailableConsults.ItemIndex = -1) OR 
     (lbAvailableConsults.Items[lbAvailableConsults.ItemIndex] = NO_CONSULTS) then
     Exit;                                                       
  ConsultIEN := IntToStr(Integer(lbAvailableConsults.Items.Objects[lbAvailableConsults.ItemIndex])); 
  FMFilerVocRehab.Clear;
  FMFilerVocRehab.AddFDA('396.914','+1,'+frmVRMedicalRequest.SelectedMedicalRequestIEN,'.01',ConsultIEN);
  If Not FMFilerVocRehab.Update Then
  Begin
    FMFilerVocRehab.DisplayErrors;
    exit;
  End
  else  // if consult updated correctly, change status to PENDING
  begin
     FMFilerVocRehab.Clear;
     FMFilerVocRehab.AddFDA('396.9',frmVRMedicalRequest.SelectedMedicalRequestIEN,'13','P');  
     If Not FMFilerVocRehab.Update Then
     Begin
       FMFilerVocRehab.DisplayErrors;
       exit;
     End
     else  // if status updated correctly, send MAILMAN message of pending
     begin
       frmMain.VocRehab.SendMailManMessage('PENDING');     
     end;
  end;
  frmVRMedicalRequest.PopulateConsultSelectedField;  // update Consult field
  frmVRMedicalRequest.SetEditsToReadOnly(TRUE); // set other fields to readonly
  Close;
end;

//------------------------------------------------------------------------------
// Button Cancel Click
//------------------------------------------------------------------------------
procedure TfrmVRAssignConsults.btnCancelClick(Sender: TObject);
begin
  Close;
end;

//------------------------------------------------------------------------------
// Consults RPC
//------------------------------------------------------------------------------
procedure TfrmVRAssignConsults.ConsultsRPC;
var
  x : Integer;
  buffer, status : string;
  ConsultList : TStringList;
begin
    lbAvailableConsults.Items.Clear;
    frmMain.RPCBroker1.Results.Clear;
    ConsultList := TStringList.Create;
    try
      frmMain.RPCBroker1.RemoteProcedure := 'ORQQCN LIST';
      frmMain.RPCBroker1.Param[1].Value := PatientIEN; // Patient IEN
      frmMain.RPCBroker1.Param[1].PType := literal;
      DocType := 'CN';
      Screen.Cursor := crHourGlass;
      Try
        frmMain.RPCBroker1.Call;
      Except
        On EBrokerError Do
        Begin
          ANURemoteProcedureCallInProgress := False;
          Screen.Cursor := crDefault;
          Application.Processmessages;
          ShowMessageCAPRI('RPC ORQQCN LIST could not be accessed!');
        End
      End;
      If frmMain.RPCBroker1.Results.Count > 0 Then
      begin
        For x := 0 To frmMain.RPCBroker1.Results.Count - 1 Do
          ConsultList.Add(frmMain.RPCBroker1.Results[x]);
      end
      else
      begin
        lbAvailableConsults.Items.Add(NO_CONSULTS);
        btnSubmit.Enabled := FALSE;
      end;
      if (Pos('PATIENT DOES NOT HAVE ANY', ConsultList[0]) > 0) Then
      begin
        lbAvailableConsults.Items.Add('No consults found.');
        btnSubmit.Enabled := FALSE;
      end                            
      else
      begin
        for x := 0 To ConsultList.Count - 1 Do
        begin
           status := Piece(ConsultList[x], '^',3);
           if (status = 'p') OR (status = 's') then  // only display consults showing Pending or Scheduled status
           begin
             buffer := FMDateTimeConvert(Piece(ConsultList[x], '^', 2)) + ' (' + status + ') ' + Piece(ConsultList[x], '^', 4);
             lbAvailableConsults.AddItem(buffer, TObject(StrToInt(Piece(ConsultList[x],'^',1))));
           end;
        end;
      end;
    finally
      FreeAndNil(ConsultList);
      Screen.Cursor := crDefault;
    end;
end;

//------------------------------------------------------------------------------
// Populate Consults List
//------------------------------------------------------------------------------
procedure TfrmVRAssignConsults.PopulateConsultsList;                                     
begin    
  // The dialog will retrieve the consult list when it opens using RPCs that CAPRI 
  // currently uses to populate the Consults tab on the main form Clinical Documents tab.
  lbAvailableConsults.Items.Clear;
  ConsultsRPC;                                                   
end;


end.
