unit VocRehabCancelExam;
//------------------------------------------------------------------------------
// CodeCR347 - JRL June, 2012 - Added unit/form
// Vocational Rehab Form 28-8861
//
// This form allows a user to cancel a request for medical services.  
// The request can only be cancelled by the user if a Consult has not been
// assigned to the Medical Request. 
//------------------------------------------------------------------------------

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, ExtCtrls, VA508AccessibilityManager,
  Diaccess, DiTypLib, Fmcmpnts, Fmcntrls, TRPCB, CCOWRPCBrokerCAPRI,
  DialogsCAPRI;                         

const
    REASON_OTHER = 'OTHER';  
type
  TfrmVRCancelExam = class(TForm)
    Panel1: TPanel;
    lblSelectReason: TLabel;
    lblCancelExplain: TLabel;
    edtMedicalServiceToBeCancelled: TEdit;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    edtCancelExplanation: TEdit;
    FMFilerVocRehab: TFMFiler;
    Label1: TLabel;
    lbCancelReason: TListBox;
    procedure btnCancelClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure lbCancelReasonClick(Sender: TObject);
  private
    { Private declarations }
    procedure PopulateCancelledList;
  public
    { Public declarations }
    InternalValue : TStringList;
  end;

var
  frmVRCancelExam: TfrmVRCancelExam;

implementation
uses
   CAPRISupport, main, VocRehabMedicalRequest, VAUtils;

{$R *.dfm}

//------------------------------------------------------------------------------
// Form Activate
//------------------------------------------------------------------------------
procedure TfrmVRCancelExam.FormActivate(Sender: TObject);                       
begin
  PopulateCancelledList;
  edtMedicalServiceToBeCancelled.Text := 
     frmVRMedicalRequest.cbMedicalServicesRequested.Items[frmVRMedicalRequest.cbMedicalServicesRequested.ItemIndex];
  if VAUtils.ScreenReaderActive then
     edtMedicalServiceToBeCancelled.TabStop := TRUE
  else
     edtMedicalServiceToBeCancelled.TabStop := FALSE;
end;  


//------------------------------------------------------------------------------
// Form Close
//------------------------------------------------------------------------------
procedure TfrmVRCancelExam.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   FreeAndNil(InternalValue);
end;


//------------------------------------------------------------------------------
// Button - Cancel Clicked
//------------------------------------------------------------------------------
procedure TfrmVRCancelExam.btnCancelClick(Sender: TObject);
begin
   Close;
end;

//------------------------------------------------------------------------------
// Button - OK Clicked
//------------------------------------------------------------------------------
// Cancel Exam Request
//   File:  396.9
//      Field #15, Cancelation Date
//      Field #16, Cancelation Reason from list
//      Field #17, Cancelation explanation
//------------------------------------------------------------------------------
procedure TfrmVRCancelExam.btnOKClick(Sender: TObject); 
begin
  if lbCancelReason.ItemIndex = -1 then
  begin
     ShowMessageCAPRI('Please select a cancellation reason!');
     exit;
  end;
  if lbCancelReason.Items[lbCancelReason.ItemIndex] = REASON_OTHER then
  begin
     if length(edtCancelExplanation.Text) < 3 then  // some reason more than two characters should be given
     begin
        ShowMessageCAPRI('Please enter a cancellation explanation!');
        exit;
     end;
  end;
  
  If Application.MessageBox('Are you sure you want to CANCEL this medical services request?', 'Cancel Request', 4) = 6 Then
  begin
    FMFilerVocRehab.Clear;
    FMFilerVocRehab.AddFDA('396.9',frmVRMedicalRequest.SelectedMedicalRequestIEN,'13','X');  // Request Status
    FMFilerVocRehab.AddFDA('396.9',frmVRMedicalRequest.SelectedMedicalRequestIEN,'15',FMToDateConvert(FormatDateTime('mm/dd/yyyy',now)));  // cancellation date
    FMFilerVocRehab.AddFDA('396.9',frmVRMedicalRequest.SelectedMedicalRequestIEN,'16',InternalValue[lbCancelReason.ItemIndex]);  // cancellation reason
    if lbCancelReason.Items[lbCancelReason.ItemIndex] = REASON_OTHER then
       FMFilerVocRehab.AddFDA('396.9',frmVRMedicalRequest.SelectedMedicalRequestIEN,'17',edtCancelExplanation.Text);  // cancellation explanation
    If Not FMFilerVocRehab.Update Then
    Begin
      FMFilerVocRehab.DisplayErrors;
      exit;
    End
    else
    begin
     frmMain.VocRehab.SendMailManMessage('CANCELLED');
     frmVRMedicalRequest.edtRequestStatus.Text := 'Request Status: CANCELLED';  // Patch197 JRL 7/17/17 Status is updated in RPC call but not reflected on screen yet
     frmVRMedicalRequest.btnRerouteRequest.Visible := FALSE;                    // Patch197 JRL - Update button status 
    end;
  end;
  Close;
end;

//------------------------------------------------------------------------------
// Populate Cancelled List
//------------------------------------------------------------------------------
// Cancelation Reason - Set of Codes
//   File:  396.9
//      Field #16, Cancelation Reason
//------------------------------------------------------------------------------
procedure TfrmVRCancelExam.PopulateCancelledList;
var
  i : Integer;
begin
  InternalValue := TStringList.Create;
  if CallRPC(frmMain.RPCBroker1, 'DVBAB GET SET', ['396.9', '16'],nil, False, False, False) then
    if (frmMain.RPCBroker1.Results.Count > 0) then
    begin
      lbCancelReason.Items.Clear;
      for i := 0 to frmMain.RPCBroker1.Results.Count - 1 do
      begin
        lbCancelReason.Items.Add(Piece(frmMain.RPCBroker1.Results[i],'^',2));
        InternalValue.Add(Piece(frmMain.RPCBroker1.Results[i],'^',1));
      end;
    end;
end;

//------------------------------------------------------------------------------
// Select Reason listbox enter - handle "OTHER" using VA508 friendly methods
//------------------------------------------------------------------------------
procedure TfrmVRCancelExam.lbCancelReasonClick(Sender: TObject);
begin
  if Uppercase(lbCancelReason.Items[lbCancelReason.ItemIndex]) = REASON_OTHER then
  begin
    lblCancelExplain.Font.Color := clBlack;
    edtCancelExplanation.Color := clWhite;
    edtCancelExplanation.ReadOnly := FALSE;
    edtCancelExplanation.SetFocus;
  end
  else
  begin
    lblCancelExplain.Font.Color := clGray;
    edtCancelExplanation.Color := clBtnFace;
    edtCancelExplanation.ReadOnly := TRUE;
    edtCancelExplanation.Clear;            
  end;
end;
end.
