unit VocRehabMedicalRequest;
//------------------------------------------------------------------------------
// CodeCR347 - JRL June, 2012 - Added unit/form
// Vocational Rehab Form 28-8861
// 
// This form enables the input and/or editing of the new electronic 
// 28-8861 Vocational Rehab form. 
//
// Updated for Patch 197 to include Re-Routing ability
//------------------------------------------------------------------------------
// RPCs called in this form:
//   XWB DIRECT RPC
//   DVBA VR REROUTE STATUS
//   DVBA VR REROUTE INFO
//   DVBA VR GET REROUTE
//------------------------------------------------------------------------------

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, ExtCtrls, ActnList, XPStyleActnCtrls,
  ActnMan, ToolWin, ActnCtrls, ActnMenus, Inifiles, DateUtils, CommCtrl,
  CCOWRPCBrokerCAPRI, Fmcntrls, DialogsCAPRI, Diaccess, Fmcmpnts, DiTypLib,
  Trpcb, ORCtrls, VA508AccessibilityManager, ComCtrls95
  ,VA508AccessibilityRouter
  ;

const
  REASON_OTHER = 'OTHER';
  NO_CONSULTS_FOUND = 'No Consults Found';
  NO_ITEMS_FOUND = 'No Items Found';
  CLOSE_FORM = '1';      // Patch 197 JRL 12/21/16
  DONT_CLOSE_FORM = '2'; // Patch 197 JRL 12/21/16
  TAB_REROUTE_ACCEPT_RETURN_MODE = '1';  // Patch 197 JRL 12/23/16
  TAB_REROUTE_DISPLAY_INFO_MODE = '2';   // Patch 197 JRL 12/23/16 
  
type
  TfrmVRMedicalRequest = class(TForm)
    btnSendRequest: TButton;
    btnCancelRequest: TButton;
    btnCloseWindow: TButton;
    FMGetsVRMedServices: TFMGets;
    FMGetsVR: TFMGets;
    FMFilerVR: TFMFiler;
    FMListerVocRehab: TFMLister;
    FMListerVRMedSvcReq: TFMLister;
    FMGetsRegionalOffice: TFMGets;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    pageVocRehab: TPage95Control;
    tsMedicalRequest: TTab95Sheet;
    tsRerouteInfo: TTab95Sheet;
    pnlBackContainer: TPanel;
    pnlReRouteInfo: TPanel;
    btnRerouteRequest: TButton;
    pnlHeader: TPanel;
    edtVAClaimNumber: TEdit;
    edtPatientName: TEdit;
    edtDateSubmitted: TEdit;
    pnlButtons: TPanel;
    btnAcceptReRoute: TButton;
    btnReturnReRoute: TButton;
    edtReRouteReason: TEdit;
    memoReRouteDescription: TMemo;
    memoReturnReason: TMemo;
    edtReRoutedStatusDate: TEdit;
    edtReRoutedStatus: TEdit;
    edtReRoutedTo: TEdit;
    edtReRoutedDate: TEdit;
    edtReRoutedFrom: TEdit;
    edtRequestStatus: TEdit;
    pnlVR2: TPanel;
    pnlVR1: TPanel;
    pnlVR3: TPanel;
    pnlRehabDate: TPanel;
    btnCalAnticipatedDate: TSpeedButton;
    leAnticipatedDate: TLabeledEdit;
    pnlRouting: TPanel;
    lblRegionalOffice: TLabel;
    FMRegionalOfficeNumberIEN: TFMEdit;
    FMRegionalOfficeNumber: TFMComboBox;
    edtRegionalOfficeReadOnly: TEdit;
    btnPOC: TBitBtn;
    pnlMedicalServices: TPanel;
    lblMedicalServicesRequested: TLabel;
    lblServicesOther: TLabel;
    lblRehabObjectives: TLabel;
    lblReasonForReferral: TLabel;
    edtServicesOther: TEdit;
    reRehabObjectives: TRichEdit;
    edtReasonForReferral: TEdit;
    cbMedicalServicesRequested: TComboBox;
    edtMedicalServicesRequestedReadOnly: TEdit;
    pnlComments: TPanel;
    lblCommentsNotes: TLabel;
    FMCommentsNotes: TFMMemo;
    pnlPreferredDate: TPanel;
    lblPreferredScheduleDateReason: TLabel;
    btnCalPreferredDate: TSpeedButton;
    edtPreferredScheduleDateReason: TEdit;
    lePreferredDate: TLabeledEdit;
    pnlConsultsSelected: TPanel;
    lblConsultsSelected: TLabel;
    btnAssignConsults: TButton;
    edtSelectedConsult: TEdit;
    edtRequestNumber: TEdit;
    lblReRoutedDate: TLabel;
    lblReroutedStatus: TLabel;
    lblReRoutedStatusDate: TLabel;
    lblReRoutedTo: TLabel;
    lblReRoutedFrom: TLabel;
    lblRerouteReason: TLabel;
    lblRerouteDescription: TLabel;
    lblReturnReason: TLabel;
    edtRerouteDetails: TEdit;
    procedure btnSendRequestClick(Sender: TObject);
    procedure btnAssignConsultsClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure dtPreferredScheduleDateChange(Sender: TObject);
    procedure cbMedicalServicesRequestedChange(Sender: TObject);
    procedure dtRehabilitationChange(Sender: TObject);
    procedure btnCancelRequestClick(Sender: TObject);
    procedure FMRegionalOfficeNumberEnter(Sender: TObject);
    procedure btnPOCClick(Sender: TObject);
    procedure btnCloseWindowClick(Sender: TObject);
    procedure FMCommentsNotesChange(Sender: TObject);
    procedure btnCalAnticipatedDateClick(Sender: TObject);
    procedure btnCalPreferredDateClick(Sender: TObject);
    procedure leAnticipatedDateExit(Sender: TObject);
    procedure lePreferredDateExit(Sender: TObject);
    procedure btnRerouteRequestClick(Sender: TObject);
    procedure btnAcceptReRouteClick(Sender: TObject);
    procedure btnReturnReRouteClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tsRerouteInfoEnter(Sender: TObject);
    procedure pageVocRehabChange(Sender: TObject);
    procedure edtRerouteDetailsEnter(Sender: TObject);
  private
    { Private declarations }
    OneRegionalOffice : Boolean;
    ShowAcceptReturnReRouteButtons : Boolean;                                   // Patch 197 JRL 01/04/17
    Patch197 : Boolean;                                                         // Patch 197 JRL 4/13/17
    procedure PopulateFieldValues;
    function  ShowCancelRequestButton : Boolean;
    function  AreRequiredFieldsComplete : Boolean;
    procedure GetRegionalOfficeValues;
    procedure GetMedicalServicesRequestedValues;
    function  EnableAssignConsultsButton : Boolean;
    procedure SendRecord(RecordIEN : string; CommentsOnly : Boolean);
    function  ConvertRichEditToString(reInput : TRichEdit) : string;  // works for small richedit fields
    procedure DateComplete(var StrLikeDate : string; MinDate : TDate; const NowIfError : Boolean = TRUE);
    function  CanRequestBeRerouted(RequestIEN: String): Boolean;  // 12/23/16 JRL Patch 197
    function  SendAcceptReRouteInfo: Boolean;  // Patch 197 JRL 12/23/16 
    function  UpdateReRoutedVistADisplayFields : Boolean; // Patch 197 JRL 12/23/16
    procedure UpdateCommentsReRouteTabs(Tab_ReRoute: String); // Patch 197 JRL 12/23/16
    procedure UpdateControlsForEditingOrReadOnly(Enabled : Boolean);   // Patch 197 JRL 12/23/16
    
    
  public
    { Public declarations }
    CurrentStationNumber : String;  // Patch 197
    ReturnValue : String;           // Patch 197
    SelectedMedicalRequestIEN : string; // Populated on main.dfm when Medical Services Request is selected
    procedure SetupNewExam;
    procedure SetupExistingExam;
    procedure ClearAndInitiateFields;
    procedure PopulateConsultSelectedField;
    procedure SetEditsToReadOnly(Value : Boolean);
  end;

var
  frmVRMedicalRequest: TfrmVRMedicalRequest;

implementation

uses Main 
     ,VocRehabAssnConsults
     ,VocRehabCancelExam 
     ,rofinder
     ,essoselect
     ,CAPRISupport
     ,VocRehabPOC
     ,VAUtils
     ,VocRehabSelectDate
     ,VocRehabRerouteRequest  // 12/19/16 JRL Patch 197
     ,VocRehabReturnRequest   // 12/19/16 JRL Patch 197
     ;

{$R *.dfm}

//------------------------------------------------------------------------------
// Form Activate
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.FormActivate(Sender: TObject);   
var
  ReRoutedRequest : Boolean;  // Patch197 JRL 04/13/17
  ReRouteAllowed : Boolean;   // Patch197 JRL      
begin


   // editing a request, set edits to editable/read-only based on medical request status
   // and user role
   if frmMain.VocRehab.AddExam = FALSE then  // editing request
   begin
    // if the request is completed or cancelled, don't allow editing. If the 
    // consult is already assigned, don't allow editing.  
     if (frmMain.VocRehab.RequestCancelled) OR
        (frmMain.VocRehab.RequestCompleted) OR 
        (btnCancelRequest.Visible = FALSE) then
        SetEditsToReadOnly(TRUE)
     else //edit request, check user role
     begin 
        // VocRehab Counselor role is not allowed to edit a request, 
        // VocRehab Coordinator is allowed to edit a request (VHAAccess=TRUE)
        if Patch197 then // Patch197 JRL 1/5/17
        begin
          if frmMain.VocRehab.VHAAccess then
             SetEditsToReadOnly(FALSE) // allow only VHA Coordinators to edit requests
          else
             SetEditsToReadOnly(TRUE)  // VHA Counselor, block editing request        
        end
        else  // Patch 193 and earlier
        begin
          SetEditsToReadOnly(FALSE);
        end;
     end // else
   end
   else // add new request
   begin
      SetEditsToReadOnly(FALSE);
   end;
   
  // If Patch 197 is installed, then determine whether or not the Re-Route
  // Request button should be enabled or disabled.  If Patch 197 is not 
  // installed, button will be hidden.  -- Patch197 JRL 1/5/17
  btnRerouteRequest.Visible := FALSE; // hide button as default for prior patches 
  if Patch197 then       
  begin
    ReRouteAllowed := FALSE;
    if CanRequestBeRerouted(SelectedMedicalRequestIEN) then                    
    begin                                                                      
      // Request status alone allows the request to be re-routed.  Now we need
      // to check the role of the user.  If the person is a VocRehab Coordinator
      // (VHAAccess=true), then a request may be re-routed.  The VHA Counselor
      // (default role) cannot re-route a request.  They can only add and edit
      // new requests.
      if frmMain.VocRehab.VHAAccess = TRUE then // VHA Coordinator role
         ReRouteAllowed := TRUE;
    end                                                                        
    else // Request status/history won't allow re-routing
    begin                                                                      
      ReRouteAllowed := FALSE;
    end;
    // Make button visible for Patch 197.
    // Enable or disable button if re-routing is allowed
    btnRerouteRequest.Visible := ReRouteAllowed;
    btnRerouteRequest.Enabled := ReRouteAllowed;
  end;  // Patch197 JRL 1/5/17

  // Complete updating Patch 197 display info  // Patch 197 JRL
  if Patch197 then
  begin
    ReRoutedRequest := UpdateReRoutedVistADisplayFields;  // Display Re-Route Info if available    // Patch 193 JRL 8/4/16
    // if there is no re-route information, don't display the re-route tab
    if ReRoutedRequest = FALSE then
    begin
       pageVocRehab.ActivePage := tsMedicalRequest;
       tsRerouteInfo.TabVisible := FALSE;
    end
    else // there is re-route information, display tab and control editing
    begin
       tsRerouteInfo.TabVisible := TRUE;
       if ShowAcceptReturnReRouteButtons then                                   // Patch 197 JRL 
         UpdateCommentsReRouteTabs(TAB_REROUTE_ACCEPT_RETURN_MODE)
       else
         UpdateCommentsReRouteTabs(TAB_REROUTE_DISPLAY_INFO_MODE);
    end;
  end;  // if Patch197

  if VAUtils.ScreenReaderActive then
  begin
    edtPatientName.TabStop := TRUE;
    edtVAClaimNumber.TabStop := TRUE;
    edtDateSubmitted.TabStop := TRUE;
    edtRequestNumber.TabStop := TRUE;   // Patch197 JRL
    edtRequestStatus.TabStop := TRUE;   // Patch197 JRL
    if Patch197 then                    // Patch197 JRL
    begin
      if tsRerouteInfo.TabVisible then  
      begin                             
         // Fields are disabled but the screen reader won't read disabled fields.
         // Enable if a screen reader is present.
         edtRerouteDetails.TabStop := TRUE;
         edtReRouteReason.Enabled := TRUE;
         memoReRouteDescription.Enabled := TRUE;
         memoReturnReason.Enabled := TRUE;
         edtReRoutedFrom.Enabled := TRUE;
         edtReRoutedTo.Enabled := TRUE;
         edtReRoutedStatusDate.Enabled := TRUE;
         edtReRoutedStatus.Enabled := TRUE;
         edtReRoutedDate.Enabled := TRUE;
      end;
      // choose control for screen reader to start on
      if pageVocRehab.ActivePage = tsMedicalRequest then
         edtPatientName.SetFocus                        
      else                                              
         edtRerouteDetails.SetFocus;
    end
    else // Patch197 = FALSE
    begin
      if (edtPatientName.Visible) then 
         edtPatientName.SetFocus;
    end;
  end
  else // no screen reader, disable tab stops on labels
  begin  
    edtPatientName.TabStop := FALSE;
    edtVAClaimNumber.TabStop := FALSE;
    edtDateSubmitted.TabStop := FALSE;
    edtRequestNumber.TabStop := FALSE;  // Patch197 JRL
    edtRequestStatus.TabStop := FALSE;  // Patch197 JRL
    if Patch197 then
    begin
      if tsRerouteInfo.TabVisible then // Patch197 JRL
      begin
         // fields don't need to be enabled for a screen reader.  Disabled them.
        edtRerouteDetails.TabStop := FALSE; 
        edtReRouteReason.Enabled := FALSE;
        memoReRouteDescription.Enabled := FALSE;
        memoReturnReason.Enabled := FALSE;
        edtReRoutedFrom.Enabled := FALSE;
        edtReRoutedTo.Enabled := FALSE;
        edtReRoutedStatusDate.Enabled := FALSE;
        edtReRoutedStatus.Enabled := FALSE;
        edtReRoutedDate.Enabled := FALSE;
      end;                                        // Patch197 JRL
      // choose control to start on without a screen reader
      if pageVocRehab.ActivePage = tsMedicalRequest then 
         leAnticipatedDate.SetFocus 
      else                          
      begin 
        // if accepting/returnin, choose button else choose re-route reason
        if btnAcceptReRoute.Visible then 
           btnAcceptReRoute.SetFocus
        else                                         
           edtReRouteReason.SetFocus;
      end;
    end
    else // Patch197 = FALSE
    begin
      if leAnticipatedDate.Visible then
         leAnticipatedDate.SetFocus;
    end;
  end;
end;

//------------------------------------------------------------------------------
// Form Create - Patch197 JRL 2/10/17
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.FormCreate(Sender: TObject);
begin
  if IsPatchInstalled('DVBA*2.7*197') then       
     Patch197 := TRUE
  else
     Patch197 := FALSE;

  pageVocRehab.ActivePage := tsMedicalRequest;  // default to Medical Request tab
  if not Patch197 then
     tsRerouteInfo.TabVisible := FALSE;   // Don't show tab unless 197 patch installed
end;

//------------------------------------------------------------------------------
// Button - Point of Contact Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnPOCClick(Sender: TObject);
begin
   // Display Point of Contact information
   try
      frmVRPointOfContact := TfrmVRPointOfContact.Create(Application);
      frmVRPointOfContact.ShowModal;
   finally
     FreeAndNil(frmVRPointOfContact);
   end;
end;

//------------------------------------------------------------------------------
// Button - Assign Consults Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnAssignConsultsClick(Sender: TObject);
begin
  // This button is only visible if the logged in user has security keys for
  // "DVBA CAPRI VHA_VRECoordinator"
   try
      frmVRAssignConsults := TfrmVRAssignConsults.Create(Application);
      frmVRAssignConsults.ShowModal;
   finally
     FreeAndNil(frmVRAssignConsults);
   end;
   if edtSelectedConsult.Text = NO_CONSULTS_FOUND then
      btnAssignConsults.Visible := TRUE
   else
      btnAssignConsults.Visible := FALSE;
end;

//------------------------------------------------------------------------------
// Button - Cancel Request Clicked
//------------------------------------------------------------------------------
// Cancel existing request for medical services.  This button is only visible
// when editing an existing medical request and a consult hasn't been assigned 
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnCancelRequestClick(Sender: TObject);
begin
   try
      frmVRCancelExam := TfrmVRCancelExam.Create(Application);
      frmVRCancelExam.ShowModal;
   finally
     FreeAndNil(frmVRCancelExam);
   end;
   frmMain.VocRehab.RequestCancelled := TRUE;
   SetEditsToReadOnly(TRUE);
   if btnAssignConsults.Visible then
      btnAssignConsults.Enabled := FALSE;
end;

//------------------------------------------------------------------------------
// Button - Close Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnCloseWindowClick(Sender: TObject);
begin
  Close;
end;

//------------------------------------------------------------------------------
// Button - Send Request Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnSendRequestClick(Sender: TObject);
begin
  // Anticipated Date of Rehabilitation cannot be blank
  if AreRequiredFieldsComplete = FALSE then
     Exit;
  if frmMain.VocRehab.AddExam then  // creating a new record
  begin
    SendRecord('+1,',FALSE);  // New record
    frmMain.VocRehab.SendMailManMessage('NEW');  
  end
  else // editing existing exam
  begin
    if (edtSelectedConsult.Text = '') OR (edtSelectedConsult.Text = NO_CONSULTS_FOUND) then // If no consult is assigned, update a full edited record
      SendRecord(SelectedMedicalRequestIEN,FALSE)
    else // Update comments only because consult is assigned
      SendRecord(SelectedMedicalRequestIEN,TRUE); 
  end;
  Close;
end;

//------------------------------------------------------------------------------
// Get Regional Office Information
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.GetRegionalOfficeValues;
var
  foundflag: boolean;
  x : Integer;
begin
  If essoversion = true Then
  Begin
    // Make sure user can be on current site before running a report
    foundflag := false;
    If formESSOSelect.slUserSites.Count > 0 Then  //CodeCR77
      For x := 0 To formESSOSelect.slUserSites.Count - 1 Do
        If uppercase(frmMain.piece(formESSOSelect.slUserSites[x], '^', 2)) = uppercase(frmMain.RPCBroker1.Server) Then
          foundflag := true;
    If foundflag <> true Then
    Begin
      ShowmessageCAPRI('You do not have direct access to this site.  Please use FILE | SWITCH SITES and pick a different site prior to running a report.');
      exit;
    End;
  End;
  If frmROFinder.ORListBoxSites.Items.Count = 0 Then
     frmMain.LoadInstitutions;
  FMRegionalOfficeNumber.Items.Clear;

  if frmMain.VocRehab.AddExam then
  begin
    // Set up division...
    searchforstring := UserDivision;
    FMRegionalOfficeNumberEnter(Application);
    searchforstring := '';
    If FMRegionalOfficeNumber.Text <> '' Then
    begin
      edtRegionalOfficeReadOnly.Text := FMRegionalOfficeNumber.Text;
      FMRegionalOfficeNumber.Enabled := False;
      FMRegionalOfficeNumber.Visible := FALSE;
      edtRegionalOfficeReadOnly.Visible := TRUE;
      if VAUtils.ScreenReaderActive then
        edtRegionalOfficeReadOnly.Enabled := TRUE;
    end;
  end
  else
  begin
    FMGetsRegionalOffice.IENS := SelectedMedicalRequestIEN;
    FMGetsRegionalOffice.GetAndFill;
    If FMRegionalOfficeNumber.Text <> '' Then
    begin
      FMRegionalOfficeNumber.Enabled := False;
      FMRegionalOfficeNumber.Visible := FALSE;
      edtRegionalOfficeReadOnly.Visible := TRUE;
      if VAUtils.ScreenReaderActive then
        edtRegionalOfficeReadOnly.Enabled := TRUE;
      OneRegionalOffice := FALSE;
    end
    else
      OneRegionalOffice := TRUE;
  end;
end;

//------------------------------------------------------------------------------
// Anticipated Date Exit
//------------------------------------------------------------------------------
// Validate date using DateComplete procedure.  If date is blank, allow that
// during the edit phase.  A blank date will not be allowed once the Send
// button is pressed.  
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.leAnticipatedDateExit(Sender: TObject);
var
  s : string;
  SaveDate : string;
  MinDate : TDate;
begin
  s := leAnticipatedDate.Text;
  SaveDate := s;
  if frmMain.VocRehab.AddExam then // if new request, the minimum date is Tomorrow
     MinDate := Tomorrow
  else  // if editing a request, allow a previous date
     MinDate := StrToDate('01/01/1900'); 
  DateComplete(s, MinDate, FALSE);
  leAnticipatedDate.Text := s;
  if copy(leAnticipatedDate.Text,1,4) = 'Min ' then
  begin
    leAnticipatedDate.Text := '';
    ShowMessage('Anticipated Date must be a future date');
    leAnticipatedDate.SetFocus;
  end;               
   
end;

//------------------------------------------------------------------------------
// Preferred Date Exit
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.lePreferredDateExit(Sender: TObject);
var
  s : string;
  SaveDate : string;
  MinDate : TDate;
begin
  if lePreferredDate.ReadOnly = TRUE then  // if it is already read-only, then  // Patch197 JRL 7/18/17
     exit;                                   // don't change anything             
  s := lePreferredDate.Text;
  SaveDate := s;
  if frmMain.VocRehab.AddExam then // if new request, the minimum date is Tomorrow
     MinDate := Tomorrow
  else
     MinDate := StrToDate('01/01/1900');
  DateComplete(s, MinDate, FALSE);
  lePreferredDate.Text := s;
  if copy(lePreferredDate.Text,1,4) = 'Min ' then
  begin
    lePreferredDate.Text := '';
    ShowMessage('Preferred Date must be a future date');
    lePreferredDate.SetFocus;
  end;   
  if (frmMain.VocRehab.RequestCancelled) OR
     (frmMain.VocRehab.RequestCompleted) then
  begin
    edtPreferredScheduleDateReason.ReadOnly := TRUE;
  end
  else
  begin
     if lePreferredDate.Text <> '' then
     begin
       edtPreferredScheduleDateReason.ReadOnly := FALSE;
       lblPreferredScheduleDateReason.Font.Color := clBlack;
       edtPreferredScheduleDateReason.Color := clWhite;
       edtPreferredScheduleDateReason.Font.Color := clBlack;
     end
     else
     begin
       edtPreferredScheduleDateReason.ReadOnly := TRUE;
       lblPreferredScheduleDateReason.Font.Color := clGray;
       edtPreferredScheduleDateReason.Color := clBtnFace;
     end;
  end;
end;

procedure TfrmVRMedicalRequest.pageVocRehabChange(Sender: TObject);
begin
end;

//------------------------------------------------------------------------------
// Comments Change
//------------------------------------------------------------------------------
// If form is in ReadOnly mode, then the SendMedicalRequest button is not 
// enabled.  If changes are made to the comments/nodes field (only field
// editable in this mode, then enable the SendMedicalRequestButton
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.FMCommentsNotesChange(Sender: TObject);
begin
  if leAnticipatedDate.ReadOnly = TRUE then // we are in ReadOnlyMode
  begin
     btnSendRequest.Enabled := TRUE;
     btnSendRequest.Visible := TRUE;
  end;
end;


//------------------------------------------------------------------------------
// Regional Office Control Enter
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.FMRegionalOfficeNumberEnter(Sender: TObject);
begin
  // code stolen from newexam.pas:  TfrmNewExam.FMRegionalOfficeNumberEnter(Sender: TObject);
  // with a few substractions for unnecessary code in this instance
  If frmROFinder.ShowModal = mrOK Then
    If frmROFinder.ORListBoxSites.ItemIndex > -1 Then
    Begin
      FMRegionalOfficeNumber.Visible := TRUE;  
      FMRegionalOfficeNumber.Text := frmMain.Piece(frmROFinder.ORListBoxSites.Items[frmROFinder.ORListBoxSites.ItemIndex], '^', 1);
      FMRegionalOfficeNumberIEN.Text := frmMain.Piece(frmROFinder.ORListBoxSites.Items[frmROFinder.ORListBoxSites.ItemIndex], '^', 2);
    End;
  // note: since this combo box is being used in a non-conventional way, it is
  // necessary for focus to pass to the next control to behave as the users are
  // accustomed
  if (ActiveControl = FMRegionalOfficeNumber) then
    SelectNext(ActiveControl as TWinControl, TRUE, TRUE);
end;

//------------------------------------------------------------------------------
// Get Medical Services Requested Values to populate cbMedicalServicesRequested
//------------------------------------------------------------------------------
// Each cbMedicalServicesRequested entry contains a string value and an object.
// The object contains the integer IEN number for the Medical Service Requested.
// The customer requested that the combobox display the data in the order shown,
// so the data was input into the database in the desired order and the combobox
// is sorted by IEN number preserving the requested order.
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.GetMedicalServicesRequestedValues;

  function SortByIEN(List: TStringList; Index1,Index2: Integer): Integer;
  var
    IEN1, IEN2 : Integer;
  begin
    Result := 0;
    IEN1 := Integer(List.Objects[Index1]);
    IEN2 := Integer(List.Objects[Index2]);
    if IEN1 > IEN2 then
      Result := 1;
    if IEN1 = IEN2 then
      Result := 0;
    if IEN1 < IEN2 then
      Result := -1;
  end; 
  
var
  x : Integer;
  buffer : string;
  slNames : TStringList;
begin     
  cbMedicalServicesRequested.Items.Clear;
  FMListerVRMedSvcReq.Screen := '';
  FMListerVRMedSvcReq.FMIndex := '';
  FMListerVRMedSvcReq.DisplayFields.Clear;
  FMListerVRMedSvcReq.FieldNumbers.Clear;
  FMListerVRMedSvcReq.FileNumber := '396.918';
  FMListerVRMedSvcReq.DisplayFields.Add('.01');
  FMListerVRMedSvcReq.FieldNumbers.Add('.01');
  FMListerVRMedSvcReq.GetList(nil);
  if FMListerVRMedSvcReq.Results.Count < 1 then
  begin
     cbMedicalServicesRequested.Items.Add(NO_ITEMS_FOUND);
     Exit;
  end;
  // Get and parse the returned records  
  for x := 0 to FMListerVRMedSvcReq.Results.Count - 1 do
  begin
    buffer := FMListerVRMedSvcReq.GetRecord(FMListerVRMedSvcReq.Results[x]).GetField('.01').FMDBExternal;
    cbMedicalServicesRequested.AddItem(buffer,TObject(StrToInt(FMListerVRMedSvcReq.Results[x]))); // add item stores the record IEN in the obejct
  end;
  // sort combobox on IEN object using a customsort.  Combobox doesn't have
  // a customsort method so assign the combobox items to a TStringlist.
  slNames := TStringList.Create;
  cbMedicalServicesRequested.Items.BeginUpdate;
  cbMedicalServicesRequested.Sorted := FALSE;
  try
    slNames.Assign(cbMedicalServicesRequested.Items);
    slNames.CustomSort(@SortByIEN);
    cbMedicalServicesRequested.Items.Assign(slNames);
  finally
    FreeAndNil(slNames);
    cbMedicalServicesRequested.Items.EndUpdate;
  end;
end;


//------------------------------------------------------------------------------
// Send Record
//------------------------------------------------------------------------------
// There are 3 different record conditions to be created/updated:
//   1:  New record,  IEN = '+1,', CommentsOnly := FALSE
//       This is a new record 
//   2:  Update record, IEN = SelectedIEN, CommentsOnly := FALSE; 
//       This is an existing record updated before a consult is assigned
//   3:  Update record, IEN = SelectedIEN, CommentsOnly := TRUE;
//       Once a consult has been assigned, the only field that can be 
//       updated is the Comments/Notes field
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.SendRecord(RecordIEN : string; CommentsOnly : Boolean);
var
  FMDateToday : string;
  buffer : string;
begin
    frmMain.FMValidator1.Clear;
    FMDateToday := FMToDateConvert(FormatDateTime('mm/dd/yyyy',now));
    buffer := FormatDateTime('hhnn',now);
    FMDateToday := FMDateToday + '.' + buffer;
    FMFilerVR.Clear;
    if CommentsOnly then // only updating comments, all other fields are read-only at this point
    begin
      FMCommentsNotes.IENS := RecordIEN;
      FMFilerVR.AddChgdControl(FMCommentsNotes); // Comments / Notes
    end
    else // updating all fields
    begin
      if RecordIEN = '+1,' then  // Only include date/time on a new record
      begin
        frmMain.FMValidator1.SetUp('396.9',RecordIEN,'.01',FmDateToday);
        If frmMain.FMValidator1.Validate then
          FMFilerVR.AddFDA('396.9',RecordIEN,'.01',frmMain.FMValidator1.Results[0])  // New record IEN - include request date/time
        else
          frmMain.FMValidator1.DisplayErrors;
      end;
      FMFilerVR.AddFDA('396.9',RecordIEN,'3',FMRegionalOfficeNumberIEN.Text);  // Regional Office
      FMFilerVR.AddFDA('396.9',RecordIEN,'4',PatientIEN);  // Patient
      FMFilerVR.AddFDA('396.9',RecordIEN,'5',ConvertRichEditToString(reRehabObjectives));  // Rehab Objective of Veteran
      FMFilerVR.AddFDA('396.9',RecordIEN,'6',edtReasonForReferral.Text);  // Reason for Referral
      FMFilerVR.AddFDA('396.9',RecordIEN,'7',FMToDateConvert(leAnticipatedDate.Text));  // Anticipated Date of Rehabilitation
      FMFilerVR.AddFDA('396.9',RecordIEN,'8',IntToStr(Integer(cbMedicalServicesRequested.Items.Objects[cbMedicalServicesRequested.ItemIndex])));  // Medical Services Requested
      FMFilerVR.AddFDA('396.9',RecordIEN,'8.1',edtServicesOther.Text);  // Other Reason
      if lePreferredDate.Text <> '' then // date exists       
      begin
        FMFilerVR.AddFDA('396.9',RecordIEN,'9',FMToDateConvert(lePreferredDate.Text));  // Preferred Schedule Date
        FMFilerVR.AddFDA('396.9',RecordIEN,'10',edtPreferredScheduleDateReason.Text);  // Preferred Schedule Date Reason
      end;
      FMFilerVR.AddFDA('396.9',RecordIEN,'11',frmMain.VocRehab.GetPointOfContactIEN(TRUE));  // Point of Contact
      FMCommentsNotes.IENS := RecordIEN;
      FMFilerVR.AddChgdControl(FMCommentsNotes); // Comments / Notes
      FMFilerVR.AddFDA('396.9',RecordIEN,'13','N');  // Request Status
    end;
    If Not FMFilerVR.Update Then
    Begin
      FMFilerVR.DisplayErrors;
      exit;
    End
    else
    begin
      // if adding a new record, we need to populate SelectMedicalRequestIEN 
      // with the recently recreated IEN prior to sending the Mail message
      if RecordIEN = '+1,' then 
         SelectedMedicalRequestIEN := FMFilerVR.FindIen('+1,');  // assign new IEN to SelectedMedicalRequestIEN
    end;
end;

//------------------------------------------------------------------------------
// Convert RichEdit Text field to a single plain text string
//------------------------------------------------------------------------------
function TfrmVRMedicalRequest.ConvertRichEditToString(reInput: TRichEdit): string;
var
  i : Integer;
  buffer : string;
begin
  buffer := '';
  for i := 0 to reInput.Lines.Count - 1 do
     buffer := buffer + reInput.Lines[i] + ' ';
  Result := buffer;    
end;

//------------------------------------------------------------------------------
// Set Edits To Read Only
//------------------------------------------------------------------------------
// If a consult has been assigned, the only editable control will be the
// Comments/Notes field.  Pressing Submit now only updates that one field.
// The POC button is enabled for viewing in any mode.
//
// The Assign Consult button is only available to those who have the
// VHA_ACCESS security key.  Even if a consult has been assigned, if the user
// has that key, the Assign Consult button will be enabled.
//
// The main purpose of the read only edit fields is to comply with VA508
// compatibility.  Read only controls are better than invisible/disabled
// controls.
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.SetEditsToReadOnly(Value : Boolean);
begin

  If Value then  // Controls should be read only
  begin
    leAnticipatedDate.ReadOnly := TRUE;
    leAnticipatedDate.Color := clBtnFace;
    btnCalAnticipatedDate.Visible := FALSE;
    // Regional Office
    lblRegionalOffice.FocusControl := edtRegionalOfficeReadOnly;
    edtRegionalOfficeReadOnly.Top := FMRegionalOfficeNumber.Top;
    FMRegionalOfficeNumber.Visible := FALSE;
    edtRegionalOfficeReadOnly.Visible := TRUE;
    if VAUtils.ScreenReaderActive then
      edtRegionalOfficeReadOnly.Enabled := TRUE;
    // Medical Services Requested
    lblMedicalServicesRequested.FocusControl := edtMedicalServicesRequestedReadOnly;
    edtMedicalServicesRequestedReadOnly.Top := cbMedicalServicesRequested.Top;
    cbMedicalServicesRequested.Visible := FALSE;
    edtMedicalServicesRequestedReadOnly.Visible := TRUE;
    // Other Reason
    edtServicesOther.ReadOnly := TRUE;
    edtServicesOther.Color := clBtnFace;
    // Reason for Referral
    edtReasonForReferral.ReadOnly := TRUE;
    edtReasonForReferral.Color := clBtnFace;
    // Rehabilitation Objectives
    reRehabObjectives.ReadOnly := TRUE;
    reRehabObjectives.Color := clBtnFace;
    // Preferred Schedule Date
    lePreferredDate.ReadOnly := TRUE;
    lePreferredDate.Color := clBtnFace;
    btnCalPreferredDate.Visible := FALSE;
    // Preferred Schedule Date Reason
    lblPreferredScheduleDateReason.Font.Color := clBlack;
    edtPreferredScheduleDateReason.Color := clBtnFace;
    edtPreferredScheduleDateReason.ReadOnly := TRUE;
    // Consult Date
    edtSelectedConsult.ReadOnly := TRUE;
    edtSelectedConsult.Color := clBtnFace;
    // Send Medical Services Button
    btnSendRequest.Visible := FALSE;
    // Cancel Medical Services Button
    btnCancelRequest.Visible := FALSE;
    // Re-Route Info
    edtReRouteReason.ReadOnly := TRUE;           // Patch197 JRL 1/5/17
    memoReRouteDescription.ReadOnly := TRUE;     // Patch197 JRL 1/5/17
    memoReturnReason.ReadOnly := TRUE;           // Patch197 JRL 1/5/17
  end
  else  // Controls are editable
  begin
    // Anticipated Date of Rehabilitation
    leAnticipatedDate.ReadOnly := FALSE;
    leAnticipatedDate.Color := clWindow;
    btnCalAnticipatedDate.Visible := TRUE;
    // Regional Office
    lblRegionalOffice.FocusControl := FMRegionalOfficeNumber;
    if OneRegionalOffice then
    begin
      FMRegionalOfficeNumber.Visible := FALSE;
      edtRegionalOfficeReadOnly.Visible := TRUE;
      if VAUtils.ScreenReaderActive then
        edtRegionalOfficeReadOnly.Enabled := TRUE;
    end
    else
    begin
      FMRegionalOfficeNumber.Visible := TRUE;
      edtRegionalOfficeReadOnly.Visible := FALSE;
    end;
    // Medical Services Requested
    lblMedicalServicesRequested.FocusControl := cbMedicalServicesRequested;
    cbMedicalServicesRequested.Visible := TRUE;
    edtMedicalServicesRequestedReadOnly.Visible := FALSE;
    // Reason for Referral
    edtReasonForReferral.ReadOnly := FALSE;
    edtReasonForReferral.Color := clWindow;
    // Rehabilitation Objectives
    reRehabObjectives.ReadOnly := FALSE;
    reRehabObjectives.Color := clWindow;
    // Preferred Schedule Date
    lePreferredDate.ReadOnly := FALSE;
    lePreferredDate.Color := clWindow;
    btnCalPreferredDate.Visible := TRUE;
    if lePreferredDate.Text = '' then // date exists so enable reason for editing
      edtPreferredScheduleDateReason.ReadOnly := TRUE
    else
      edtPreferredScheduleDateReason.ReadOnly := FALSE;
    // Consult Date
    edtSelectedConsult.Color := clWindow;
    // Send Medical Services Button
    btnSendRequest.Visible := TRUE;
    // Cancel Medical Services Button
    btnCancelRequest.Visible := ShowCancelRequestButton;
  end;
  // Comments are enabled while the request is new or pending
  // Comments are disabled when the request is cancelled or completed
  if (frmMain.VocRehab.RequestCancelled) OR
     (frmMain.VocRehab.RequestCompleted) then
  begin
     if VAUtils.ScreenReaderActive then
     begin
       pnlComments.Enabled := TRUE;      // Allow the field to be read by the
       FMCommentsNotes.ReadOnly := TRUE; // screen reader even if disabled
     end
     else
     begin
       pnlComments.Enabled := FALSE;     // don't even tab to that field if the
     end;                                // screen reader is not active
     FMCommentsNotes.Color := clBtnFace;
     btnSendRequest.Visible := FALSE;
  end
  else // Request is not cancelled/complete so enable comments field
  begin
     pnlComments.Enabled := TRUE;
     FMCommentsNotes.ReadOnly := FALSE;
     FMCommentsNotes.Color := clWindow;
     btnSendRequest.Visible := TRUE;
     btnAssignConsults.Enabled := EnableAssignConsultsButton;
  end;
end;

//------------------------------------------------------------------------------
// Setup New Exam
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.SetupNewExam;
begin
  ClearAndInitiateFields;
  frmMain.VocRehab.RequestCancelled := FALSE;
  frmMain.VocRehab.RequestCompleted := FALSE;
  pnlConsultsSelected.Visible := FALSE;
  btnCancelRequest.Visible := ShowCancelRequestButton;
  edtSelectedConsult.ReadOnly := TRUE;
  edtSelectedConsult.Color := clBtnFace;
  lblConsultsSelected.Font.Color := clGray;
  btnSendRequest.Visible := TRUE;
end;

//------------------------------------------------------------------------------
// Setup Existing Exam
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.SetupExistingExam;
begin
  ClearAndInitiateFields;
  pnlConsultsSelected.Visible := TRUE;
  PopulateFieldValues;
  btnCancelRequest.Visible := ShowCancelRequestButton;
  btnAssignConsults.Enabled := EnableAssignConsultsButton;
  btnSendRequest.Caption := 'Update Medical Services Request';
end;

//------------------------------------------------------------------------------
// Populate Field Values
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.PopulateFieldValues;
var
  buffer : string;
begin
  // clear fields before populating - added to allow re-routing to re-populate fields Patch197 JRL
  edtRequestNumber.Text := 'Request #: '; 
  edtRequestNumber.Tag := 0;
  edtDateSubmitted.Text := 'Request Date: ';
  edtRegionalOfficeReadOnly.Text := '';
  FMRegionalOfficeNumberIEN.Text := '';
  FMRegionalOfficeNumber.ItemIndex := -1;
  reRehabObjectives.Lines.Clear;
  edtReasonForReferral.Clear;
  leAnticipatedDate.Clear;
  lePreferredDate.Clear;
  cbMedicalServicesRequested.ItemIndex := -1;
  edtMedicalServicesRequestedReadOnly.Text := '';
  edtRequestStatus.Text := 'Request Status: ';
  edtServicesOther.Text := '';
  edtSelectedConsult.Clear;
  FMCommentsNotes.Lines.Clear;
  // populate fields
  FMGetsVR.FileNumber := '396.9';
  FMGetsVR.FieldNumbers.Add('.01');  // Request Date
  FMGetsVR.FieldNumbers.Add('1');    // Send To Routing Location
  FMGetsVR.FieldNumbers.Add('3');    // Regional Office
  FMGetsVR.FieldNumbers.Add('5');    // Rehabilitation Objectives
  FMGetsVR.FieldNumbers.Add('6');    // Reason for Referral
  FMGetsVR.FieldNumbers.Add('7');    // Anticipated Date of Rehabilitation
  FMGetsVR.FieldNumbers.Add('8');    // Medical Services Requested
  FMGetsVR.FieldNumbers.Add('8.1');  // Other Reason
  FMGetsVR.FieldNumbers.Add('9');    // Preferred Schedule Date
  FMGetsVR.FieldNumbers.Add('10');   // Preferred Schedule Date Reason
  FMGetsVR.FieldNumbers.Add('12');   // Comments / Notes
  FMGetsVR.FieldNumbers.Add('13');   // Requst Status - Patch197 JRL 1/13/17
  FMGetsVR.FieldNumbers.Add('14');   // Consults
  FMGetsVR.IENS := SelectedMedicalRequestIEN;      // Populated on main form / on click event
  FMGetsVR.GetData;
  if FMGetsVR.Results.Count < 1 then
     // Show Error
  else
  begin
     try
       edtRequestNumber.Text := 'Request #: ' + SelectedMedicalRequestIEN;      // Patch197 JRL 4/13/17 
       edtRequestNumber.Tag := StrToInt(SelectedMedicalRequestIEN);                       // Patch197 JRL 7/12/17
       edtDateSubmitted.Text := 'Request Date: ' + FMGetsVR.GetField('.01').FMDBExternal;
       edtRegionalOfficeReadOnly.Text := FMGetsVR.GetField('3').FMDBExternal; // read only display
       FMRegionalOfficeNumberIEN.Text := FMGetsVR.GetField('3').FMDBInternal;
       FMRegionalOfficeNumber.Visible := FALSE;
       reRehabObjectives.Lines.Add(FMGetsVR.GetField('5').FMDBExternal);
       edtReasonForReferral.Text := FMGetsVR.GetField('6').FMDBExternal;
       leAnticipatedDate.Text := ConvertCalendarDateToDelphiDate(FMGetsVR.GetField('7').FMDBExternal);
       leAnticipatedDate.ReadOnly := TRUE;  // read only display
       buffer := FMGetsVR.GetField('8').FMDBExternal;
       cbMedicalServicesRequested.ItemIndex := cbMedicalServicesRequested.Items.IndexOf(buffer);
       edtMedicalServicesRequestedReadOnly.Text := cbMedicalServicesRequested.Items[cbMedicalServicesRequested.ItemIndex];
       frmMain.VocRehab.RequestStatus := FMGetsVR.GetField('13').FMDBExternal;  // Patch197 JRL 1/13/17
       edtRequestStatus.Text := 'Request Status: ' + frmMain.VocRehab.RequestStatus;  // Patch197 JRL 7/17/17               // Patch197 JRL 1/13/17
       edtServicesOther.Text := FMGetsVR.GetField('8.1').FMDBExternal;
       if edtServicesOther.Text <> '' then
       begin
         lblServicesOther.Font.Color := clBlack;
         edtServicesOther.Color := clWhite;
         edtServicesOther.Font.Color := clBlack;
       end
       else
       begin
         lblServicesOther.Font.Color := clGray;
         edtServicesOther.Color := clBtnFace;
         edtServicesOther.Clear;
       end;
       buffer := ConvertCalendarDateToDelphiDate(FMGetsVR.GetField('9').FMDBExternal);
       if (buffer = '') or (buffer = '//') then // if blank date string
          begin
          lePreferredDate.Text := ''; // display blank date
          lblPreferredScheduleDateReason.Font.Color := clGray;
          edtPreferredScheduleDateReason.Color := clBtnFace;
          edtPreferredScheduleDateReason.Clear;
          end
       else // valid date - get reason field too
       begin
          lePreferredDate.Text := buffer;
          edtPreferredScheduleDateReason.Text := FMGetsVR.GetField('10').FMDBExternal;
          if edtPreferredScheduleDateReason.Text <> '' then
          begin
            lblPreferredScheduleDateReason.Font.Color := clBlack;
            edtPreferredScheduleDateReason.Color := clWhite;
            edtPreferredScheduleDateReason.Font.Color := clBlack;
          end
          else
          begin
            lblPreferredScheduleDateReason.Font.Color := clGray;
            edtPreferredScheduleDateReason.Color := clBtnFace;
            edtPreferredScheduleDateReason.Clear;
          end;
       end;
       FMCommentsNotes.Lines.AddStrings(FMGetsVR.GetField('12').FMWPTextLines);
       pnlConsultsSelected.Visible := TRUE;
       PopulateConsultSelectedField;  // Get and populate consult assigned/linked field
     except
       ShowMessageCAPRI('Error populating fields');
     end;
  end;
end;

//------------------------------------------------------------------------------
// Populate Field Values
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.PopulateConsultSelectedField;
var
  aIEN : string;
begin    
     // populate Consult Selected field if available
     edtSelectedConsult.Clear;
     FMListerVocRehab.Screen := '';
     FMListerVocRehab.FMIndex := '';
     FMListerVocRehab.DisplayFields.Clear;
     FMListerVocRehab.FieldNumbers.Clear;
     FMListerVocRehab.IENS := ',' + SelectedMedicalRequestIEN;
     FMListerVocRehab.FileNumber := '396.914';
     FMListerVocRehab.DisplayFields.Add('.01');
     FMListerVocRehab.FieldNumbers.Add('.01');
     FMListerVocRehab.GetList(nil);
     if FMListerVocRehab.Results.Count < 1 then
     begin
       edtSelectedConsult.Text := NO_CONSULTS_FOUND;
       Exit;
     end
     else
     begin  
       // while this field is a multiple field, current restrictions have a 
       // "one consult to one form 8861" so there should only be one entry in this list
       aIEN := FMListerVocRehab.GetRecord(FMListerVocRehab.Results[0]).GetField('.01').FMDBInternal; // get the first entry only
     end;
     FMGetsVR.FileNumber := '123';
     FMGetsVR.FieldNumbers.Add('.01');  // Request Date
     FMGetsVR.FieldNumbers.Add('1');    // Send To Location (Regional Office)
     FMGetsVR.IENS := aIEN;
     FMGetsVR.GetData;
     if FMGetsVR.Results.Count < 1 then
        // no records
     else
     begin
        edtSelectedConsult.Text := FMGetsVR.GetField('.01').FMDBExternal + ' - ' +
                                   FMGetsVR.GetField('1').FMDBExternal;
     end;
end;


//------------------------------------------------------------------------------
// Are Required Fields Complete
//------------------------------------------------------------------------------
// Requirement Fields:  
//   * Anticipated Date of Rehabilitation cannot be blank 
//   * If Reason for Referal is OTHER, then OTHER description must not be blank
//   * Reason for Referal can't be blank
//   * if Preferred Schedule date isn't blank, then Preferred Schedule Date
//     reason can't be blank
//   * if screen reader is active and preferred schedule date or anticipated
//     date of rehabilitation is today, then the field isn't correct
//   * Reason for Referral - required field
//   * Rehabilitation Objectives - required field
//------------------------------------------------------------------------------
function TfrmVRMedicalRequest.AreRequiredFieldsComplete: Boolean;
begin
  Result := TRUE;
  // If the datetime control is not visible, we are in read-only mode.
  // Don't do field validation in read-only mode.
  if leAnticipatedDate.ReadOnly = TRUE then
     exit;

  if frmMain.VocRehab.AddExam then // don't validate date when "editing" as a past date may be valid then
  begin
    if leAnticipatedDate.Text = '' then
    begin
      Result := FALSE;
      ShowMessageCAPRI('Anticipated date of rehabilitation cannot be blank.');
      exit;
    end;
  end;    
  if FMRegionalOfficeNumberIEN.Text = '' then
  begin
     ShowMessageCAPRI('Regional Office must be completed.');
     Result := FALSE;
     exit;
  end;
  if cbMedicalServicesRequested.ItemIndex = -1 then
  begin
     ShowMessageCAPRI('Medical Services Requested field cannot be blank.');
     Result := FALSE;
     exit;
  end;
  if (UpperCase(cbMedicalServicesRequested.Items[cbMedicalServicesRequested.ItemIndex]) = REASON_OTHER) AND
     (Length(edtServicesOther.Text)< 3) then
  begin
     ShowMessageCAPRI('Other Description field must be completed.');
     Result := FALSE;
     exit;
  end;
  if (lePreferredDate.Text <> '') AND 
     (length(edtPreferredScheduleDateReason.Text) < 3) then
  begin
     ShowMessageCAPRI('Setting a Preferred Schedule Date must include a reason.');
     Result := FALSE;
     exit;
  end;
  if Length(edtReasonForReferral.Text) < 3 then
  begin
     ShowMessageCAPRI('Reason for Referral field must be completed.');
     Result := FALSE;
     exit;
  end;
  if Length(reRehabObjectives.Text) < 3 then
  begin
     ShowMessageCAPRI('Rehabilitation Objectives must be completed.');
     Result := FALSE;
     exit;
  end;
end;


//------------------------------------------------------------------------------ 
// VA508 Compatability - don't disable, just gray out and make read only
//------------------------------------------------------------------------------ 
procedure TfrmVRMedicalRequest.cbMedicalServicesRequestedChange(Sender: TObject);
begin
   if UpperCase(cbMedicalServicesRequested.Items[cbMedicalServicesRequested.ItemIndex]) = REASON_OTHER then
   begin
      lblServicesOther.Font.Color := clBlack;
      edtServicesOther.ReadOnly := FALSE;
      edtServicesOther.Clear;
      edtServicesOther.Font.Color := clBlack;
      edtServicesOther.Color := clWhite;
      if frmVRMedicalRequest.Visible then
         edtServicesOther.SetFocus;
   end
   else
   begin
      lblServicesOther.Font.Color := clGray;
      edtServicesOther.Clear;
      edtServicesOther.Color := clBtnFace;
      edtServicesOther.ReadOnly := TRUE;
   end;
end;

//------------------------------------------------------------------------------
// Clear And Initiate Fields
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.ClearAndInitiateFields;
begin
   edtVAClaimNumber.Text := 'VA Claim #: ' + frmMain.VocRehab.VAClaimNumber;
   edtPatientName.Text := 'Patient Name: ' + PatientName;
   edtDateSubmitted.Text := 'Request Date:    ' + DateToStr(now);
   // Regional Office
   GetRegionalOfficeValues;
   // Medical Services Requested
   GetMedicalServicesRequestedValues;
   // Rehabilitation Objectives
   reRehabObjectives.Lines.Clear;
   // Reason For Referral
   edtReasonForReferral.Text := '';
   edtServicesOther.Text := '';
   edtServicesOther.ReadOnly := TRUE;
   edtServicesOther.Font.Color := clGrayText;
   edtServicesOther.Color := clBtnFace;
   // Preferred Schedule Date
   edtPreferredScheduleDateReason.Clear;
   edtPreferredScheduleDateReason.Color := clBtnFace;
   edtPreferredScheduleDateReason.Font.Color := clGrayText;
   edtPreferredScheduleDateReason.ReadOnly := TRUE;
   lblPreferredScheduleDateReason.Font.Color := clGray;
   // Comments or Notes
   FMCommentsNotes.Lines.Clear;
   // Consults (only if edit has been pressed previously)
   edtSelectedConsult.Clear;
   
end;


//------------------------------------------------------------------------------
// Date: Preferred Schedule Date Change
//------------------------------------------------------------------------------
// Property inspector for TDateTimePicker has a format := ' ' to show a blank 
// date upon startup.  Upon change, the format property should be changed to 
// the ShortDateFormat;
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.dtPreferredScheduleDateChange(Sender: TObject);
begin
  (Sender as TDateTimePicker).Format:=ShortDateFormat;  // this is needed to support a "blank" optional date
  // VA508 Compatability - don't disable, just gray out and make read only
  lblPreferredScheduleDateReason.Font.Color := clBlack;
  edtPreferredScheduleDateReason.ReadOnly := FALSE;
  edtPreferredScheduleDateReason.Color := clWhite;
  edtPreferredScheduleDateReason.Font.Color := clBlack;
  if frmVRMedicalRequest.Visible then
     edtPreferredScheduleDateReason.SetFocus;
end;

//------------------------------------------------------------------------------
// Date: Anticipated Date of Rehabilitation Change
//------------------------------------------------------------------------------
// Property inspector for TDateTimePicker has a format := ' ' to show a blank 
// date upon startup.  Upon change, the format property should be changed to 
// the ShortDateFormat;
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.dtRehabilitationChange(Sender: TObject);
begin
  (Sender as TDateTimePicker).Format:=ShortDateFormat;  // this is needed to support a "blank" optional date
end;


procedure TfrmVRMedicalRequest.edtRerouteDetailsEnter(Sender: TObject);  // Patch197 JRL
begin
  if ScreenReaderActive then // just in case
     GetScreenReader.Speak(edtRerouteDetails.Text);
end;

//------------------------------------------------------------------------------
// Enable Assign Consults Button
//------------------------------------------------------------------------------
// Assign Consults button is never visible on the Add Exam screen.  On the
// Edit Exam screen, it is only visible if the user has VHA Coordinator login
// credentials AND if a consult is not already assigned.  If an exam is
// cancelled or completed, it should not be enabled.
// Otherwise, the button stays hidden.
//------------------------------------------------------------------------------
function TfrmVRMedicalRequest.EnableAssignConsultsButton: Boolean;
begin
  Result := FALSE;
  if frmMain.VocRehab.AddExam then 
     Exit;
  if frmMain.VocRehab.VHAAccess = FALSE then
     Exit;
  if (frmMain.VocRehab.RequestCancelled) OR
     (frmMain.VocRehab.RequestCompleted) then
     Exit;
  if edtSelectedConsult.Text = NO_CONSULTS_FOUND then
     Result := TRUE
  else
     Result := FALSE;
end;

//------------------------------------------------------------------------------
// Show Cancel Request Button
//------------------------------------------------------------------------------
// The Cancel Request button is not visible when creating a new exam request.
// The Cancel Request button is only available when the Consult status is NEW.
// This means that no consults have been assigned to the request.  If no 
// consults are assigned, then allow the user to cancel the request.
//------------------------------------------------------------------------------
function TfrmVRMedicalRequest.ShowCancelRequestButton: Boolean;
begin
  Result := FALSE;  // don't show button
  if frmMain.VocRehab.AddExam then 
     Exit;
  if (edtSelectedConsult.Text = '') OR (edtSelectedConsult.Text = NO_CONSULTS_FOUND) then // consult not assigned yet
     Result := TRUE;  // show button
end;

procedure TfrmVRMedicalRequest.tsRerouteInfoEnter(Sender: TObject);  
begin
end;

//------------------------------------------------------------------------------
// Button - Anticipated Date Calendar Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnCalAnticipatedDateClick(Sender: TObject);
var
  ADate : TDateTime;
begin
  frmVRSelectDate := TfrmVRSelectDate.Create(Application);
  try
    if frmMain.VocRehab.AddExam = TRUE then  // new exam - enforce no previous dates
       frmVRSelectDate.MinimumDate := Tomorrow
    else
       frmVRSelectDate.MinimumDate := StrToDate('01/01/1900');  // editing - allow reasonable dates
    if TryStrToDate(leAnticipatedDate.Text,ADate) then
      frmVRSelectDate.calSelectDate.Date := ADate
    else
      frmVRSelectDate.calSelectDate.Date := Tomorrow;
    frmVRSelectDate.ShowModal;
    leAnticipatedDate.Text := frmVRSelectDate.SelectedDate;
  finally
    FreeAndNil(frmVRSelectDate);
  end;
  if (ActiveControl = leAnticipatedDate) then
    SelectNext(ActiveControl as TWinControl, TRUE, TRUE);
end;

//------------------------------------------------------------------------------
//  Button - Preferred Date Calendar Clicked
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnCalPreferredDateClick(Sender: TObject);
var
  ADate : TDateTime;
begin     
  frmVRSelectDate := TfrmVRSelectDate.Create(Application);
  try
    if frmMain.VocRehab.AddExam = TRUE then  // new exam - enforce no previous dates
       frmVRSelectDate.MinimumDate := Tomorrow
    else
       frmVRSelectDate.MinimumDate := StrToDate('01/01/1900');  // editing - allow reasonable dates
    if TryStrToDate(lePreferredDate.Text,ADate) then
      frmVRSelectDate.calSelectDate.Date := ADate
    else
      frmVRSelectDate.calSelectDate.Date := Tomorrow;
    frmVRSelectDate.ShowModal;
    lePreferredDate.Text := frmVRSelectDate.SelectedDate;
  finally
    FreeAndNil(frmVRSelectDate);
  end;
  if lePreferredDate.Text <> '' then
  begin
    edtPreferredScheduleDateReason.ReadOnly := FALSE;
    lblPreferredScheduleDateReason.Font.Color := clBlack;
    edtPreferredScheduleDateReason.Color := clWhite;
    edtPreferredScheduleDateReason.Font.Color := clBlack;
    edtPreferredScheduleDateReason.SetFocus;
  end
  else
  begin
    edtPreferredScheduleDateReason.ReadOnly := TRUE;
    lblPreferredScheduleDateReason.Font.Color := clGray;
    edtPreferredScheduleDateReason.Color := clBtnFace;
  end;
end;

//------------------------------------------------------------------------------
// Date Complete - Date validation function
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.DateComplete(var StrLikeDate : string; MinDate : TDate; const NowIfError : Boolean = TRUE);
var
   DateStr : string;
   Year, Month, Day : Word;
   cnt, SepCount : Integer;
begin
   DateStr := StrLikeDate;
   if DateStr = '' then
      Exit;
   SepCount := 0;
   for cnt := 1 to Length(DateStr) do
   begin
     if not (DateStr[cnt] in ['0'..'9']) then
     begin
       DateStr[cnt] := DateSeparator;
       inc(SepCount);
     end;
   end;
   while (DateStr <> '') and (DateStr[Length(DateStr)]=DateSeparator) do
   begin
     Delete(DateStr, Length(DateStr),1);
     Dec(SepCount);
   end;
   DecodeDate(Now,Year,Month,Day);
   if SepCount = 0 then
   begin
     case Length(DateStr) of
       0   : DateStr := DateToStr(Now);
       1, 2: DateStr := DateStr+DateSeparator+IntToStr(Day);
       4   : Insert(DateSeparator, DateStr, 3);
       6, 8: begin
             Insert(DateSeparator, DateStr, 5);
             Insert(DateSeparator, DateStr, 3);
             end; 
     end; // case
   end; // if SepCount
   try
     StrLikeDate := FormatDateTime('mm/dd/yyyy',StrToDate(DateStr));
   Except
     if NowIfError = TRUE then
     begin
      StrLikeDate := DateToStr(Date)
     end
     else
        StrLikeDate := '';
   end;
   // Date is valid, check for greater than minimum date
   if StrToDate(DateStr) < MinDate then
      StrLikeDate := 'Min ' + StrLikeDate;
end;

//------------------------------------------------------------------------------
// Re-Route Voc Rehab Request                                          Patch 197
//------------------------------------------------------------------------------
// Click the Re-Route button and show the Re-Route Request form.
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnRerouteRequestClick(Sender: TObject);
var
   frmVocRehabRerouteRequest : TfrmVocRehabRerouteRequest;
   modal_result : Integer;
begin
  btnRerouteRequest.Enabled := FALSE;  // don't allow multiple attempts
  frmVocRehabRerouteRequest := TfrmVocRehabRerouteRequest.Create(Nil);
  try
    // Show Re-Route Request screen
    modal_result := frmVocRehabRerouteRequest.ShowModal;
  finally
    FreeAndNil(frmVocRehabRerouteRequest);
  end;

  // If you submit the re-route, close form. If you cancel the re-route, keep form open
  if ReturnValue = CLOSE_FORM then
     Close
  else
     btnRerouteRequest.Enabled := TRUE;  // allow re-routing again
end;


//------------------------------------------------------------------------------
// Return Re-Routed Voc Rehab Request                                  Patch 197
//------------------------------------------------------------------------------
// Click the Re-Route Return button.  Show the Re-Route Return screen.
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnReturnReRouteClick(Sender: TObject);
var
  frmVocRehabReturnRequest : TForm;
begin
  try
    // Show Return screen
    frmVocRehabReturnRequest := TfrmVocRehabReturnRequest.Create(Application);
    frmVocRehabReturnRequest.ShowModal;
    // If you submit the return, close form. If you cancel the return, keep form open
    if ReturnValue = CLOSE_FORM then
       frmVRMedicalRequest.Close;
  finally
    FreeAndNil(frmVocRehabReturnRequest);
  end;

end;

//------------------------------------------------------------------------------
// Accept Re-Routed Voc Rehab Request                                  Patch 197
//------------------------------------------------------------------------------
// Click the Re-Route Accept button.
//------------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.btnAcceptReRouteClick(Sender: TObject);
const
  SUCCESS = TRUE;
begin
  if SendAcceptReRouteInfo = SUCCESS then
  begin
    UpdateCommentsReRouteTabs(TAB_REROUTE_DISPLAY_INFO_MODE);
    UpdateReRoutedVistADisplayFields;          // refresh VistA Re-Route fields
    SaveFlag := FALSE;  // reset form update flag so screen refreshes don't trigger a save message
  end;

end;

// -----------------------------------------------------------------------------
// Send Accept ReRoute Info                                            Patch 197
// -----------------------------------------------------------------------------
// This will be a two-step process.  First, we accept the request and the 
// status information is updated on the local VistA site.  Successful accepting
// at the local site provides the station number and original IEN of the 
// originating site.  We call the same RPC again via the XWB DIRECT RPC call
// to update the originating site.
//
// RPC Input:    
//    REQUEST IEN
//    ACCEPT 'A' or RETURN 'R'
//    RETURN REASON (255 CHARACTERS) - only used for return
// RPC Output (Reports sending of MailMan message):   
//    1:  SUCCESS = 1 or FAILURE = 0 ^
//    2:  SUCCESS or FAILURE Message ^
//    The following two values are only returned when we are updating the site
//    we are logged into.  If we are calling via the XWB DIRECT RPC call, only
//    the first two pieces are returned.
//    3:  Station Number ^ (for notifying originating site) - if no station 
//        number is returned, then originating site is the same 
//    4:  Original IEN (for notifying originating site)
// -----------------------------------------------------------------------------
function TfrmVRMedicalRequest.SendAcceptReRouteInfo: Boolean;
CONST   
  RPC_DIRECT = 'XWB DIRECT RPC'; 
  RPCNAME = 'DVBA VR REROUTE STATUS';
  ACCEPT = 'A';
  SUCCESS = '1';
  FAILURE = '0';
  PENDING = 'PENDING';
var
  Status, StatusMessage, StationNum, OriginalIEN, buffer : string;
begin
  Screen.Cursor := crHourGlass;
  Application.ProcessMessages;
  Result := FALSE;
  // Update local VistA with ACCEPT status
  frmMain.RPCBroker1.Results.Clear;
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
  frmMain.RPCBroker1.Param[0].Value := SelectedMedicalRequestIEN;  // Request IEN
  frmMain.RPCBroker1.Param[0].PType := literal;
  frmMain.RPCBroker1.Param[1].Value := ACCEPT;
  frmMain.RPCBroker1.Param[1].PType := literal;
  frmMain.RPCBrokerCall;
  Try
    frmMain.RPCBroker1.Call;
  Except  // RPC Error - RPC failed, user exits
    On E: EBrokerError Do  
    begin
      ShowMessageCAPRI(RPCNAME + ' could not be accessed!  '+#10#13+E.Message);
      Screen.Cursor := crDefault;
      Exit;  // allow user to try again or hit cancel
    end;
  End;  // try
  
  If frmMain.RPCBroker1.Results.Count > 0 Then
  begin  // RPC returned results - process them
    Status := Piece(frmMain.RPCBroker1.Results[0], '^', 1); 
    StatusMessage := Piece(frmMain.RPCBroker1.Results[0], '^', 2);
    StationNum := Piece(frmMain.RPCBroker1.Results[0], '^', 3);
    OriginalIEN := Piece(frmMain.RPCBroker1.Results[0], '^', 4);
    if Status = SUCCESS then // Success, request was re-routed
    begin
      // Update Originating site's status (same RPC called via XWB DIRECT RPC)
      frmMain.RPCBroker1.RemoteProcedure := RPC_DIRECT;
      frmMain.RPCBroker1.Param[0].PType := literal;
      frmMain.RPCBroker1.Param[0].Value := StationNum;
      frmMain.RPCBroker1.Param[1].PType := literal;
      frmMain.RPCBroker1.Param[1].Value := RPCNAME;
      frmMain.RPCBroker1.Param[2].PType := literal;
      frmMain.RPCBroker1.Param[2].Value := '';  // RPC version parameter (Optional)
      frmMain.RPCBroker1.Param[3].PType := literal;
      frmMain.RPCBroker1.Param[3].Value := OriginalIEN; 
      frmMain.RPCBroker1.Param[4].Value := ACCEPT;
      frmMain.RPCBroker1.Param[4].PType := literal;
      try
        frmMain.RPCBrokerCall;
        frmMain.RPCBroker1.Call;
        If frmMain.RPCBroker1.Results.Count > 0 Then
        begin // RPC returned results - process them
          Status := Piece(frmMain.RPCBroker1.Results[0], '^', 1);
          StatusMessage := Piece(frmMain.RPCBroker1.Results[0], '^', 2); // debug only, not displaying
          if Status = SUCCESS then
          begin
             Result := TRUE;
             frmMain.VocRehab.SendMailManMessage('ACCEPTED');  
             PopulateFieldValues;  // update fields including Request Status
          end
          else // if Status = FAILURE then  // Originating VistA site was not updated
          begin
            buffer := RPC_DIRECT + ' : ' + RPCNAME + #13#10 +
                      ' did not update the 8861 Request Status correctly.  ' +
                      edtReRoutedFrom.Text + ' has not been notified.';
             ShowMessageCAPRI(buffer);
          end;
        end // if RPCBroker1.Results.Count > 0
        else // RPCBroker1.Results.Count = 0
        begin
          buffer := RPC_DIRECT + ' : ' + RPCNAME + #13#10 +
                    ' did not return any results.  ' +
                    edtReRoutedFrom.Text + ' has not been notified.';
           ShowMessageCAPRI(buffer);
        end;
      except
      On E: EBrokerError Do
        begin
          Screen.Cursor := crDefault;
          ShowMessageCAPRI(RPCNAME + ' could not be accessed at Station Number ' + StationNum + '!'+#13#10+E.Message);
        end;
      end; // try except
    end // Status = SUCCESS
    else // Status = FAILURE
    begin
      buffer := RPCNAME + ' did not update the 8861 Request Status correctly.' +
                edtReRoutedFrom.Text + ' has not been notified.';
      ShowMessageCAPRI(buffer);
    end;
  end // if Broker1.Results.Count > 0
  else // RPC call returned no results
  begin
    buffer := RPCNAME + ' failed. ' + edtReRoutedFrom.Text + ' has not been notified.';
    ShowMessageCAPRI(buffer);
  end;
 Screen.Cursor := crDefault;
end;

// -----------------------------------------------------------------------------
// Update VistA Display Fields for Re-Routed Information               Patch 197
// -----------------------------------------------------------------------------
// RPC:  DVBA VR REROUTE INFO
//   Input:   Request IEN
//   Output:  
//     Line 1:  Re-Route To Site ^
//              Re-Route Date ^
//              Re-Route Status ^
//              Re-Route Status Date ^
//              Re-Routed From Site ^
//              Re-Route Reason ^
//              Show/Hide Accept and Return buttons (1 = show) ^
//              Re-Route From Routing Location ^      
//              Re-Route To Routing Location
//     Line 2:  Re-Route Description (250 chars)
//     Line 3:  Return Reason     (250 chars)
//
// Procedure returns TRUE if the request has re-route information, FALSE otherwise
//------------------------------------------------------------------------------
function TfrmVRMedicalRequest.UpdateReRoutedVistADisplayFields : Boolean;
const
  RPCNAME = 'DVBA VR REROUTE INFO';
  NOT_REROUTED = 'THIS 8861 REQUEST HAS NOT BEEN REROUTED';
  REROUTED_NEW = 'NEW';
  REROUTED_ACCEPTED = 'ACCEPT';
  REROUTED_RETURNED = 'RETURN';
var
  ReRoutedMsg : String;
  i : Integer;
  Flag : String;
begin
  // clear all the fields
  edtReRoutedTo.Clear;
  edtReRoutedDate.Clear;
  edtReRoutedStatus.Clear;
  edtReRoutedStatusDate.Clear;
  edtReRoutedFrom.Clear;
  edtReRouteReason.Clear;
  memoReRouteDescription.Lines.Clear;
  memoReturnReason.Lines.Clear;
  // Get Re-Route Information
  frmMain.RPCBroker1.Results.Clear;                                         
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;  
  frmMain.RPCBroker1.Param[0].PType := literal;
  frmMain.RPCBroker1.Param[0].Value := SelectedMedicalRequestIEN;
  frmMain.RPCBrokerCall;                                                    
  try                                                                       
    frmMain.RPCBroker1.Call;                                                
  except                                                                    
    On E: EBrokerError Do                                                      
       ShowMessageCAPRI('RPC: '+RPCNAME + ' could not be accessed!  '+ e.Message); 
  end;                                                                      
  if frmMain.RPCBroker1.Results.Count > 0 then                              
  begin
    // RPC returns the message: THIS 8861 REQUEST HAS NOT BEEN REROUTED
    // in the 'Re-Routed To' field if this request has no re-routing data.  
    // Do not display that message.
    ReRoutedMsg := Uppercase(Piece(frmMain.RPCBroker1.Results[0], '^', 1));
    if ReRoutedMsg <> NOT_REROUTED then // valid data, fill in display
    begin
      edtReRoutedTo.Visible := TRUE;
      edtReRoutedDate.Visible := TRUE;
      edtReRoutedStatus.Visible := TRUE;
      edtReRoutedStatusDate.Visible := TRUE;
      edtReRoutedFrom.Visible := TRUE;
      edtReRouteReason.Visible := TRUE;
      lblReRoutedTo.Visible := TRUE;
      lblReRoutedDate.Visible := TRUE;
      lblReRoutedStatus.Visible := TRUE;
      lblReRoutedStatusDate.Visible := TRUE;
      lblReRoutedFrom.Visible := TRUE;
      lblReRouteReason.Visible := TRUE;
      edtReRoutedTo.Text := ReRoutedMsg;
      edtReRoutedDate.Text := Piece(frmMain.RPCBroker1.Results[0], '^', 2);
      edtReRoutedStatus.Text := Piece(frmMain.RPCBroker1.Results[0], '^', 3);
      edtReRoutedStatusDate.Text := Piece(frmMain.RPCBroker1.Results[0], '^', 4);
      edtReRoutedFrom.Text := Piece(frmMain.RPCBroker1.Results[0], '^', 5);
      edtReRouteReason.Text := Piece(frmMain.RPCBroker1.Results[0], '^', 6);
      Flag := Piece(frmMain.RPCBroker1.Results[0], '^', 7); // re-route button flag
      // check whether or not re-route buttons should be displayed
      if Flag = '1' then
         ShowAcceptReturnReRouteButtons := TRUE
      else
         ShowAcceptReturnReRouteButtons := FALSE;
      // setup Re-Route tab display and data
      if (edtReRoutedStatus.Text = REROUTED_ACCEPTED) OR
         (edtReRoutedStatus.Text = REROUTED_NEW) then  // no return reason displayed
      begin
         memoReRouteDescription.Height := 85;
         lblReturnReason.Visible := FALSE;
         memoReturnReason.Visible := FALSE;
         if frmMain.RPCBroker1.Results.Count > 1 then
           memoReRouteDescription.Lines.Add(frmMain.RPCBroker1.Results[1]);
      end;
      if (edtReRoutedStatus.Text = REROUTED_RETURNED) then 
      begin
         memoReRouteDescription.Height := 35;
         lblReturnReason.Visible := TRUE;
         memoReturnReason.Visible := TRUE;
         if frmMain.RPCBroker1.Results.Count > 1 then
            memoReRouteDescription.Lines.Add(frmMain.RPCBroker1.Results[1]);
         if (frmMain.RPCBroker1.Results.Count > 2) then
            memoReturnReason.Lines.Add(frmMain.RPCBroker1.Results[2]);
      end;
      Result := TRUE; // re-route information exists
    end
    else
    begin
      Result := FALSE; // no re-route information exists
      edtReRoutedTo.Visible := FALSE;
      edtReRoutedDate.Visible := FALSE;
      edtReRoutedStatus.Visible := FALSE;
      edtReRoutedStatusDate.Visible := FALSE;
      edtReRoutedFrom.Visible := FALSE;
      edtReRouteReason.Visible := FALSE;
      lblReRoutedTo.Visible := FALSE;
      lblReRoutedDate.Visible := FALSE;
      lblReRoutedStatus.Visible := FALSE;
      lblReRoutedStatusDate.Visible := FALSE;
      lblReRoutedFrom.Visible := FALSE;
      lblReRouteReason.Visible := FALSE;
    end;
  end;                       
end;


// -----------------------------------------------------------------------------
// Can Request Be Re-Routed?                                           Patch 197
// -----------------------------------------------------------------------------
// A Request can be re-routed if it has one of the statuses below and hasn't 
// already been re-routed.
//   REQUEST STATUS:
//      * NEW
//      * PENDING
//      * NEW, RE-ROUTED
// -----------------------------------------------------------------------------
// Input:    Request IEN
// Output:   '0' - request cannot be re-routed
//           '1' - request can be re-routed
//           ^
//           Current Station Number (needed for re-routing within the current site)
// -----------------------------------------------------------------------------
function TfrmVRMedicalRequest.CanRequestBeRerouted(RequestIEN: String): Boolean;
CONST
  RPCNAME = 'DVBA VR GET REROUTE';
var
  i : Integer;
  Status : string;
begin

  frmMain.RPCBroker1.Results.Clear;
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
  frmMain.RPCBroker1.Param[0].Value := RequestIEN;
  frmMain.RPCBroker1.Param[0].PType := literal;
  frmMain.RPCBrokerCall;
  Try
    frmMain.RPCBroker1.Call;
  Except
    On E: EBrokerError Do
      ShowMessageCAPRI(RPCNAME + ' could not be accessed!'+#13#10+E.Message);
  End;

  Result := FALSE; // Request cannot be re-routed
  
  If frmMain.RPCBroker1.Results.Count > 0 Then
  begin
    Status := Piece(frmMain.RPCBroker1.Results[0], '^', 1); 
    CurrentStationNumber := Piece(frmMain.RPCBroker1.Results[0], '^', 2); // store station number for possible re-routing later
    if Status = '1' then // Request can be re-routed
      Result := TRUE;
  end;
end;

// -----------------------------------------------------------------------------
// Update Comments / Re-Route Tab                                      Patch 197 
// -----------------------------------------------------------------------------
// If the request is a newly re-routed request, we want to display the
// Re-Route tab and Re-Route buttons.  Highlight tab in orange to make it stand
// out. All controls should be read-only until the request is accepted/returned.
// Input Parameter is:  TAB_REROUTE_ACCEPT_RETURN_MODE  
//
// If the request has already been accepted, the re-route tab should be blue. 
// All controls are editable.  
// If the re-route status is shown as accepted, then show the comments tab on top.
// If the re-route status is shown as returned and the current request status is
// NEW or PENDING, REPORTED, show the re-route tab on top.  This enables a user
// to see why a request has been returned.
// Input Parameter is:  TAB_REROUTE_DISPLAY_INFO_MODE
// -----------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.UpdateCommentsReRouteTabs(Tab_ReRoute: String);
const
  REROUTED_RETURNED = 'RE-ROUTED RETURNED';
  REROUTED_ACCEPTED = 'RE-ROUTED ACCEPTED';
  NEW = 'NEW';
  PENDING = 'PENDING';
  NEW_REROUTED = 'NEW, RE-ROUTED';
  OPEN = 'OPEN';
  ORANGE = $000080FF;
  BLUE = clSkyBlue;
begin
  if Tab_ReRoute = TAB_REROUTE_ACCEPT_RETURN_MODE then
  begin
     UpdateControlsForEditingOrReadOnly(TRUE);              // Read Only Mode until you accept
     tsRerouteInfo.TabVisible := TRUE;                      // Show tab
     pageVocRehab.ActivePage := tsRerouteInfo;              // Re-Route Tab
     btnAcceptReRoute.Visible := TRUE;                      // Show Accept button
     btnReturnReRoute.Visible := TRUE;                      // Show Return button
   end
   else // Tab_ReRoute = TAB_REROUTE_DISPLAY_INFO_MODE
   begin  
     // check statuses to determine if fields should be editable.
     if (frmMain.VocRehab.RequestStatus = NEW) OR
        (frmMain.VocRehab.RequestStatus = PENDING) then
        UpdateControlsForEditingOrReadOnly(FALSE)           // Edit Mode 
     else
        UpdateControlsForEditingOrReadOnly(TRUE);           // Read-Only Mode 
     pageVocRehab.ActivePage := tsMedicalRequest;           // Comment Tab
     if edtReRoutedStatus.Text = REROUTED_RETURNED then     // returned, always show
        pageVocRehab.ActivePage := tsRerouteInfo;           // return tab
     btnAcceptReRoute.Visible := FALSE;                     // Hide Accept button
     btnReturnReRoute.Visible := FALSE;                     // Hide Return button
     pnlButtons.Color := BLUE;                              // panel color blue
   end;
end;


// -----------------------------------------------------------------------------
// Update Controls for Editing or Read Only                            Patch 197 
// -----------------------------------------------------------------------------
// Do not allow the user to edit controls until they have accepted the 
// re-routed request.  This routine enabled or disables the controls 
// accordingly.
// -----------------------------------------------------------------------------
procedure TfrmVRMedicalRequest.UpdateControlsForEditingOrReadOnly(
  Enabled: Boolean);
begin
  SetEditsToReadOnly(Enabled);
  // In addition to normal controls, re-routing also requires additional controls
  // to be enabled/disabled depending upon a request status.  If a request has 
  // be re-routed, no controls are editable
  FMCommentsNotes.Enabled := not Enabled;   
  btnSendRequest.Visible := not Enabled;   
  btnCancelRequest.Visible := not Enabled; 
  btnAssignConsults.Visible := not Enabled; 
end;

end.
