unit VocRehabReportSetup;
//------------------------------------------------------------------------------
// CodeCR347 - JRL June, 2012 - Added unit/form
// Vocational Rehab Form 28-8861
// 
// This form supports the Report Setup for the Vocational Rehab status report.
// This report is located on the Report Menu under the heading: --VR & E REPORTS--
// selecting "VR&E Status".  It is called from REPORTS.PAS.
//------------------------------------------------------------------------------

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ORCtrls, ComCtrls, ExtCtrls, Fmcntrls, dialogsCAPRI;

const
   DATE_RANGE = 'Date Range';
   
type
  TfrmVREReportSetup = class(TForm)
    Panel1: TPanel;
    lbVRReportBy: TListBox;
    lbVRStatus: TListBox;
    dtVRStartDate: TDateTimePicker;
    dtVRStopDate: TDateTimePicker;
    btnCancel: TButton;
    btnOK: TButton;
    lblRegionalOffice: TLabel;
    lblStartDate: TLabel;
    lblStopDate: TLabel;
    lblStatus: TLabel;
    lblReportBy: TLabel;
    FMRegionalOfficeNumberIEN: TFMEdit;
    FMRegionalOfficeNumber: TFMComboBox;
    pnlReportOutputFormat: TPanel;
    rbPlainText: TRadioButton;
    rbDelimited: TRadioButton;
    Label2: TLabel;
    edtFormTitle: TEdit;
    chkAllROs: TCheckBox;
    procedure btnCancelClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure lbVRReportByClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FMRegionalOfficeNumberEnter(Sender: TObject);
    function  CheckForErrors : Boolean;
    procedure chkAllROsClick(Sender: TObject);
    procedure dtVRStartDateExit(Sender: TObject);
    procedure dtVRStopDateExit(Sender: TObject);
  private
    { Private declarations }
    Date30MonthsAgo : TDateTime;
  public
    { Public declarations }
    procedure SetupReportParameters;
  end;

var
  frmVREReportSetup: TfrmVREReportSetup;

implementation

uses Main, rofinder, VocRehabClass, VAUtils;

{$R *.dfm}


//------------------------------------------------------------------------------
// Form Activate
//------------------------------------------------------------------------------
procedure TfrmVREReportSetup.FormActivate(Sender: TObject);
begin
  SetupReportParameters;
  if VAUtils.ScreenReaderActive then
     edtFormTitle.TabStop := TRUE
  else
     edtFormTitle.TabStop := FALSE;
  // All R.O.'s is checked by default now so setup controls accordingly  -- JRL Patch 197 3/7/17 
  FMRegionalOfficeNumber.Text := '';
  FMRegionalOfficeNumberIEN.Text := 'ALL';
  FMRegionalOfficeNumber.Enabled := FALSE;
     
end;


//------------------------------------------------------------------------------
// Cancel button Clicked
//------------------------------------------------------------------------------
procedure TfrmVREReportSetup.btnCancelClick(Sender: TObject);
begin
   Close;
end;

//------------------------------------------------------------------------------
// Ok button clicked
//------------------------------------------------------------------------------
procedure TfrmVREReportSetup.btnOKClick(Sender: TObject);
begin
   if CheckForErrors then
   begin
     ModalResult := mrNone;
     Exit;
   end;
   // Process Report
   frmMain.VocRehab.ReportStartDate := dtVRStartDate.DateTime;
   frmMain.VocRehab.ReportStopDate := dtVRStopDate.DateTime;
   frmMain.VocRehab.ReportIsDelimited := rbDelimited.Checked;
// frmMain.VocRehab.ReportStatus := Chr(Integer(lbVRStatus.Items.Objects[lbVRStatus.ItemIndex])); // Integer stored is ASCII Value for (A)All, (P)Pending, (X)Cancelled, (C)Complete, (N)New
   frmMain.VocRehab.ReportStatus := frmMain.VocRehab.VocRehabRequestStatusCodes[lbVRStatus.ItemIndex];  //Patch197 JRL 1/20/17
   frmMain.VocRehab.ReportRegionalOffice := FMRegionalOfficeNumberIEN.Text;
end;


//------------------------------------------------------------------------------
// Check For Errors
//------------------------------------------------------------------------------
function TfrmVREReportSetup.CheckForErrors: Boolean;
begin
  Result := FALSE;
  if frmMain.VocRehab.ValidateDateRange(dtVRStartDate.DateTime,dtVRStopDate.DateTime) then
  begin
     ShowMessageCAPRI('Please correct the error in the Date Range.');
     Result := TRUE;
  end;
  if (FMRegionalOfficeNumber.Text = '') AND
     (chkAllROs.Checked = FALSE) then
  begin
     ShowMessageCAPRI('Please complete the Regional Office selection.');
     Result := TRUE;
  end;
  if FMRegionalOfficeNumberIEN.Text = '' then
  begin
     ShowMessageCAPRI('Regional Office IEN is blank.');
     Result := TRUE;
  end;
                                        
end;

//------------------------------------------------------------------------------
// Checkbox All Regional Offices clicked
//------------------------------------------------------------------------------
procedure TfrmVREReportSetup.chkAllROsClick(Sender: TObject);
begin
  if chkAllROs.Checked then
  begin
    FMRegionalOfficeNumber.Text := '';
    FMRegionalOfficeNumberIEN.Text := 'ALL';
    FMRegionalOfficeNumber.Enabled := FALSE;
  end
  else
  begin                                     
    FMRegionalOfficeNumber.Text := '';
    FMRegionalOfficeNumberIEN.Text := '';
    FMRegionalOfficeNumber.Enabled := TRUE;
  end;
end;

//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
procedure TfrmVREReportSetup.dtVRStartDateExit(Sender: TObject);
begin
  try
    if dtVRStartDate.Date <  Date30MonthsAgo then
       dtVRStartDate.Date := Date30MonthsAgo;
  except
    on e:exception do
      dtVRStartDate.Date := Date30MonthsAgo;
  end;
  try
    if dtVRStartDate.Date > Now then
       dtVRStartDate.Date := Now;
  except
    on e:exception do
       dtVRStartDate.Date := Now;
  end;

end;

//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
procedure TfrmVREReportSetup.dtVRStopDateExit(Sender: TObject);
begin
  try
    if dtVRStopDate.Date <  Date30MonthsAgo then
       dtVRStopDate.Date := Date30MonthsAgo;
  except
    on e:exception do
       dtVRStopDate.Date := Date30MonthsAgo;
  end;
  try
    if dtVRStopDate.Date > Now then
       dtVRStopDate.Date := Now;
  except
    on e:exception do
       dtVRStopDate.Date := Now;
  end;
end;

//------------------------------------------------------------------------------
// Regional Office control entered
//------------------------------------------------------------------------------
procedure TfrmVREReportSetup.FMRegionalOfficeNumberEnter(Sender: TObject);
begin
  // code stolen from newexam.pas:  TfrmNewExam.FMRegionalOfficeNumberEnter(Sender: TObject);
  // with a few substractions for unnecessary code in this instance 
  If frmROFinder.ShowModal = mrOK Then
    If frmROFinder.ORListBoxSites.ItemIndex > -1 Then
    Begin
      FMRegionalOfficeNumber.Text := frmMain.Piece(frmROFinder.ORListBoxSites.Items[frmROFinder.ORListBoxSites.ItemIndex], '^', 1);
      FMRegionalOfficeNumberIEN.Text := frmMain.Piece(frmROFinder.ORListBoxSites.Items[frmROFinder.ORListBoxSites.ItemIndex], '^', 2);
    End;
  // note: since this combo box is being used in a non-conventional way, it is
  // necessary for focus to pass to the next control to behave as the users are
  // accustomed
  if (ActiveControl = FMRegionalOfficeNumber) then
    SelectNext(ActiveControl as TWinControl, TRUE, TRUE);
  exit;
end;

//------------------------------------------------------------------------------
// Setup Report Parameters
//------------------------------------------------------------------------------
procedure TfrmVREReportSetup.SetupReportParameters;
begin
   // make sure VocRehab class is setup for report
   if not Assigned(frmMain.VocRehab) then
      frmMain.VocRehab := TVocRehab.Create;
      
   fmRegionalOfficeNumber.Text := '';
   FMRegionalOfficeNumberIEN.Text := '';
   fmRegionalOfficeNumber.Visible := true;
   FMRegionalOfficeNumber.Enabled := TRUE;
   chkAllROs.Visible := TRUE;
   
   frmMain.VocRehab.ReportRegionalOffice := FMRegionalOfficeNumberIEN.Text;
   // Report Status Options
   frmMain.VocRehab.GetReportStatusList;
   lbVRStatus.ItemIndex := lbVRStatus.Items.IndexOf('ALL');
   // Set up the Date Options
   lbVRReportBy.Items.Clear;
   lbVRReportBy.Items.AddStrings(frmMain.VocRehab.DateOptionList);
   lbVRReportBy.ItemIndex := 0;
   Date30MonthsAgo := IncMonth(Now, -30);
//   dtVRStartDate.MinDate := Date30MonthsAgo;
//   dtVRStopDate.MinDate := Date30MonthsAgo;
   dtVRStartDate.Date := now;
   dtVRStopDate.Date := now;
end;     


//------------------------------------------------------------------------------
// Report By Listbox Clicked
//------------------------------------------------------------------------------
procedure TfrmVREReportSetup.lbVRReportByClick(Sender: TObject);
var
  StartDate,StopDate : TDateTime;
  InputString : String;
begin  
   // Radio Group box needs "Date Range" as the first item
   InputString := lbVRReportBy.Items[lbVRReportBy.ItemIndex];
   frmMain.VocRehab.GetDateRanges(InputString,StartDate,StopDate);
   dtVRStartDate.DateTime := StartDate;
   dtVRStopDate.DateTime := StopDate;
   // setting the DateTimePicker MinDate and MaxDate can cause repercussions when
   // calling the VocRehab class to setup the start/stop dates for all years
   // Check date programmatically.
   if InputString = DATE_RANGE then
   begin
     dtVRStartDate.Enabled := TRUE;
     dtVRStopDate.Enabled := TRUE;
   end
   else
   begin
     dtVRStartDate.Enabled := FALSE;
     dtVRStopDate.Enabled := FALSE;
   end;
end;

end.
