{ VocRehabReRouteRequest

Added Patch 197 JRL 2/1/17

This unit is called when a Voc Rehab 8861 Medical Request is re-routed.

RPCs called:
  DVBA VR SEND REROUTE
  DVBA CAPRI REROUTE VAMC
  DVBA VR GET REROUTE CODE
  XWB DIRECT RPC
  DVBA CHECK PATCH
}

unit VocRehabRerouteRequest;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms
  ,Dialogs
  ,StdCtrls
  ,Trpcb
  ,VA508AccessibilityManager
  ,VA508AccessibilityRouter
  ,CAPRISupport
  ,dialogsCAPRI
  ;

type
  TfrmVocRehabRerouteRequest = class(TForm)
    lblReRouteReason: TLabel;
    cbReRouteReason: TComboBox;
    lblVAMC: TLabel;
    cbVAMC: TComboBox;
    lblSortByState: TLabel;
    cbxSortByState: TCheckBox;
    lblDescription: TLabel;
    memoReRouteDescription: TMemo;
    btnRerouteRequest: TButton;
    btnCancelReRoute: TButton;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cbxSortByStateClick(Sender: TObject);
    procedure btnCancelReRouteClick(Sender: TObject);
    procedure btnRerouteRequestClick(Sender: TObject);
    procedure cbVAMCEnter(Sender: TObject);
    procedure memoReRouteDescriptionEnter(Sender: TObject);
    procedure cbVAMCExit(Sender: TObject);
  private
    { Private declarations }
    VAMC_IEN : String;
    VAMC_IEN_List : TStringList;
    VAMC_Station_List : TStringList;
    VAMC_Name_List : TStringList;
    VAMC_State_List : TStringList;
    ReRouteReasonIENList : TStringList;
    SaveStationNumber : String;
    SelectedStation : String;
    procedure GetVAMCLocations;
    procedure GetRerouteReasons;
    function  CheckUserInputValues: Boolean;
    function  CreateReRouteRequest : Boolean;
    function  CheckIfVAMCCanHandleReroutes: Boolean;
  public
    { Public declarations }
  end;

var
  frmVocRehabRerouteRequest: TfrmVocRehabRerouteRequest;

implementation

uses VocRehabMedicalRequest, main;

const
  CLOSE_FORM = '1';      // Patch 197 JRL 12/21/16
  DONT_CLOSE_FORM = '2'; // Patch 197 JRL 12/21/16

{$R *.dfm}


//------------------------------------------------------------------------------
// Form Create
//------------------------------------------------------------------------------
procedure TfrmVocRehabRerouteRequest.FormCreate(Sender: TObject);
begin
  // Create string lists
  VAMC_IEN_List := TStringList.Create;
  VAMC_Station_List := TStringList.Create;
  VAMC_Name_List := TStringList.Create;
  VAMC_State_List := TStringList.Create;
  ReRouteReasonIENList := TStringList.Create;
  // Populate initial fields
  GetReRouteReasons; // Populate Re-Routing Reasons dropdown
  GetVAMCLocations;  // Populate VAMC Locations dropdown
  SaveStationNumber := '';
end;


//------------------------------------------------------------------------------
// Form Close
//------------------------------------------------------------------------------
procedure TfrmVocRehabRerouteRequest.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // Destroy string lists
  FreeAndNil(VAMC_IEN_List);
  FreeAndNil(VAMC_Station_List);
  FreeAndNil(VAMC_Name_List);
  FreeAndNil(VAMC_State_List);
  FreeAndNil(ReRouteReasonIENList);
end;


//------------------------------------------------------------------------------
// Cancel Re-Route button Clicked
//------------------------------------------------------------------------------
procedure TfrmVocRehabRerouteRequest.btnRerouteRequestClick(Sender: TObject);
var
  request_rerouted : Boolean;
  CloseForm : Boolean;
begin
  if CheckUserInputValues then // if all inputs are satisfied, allow re-routing
  begin
     request_rerouted := CreateReRouteRequest;     // send the re-routed request
     if request_rerouted = TRUE then // re-route was successful
        frmVRMedicalRequest.ReturnValue := CLOSE_FORM
     else
        frmVRMedicalRequest.ReturnValue := DONT_CLOSE_FORM;
     Close;
  end;  // CheckUserInputValues
end;



//------------------------------------------------------------------------------
// Cancel Re-Route button Clicked
//------------------------------------------------------------------------------
procedure TfrmVocRehabRerouteRequest.btnCancelReRouteClick(Sender: TObject);
begin
  // No action needed except to close the form
  Close;
end;


//------------------------------------------------------------------------------
// Sort By State checkbox clicked
//------------------------------------------------------------------------------
procedure TfrmVocRehabRerouteRequest.cbxSortByStateClick(Sender: TObject);
begin
  GetVAMCLocations;
end;


//------------------------------------------------------------------------------
// Create Re-Route Request
//------------------------------------------------------------------------------
// RPC Input:    
//   Request IEN
//   Patient IEN
//   VAMC IEN (Station Number)
//   Routing Location IEN
//   Re-Route Reason IEN
//   Re-Route Description
//
// RPC Output:  
//   1^Transmitted as message # 909761 from this site to SERVER.DOMAIN       
//   0^Message transmission error! Request WILL NOT be rerouted
// -----------------------------------------------------------------------------
// FROM THE AMIE TRANSFER DOCUMENTATION:
//   If the mail message is sent out successfully, the user will see:
//   "Transmitted as message # {number} from this site to {site name}.VA.GOV."
// 
//   If the message fails (and therefore the transfer fails) the user will be so informed:
//   "Message transmission error!   Request WILL NOT be sent !"
// -----------------------------------------------------------------------------
function TfrmVocRehabRerouteRequest.CreateReRouteRequest : Boolean;
CONST
  RPCNAME = 'DVBA VR SEND REROUTE';  
var
  MailmanRPCStatus : Integer;
  MailmanMessage : String;
  buffer : string;
begin
  Result := FALSE; // default to re-route failed
  try
    Screen.Cursor := crHourGlass;
    // Setup RPC
    frmMain.RPCBroker1.Results.Clear;
    frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
    frmMain.RPCBroker1.Param[0].Value := frmVRMedicalRequest.SelectedMedicalRequestIEN;  // Request IEN
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[1].Value := PatientIEN;                              // Patient IEN
    frmMain.RPCBroker1.Param[1].PType := literal;
    frmMain.RPCBroker1.Param[2].Value := VAMC_IEN;
    frmMain.RPCBroker1.Param[2].PType := literal;
    frmMain.RPCBroker1.Param[3].Value := ReRouteReasonIENList[cbReRouteReason.ItemIndex];
    frmMain.RPCBroker1.Param[3].PType := literal;
    frmMain.RPCBroker1.Param[4].Value := memoReRouteDescription.Text;
    frmMain.RPCBroker1.Param[4].Ptype := literal;
    
    frmMain.RPCBrokerCall;
    Try
      frmMain.RPCBroker1.Call;
    Except
      On e: EBrokerError Do
      begin
        buffer := RPCNAME + ' could not be accessed!  '+#10#13 + E.Message + '.';
        ShowMessageCAPRI(buffer);
      end;
    End;
  finally
    Screen.Cursor := crDefault;
  end;

  // Check RPC results  
  If frmMain.RPCBroker1.Results.Count > 0 Then
  begin
    MailmanRPCStatus := StrToIntDef(Piece(frmMain.RPCBroker1.Results[0], '^', 1),0);
    MailmanMessage := Piece(frmMain.RPCBroker1.Results[0], '^', 2);
  
    if MailmanRPCStatus > 0 then // Success, request was re-routed
    begin
      // FYI: An email is sent to the original requester upon successful re-routing (all M code)
      ShowMessageCAPRI(MailmanMessage);  
      Result := TRUE;
      Close;
    end
  end;
  // if Re-Route message failed
  if not Result then
    ShowMessageCAPRI(RPCNAME + ' returned a failure.'+#10#13+MailmanMessage);
end;


//------------------------------------------------------------------------------
// Get VAMC Locations
//------------------------------------------------------------------------------
procedure TfrmVocRehabRerouteRequest.GetVAMCLocations;
var
  i : Integer;
CONST
  RPCNAME = 'DVBA CAPRI REROUTE VAMC';
  STRINGFORMAT = '%-28s%-15s%';
begin
  // setup and call RPC
  frmMain.RPCBroker1.Results.Clear;
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
  if cbxSortByState.Checked then
     frmMain.RPCBroker1.Param[0].Value := 'STATE'
  else
     frmMain.RPCBroker1.Param[0].Value := 'NAME';
  frmMain.RPCBroker1.Param[0].PType := literal;
  frmMain.RPCBrokerCall;
  Try
    frmMain.RPCBroker1.Call;
  Except
    On E: EBrokerError Do
      ShowMessageCAPRI(RPCNAME + ' could not be accessed!'+#13#10+E.Message);
  End;
  // clear all lists
  cbVAMC.Items.Clear;
  VAMC_Name_List.Clear;
  VAMC_State_List.Clear;
  VAMC_IEN_List.Clear;
  VAMC_Station_List.Clear;
  // populate all lists
  If frmMain.RPCBroker1.Results.Count > 0 Then
  begin
    For i := 0 To frmMain.RPCBroker1.Results.Count - 1 Do
    begin
      VAMC_Name_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 2));
      VAMC_State_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 1));
      VAMC_IEN_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 3));
      VAMC_Station_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 4));
    end;
  end;
  // Add data to listbox and format it into columns
  for i := 0 to VAMC_Name_List.Count - 1 do
    cbVAMC.Items.Add(Format(STRINGFORMAT,[VAMC_Name_List[i],VAMC_State_List[i]]));
end;



//------------------------------------------------------------------------------
// Get Re-Route Reasons
//------------------------------------------------------------------------------
// RPC:  DVBA CAPRI REROUTEREASONS
//   Input:   None
//   Output:  Re-Route Code^IEN
//------------------------------------------------------------------------------
procedure TfrmVocRehabRerouteRequest.GetReRouteReasons;
const
  RPCNAME = 'DVBA VR GET REROUTE CODE';
var
  i : Integer;
begin
  frmMain.RPCBroker1.Results.Clear;                                         
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;          
  frmMain.RPCBrokerCall;                                                    
  try                                                                       
    frmMain.RPCBroker1.Call;                                                
  except                                                                    
    On EBrokerError Do                                                      
       ShowMessageCAPRI('RPC: '+RPCNAME + ' could not be accessed!'); 
  end;                                                                      
  cbReRouteReason.Items.Clear;                                     
  ReRouteReasonIENList := TStringList.Create;                          
  if frmMain.RPCBroker1.Results.Count > 0 then                              
  begin                                                                     
    for i := 0 To frmMain.RPCBroker1.Results.Count - 1 do                   
    begin                                                                   
      cbReRouteReason.Items.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 2));
      ReRouteReasonIENList.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 1));
    end;                                                                   
  end;
end;
 
//------------------------------------------------------------------------------
// Check User Input Values
//------------------------------------------------------------------------------
// Check all user input values.  If values are input correctly, return TRUE.
// If values are missing, routine short circuits back to input form.
//------------------------------------------------------------------------------
function TfrmVocRehabRerouteRequest.CheckUserInputValues: Boolean;
begin
  if cbReRouteReason.Text = '' then
  begin
     ShowMessageCAPRI('Please enter a Re-Routing Reason.');
     Result := FALSE;
     exit;  // make user complete form before sending
  end;
  if cbReRouteReason.Text = 'OTHER' then
  begin
    if Length(memoReRouteDescription.Lines.Text) = 0 then
    begin
       ShowMessageCAPRI('Please enter a Re-Routing Description.');
       Result := FALSE;
       exit;  // make user complete form before sending
    end;
  end;
  if cbVAMC.ItemIndex = -1 then
  begin
    ShowMessageCAPRI('Please enter a VAMC.');
    Result := FALSE;
    exit;  // make user complete form before sending
  end;
  Result := TRUE;
end;
 

//------------------------------------------------------------------------------
// Check if VAMC Can Handle Re-Routes
//------------------------------------------------------------------------------
// If the destination site doesn't have Patch 193 installed, then it will not
// know how to receive a re-routed request.  Don't allow the user to re-route
// to a site where Patch 193 isn't installed. 
// Input:    Station Number at distant site
// Output:   
//    TRUE:  ok to re-route
//    FALSE: cannot re-route to this site 
// -----------------------------------------------------------------------------
function TfrmVocRehabReRouteRequest.CheckIfVAMCCanHandleReroutes: Boolean;
const
   RPC_DIRECT = 'XWB DIRECT RPC'; 
   RPCNAME = 'DVBA CHECK PATCH';
   PATCHNUM = 'DVBA*2.7*197';   
begin
        
  Result := FALSE;
  // if ItemIndex is not selected (or forced to -1), don't test.  Just fail
  if cbVAMC.ItemIndex = -1 then  
     exit;
  
  try
    Screen.Cursor := crHourGlass;
    // check patch number at selected site.  If Patch 197 is not installed, error
    frmMain.RPCBroker1.RemoteProcedure := RPC_DIRECT;
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[0].Value := SelectedStation;
    frmMain.RPCBroker1.Param[1].PType := literal;
    frmMain.RPCBroker1.Param[1].Value := RPCNAME;
    frmMain.RPCBroker1.Param[2].PType := literal;
    frmMain.RPCBroker1.Param[2].Value := '';  // RPC version parameter (Optional)
    frmMain.RPCBroker1.Param[3].PType := literal;
    frmMain.RPCBroker1.Param[3].Value := PATCHNUM;
    try
      frmMain.RPCBrokerCall;
      frmMain.RPCBroker1.Call;
      if frmMain.RPCBroker1.Results.Count > 0 then
        if piece(frmMain.RPCBroker1.Results[0], '^', 1) = '1' then
           Result := TRUE;
    except
      begin
        ShowMessageCAPRI(RPCNAME + ' could not be accessed at Station Number ' + SelectedStation + '!');
        Result := FALSE;
      end
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;


//------------------------------------------------------------------------------
// Exit VAMC Combobox
//------------------------------------------------------------------------------
procedure TfrmVocRehabRerouteRequest.cbVAMCExit(Sender: TObject);
var
  buffer : string;
begin
  if cbVAMC.ItemIndex <> -1 then
     VAMC_IEN := VAMC_IEN_list[cbVAMC.ItemIndex]
  else
     exit;

  SelectedStation := VAMC_Station_List[cbVAMC.ItemIndex];
  if SelectedStation = SaveStationNumber then
     Exit  // no need to repopulate things if just tabbing through controls
  else
     SaveStationNumber := SelectedStation;
//   If we can't re-route to the selected VAMC, set the VAMC to blank and
//   give the user a message.
  if CheckIfVAMCCanHandleReroutes = FALSE then
  begin
    buffer := VAMC_Name_List[cbVAMC.ItemIndex] + 
              VAMC_State_List[cbVAMC.ItemIndex] + 
              ' cannot accept re-routed requests at this time.';
    SaveStationNumber := '';  // make sure we retest the station number if previous one errored out
    cbVAMC.ItemIndex := -1;
    ShowMessageCAPRI(buffer);
    cbVAMC.SetFocus;
  end;
  end;


//------------------------------------------------------------------------------
// 508 Help
//------------------------------------------------------------------------------
procedure TfrmVocRehabRerouteRequest.cbVAMCEnter(Sender: TObject);
begin
  if cbVAMC.ItemIndex <> -1 then
     GetScreenReader.Speak('VAMC');
end;


procedure TfrmVocRehabRerouteRequest.memoReRouteDescriptionEnter(Sender: TObject);
begin
  GetScreenReader.Speak('Description');
  GetScreenReader.Speak(memoReRouteDescription.Text);
end;


end.
