unit VocRehabReturnRequest;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, buttons, Trpcb, CAPRISupport, VA508AccessibilityManager, 
  dialogsCAPRI, StdCtrls;

type
  TfrmVocRehabReturnRequest = class(TForm)
    memoReturnRequestReason: TMemo;
    btnReturnRequest: TButton;
    btnReturnCancel: TButton;
    stReturnReasonText: TStaticText;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    procedure btnReturnRequestClick(Sender: TObject);
    procedure btnReturnCancelClick(Sender: TObject);
  private
    { Private declarations }
    OriginalIEN : String;  // allow this value to be available to return email message 
    function SendReturnInfo : Boolean;
  public
    { Public declarations }
  end;

var
  frmVocRehabReturnRequest: TfrmVocRehabReturnRequest;

implementation

uses
  VocRehabMedicalRequest, Main, viewexam;

{$R *.dfm}



//------------------------------------------------------------------------------
// Cancel button clicked
//------------------------------------------------------------------------------
procedure TfrmVocRehabReturnRequest.btnReturnCancelClick(Sender: TObject);
begin
  frmVRMedicalRequest.ReturnValue := DONT_CLOSE_FORM;
  Close;
end;

//------------------------------------------------------------------------------
// Return Request button clicked
// -----------------------------------------------------------------------------
// Database restrictions:
//   Minimum length:  5 characters
//   Maximum length:  250 characters
// ReturnValue is not being used to close or not close the ViewExam screen.  
// -----------------------------------------------------------------------------
procedure TfrmVocRehabReturnRequest.btnReturnRequestClick(Sender: TObject);
const
  SUCCESS = TRUE;
  FAILURE = FALSE;
var
  Unused : Boolean;
begin
  if length(memoReturnRequestReason.text) >= 5 then // database requires 3 or more characters
  begin
    Unused := SendReturnInfo;  // return value is not being use, always close form
    frmVRMedicalRequest.ReturnValue := CLOSE_FORM;
    Close;
  end
  else
  begin
    ShowMessageCAPRI('You must enter a full explanation for returning this request.');
    memoReturnRequestReason.Lines.Clear;
    memoReturnRequestReason.SetFocus;
  end;
end;

// -----------------------------------------------------------------------------
// Send Return Info
// -----------------------------------------------------------------------------
// This will be a two-step process.  First, we return the request and the 
// status information is updated on the local VistA site.  Successful returning
// at the local site provides the station number and original IEN of the 
// originating site.  We call the same RPC again via the XWB DIRECT RPC call
// to update the originating site.
//
// RPC Input:    
//    REQUEST IEN
//    ACCEPT 'A' or RETURN 'R'
//    RETURN REASON (255 CHARACTERS)
// RPC Output (Reports sending of MailMan message):   
//    1:  SUCCESS = 1 or FAILURE = 0 ^
//    2:  SUCCESS or FAILURE Message ^
//    The following two values are only returned when we are updating the site
//    we are logged into.  If we are calling via the XWB DIRECT RPC call, only
//    the first two pieces are returned.
//    3:  Station Number ^ (for notifying originating site) - if no station 
//        number is returned, then originating site is the same 
//    4:  Original IEN (for notifying originating site)
// -----------------------------------------------------------------------------
function TfrmVocRehabReturnRequest.SendReturnInfo : Boolean;
CONST   
  RPC_DIRECT = 'XWB DIRECT RPC'; 
  RPCNAME = 'DVBA VR REROUTE STATUS';
  RETURN = 'R';
  SUCCESS = '1';
  FAILURE = '0';
var
  Status, StatusMessage, StationNum, buffer : string;
begin
  Screen.Cursor := crHourGlass;
  Application.ProcessMessages;
  Result := FALSE;

  // Update local VistA with RETURN status
  frmMain.RPCBroker1.Results.Clear;
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
  frmMain.RPCBroker1.Param[0].Value := IntToStr(frmVRMedicalRequest.edtRequestNumber.Tag);  // Request IEN
  frmMain.RPCBroker1.Param[0].PType := literal;
  frmMain.RPCBroker1.Param[1].Value := RETURN;
  frmMain.RPCBroker1.Param[1].PType := literal;
  frmMain.RPCBroker1.Param[2].Value := memoReturnRequestReason.Text;
  frmMain.RPCBroker1.Param[2].PType := literal;   
  frmMain.RPCBrokerCall;
  Try
    frmMain.RPCBroker1.Call;
  Except // RPC Error - RPC failed, user exits
    On E: EBrokerError Do
    begin
      ShowMessageCAPRI(RPCNAME + ' could not be accessed!  '+#10#13+E.Message);
      Exit;  // allow user to try again or hit cancel
    end;
  End; // try
  
  If frmMain.RPCBroker1.Results.Count > 0 Then
  begin  // RPC returned results - process them
    Status := Piece(frmMain.RPCBroker1.Results[0], '^', 1); 
    StatusMessage := Piece(frmMain.RPCBroker1.Results[0], '^', 2);
    StationNum := Piece(frmMain.RPCBroker1.Results[0], '^', 3);
    OriginalIEN := Piece(frmMain.RPCBroker1.Results[0], '^', 4);
    if Status = SUCCESS then // Success, request was returned
    begin
      // Local VistA site was updated successfully.
      Application.ProcessMessages;

      // Update Originating site's status (same RPC called via WXB DIRECT RPC)
      frmMain.RPCBroker1.RemoteProcedure := RPC_DIRECT;
      frmMain.RPCBroker1.Param[0].PType := literal;
      frmMain.RPCBroker1.Param[0].Value := StationNum;
      frmMain.RPCBroker1.Param[1].PType := literal;
      frmMain.RPCBroker1.Param[1].Value := RPCNAME;
      frmMain.RPCBroker1.Param[2].PType := literal;
      frmMain.RPCBroker1.Param[2].Value := '';  // RPC version parameter (Optional)
      frmMain.RPCBroker1.Param[3].PType := literal;
      frmMain.RPCBroker1.Param[3].Value := OriginalIEN; 
      frmMain.RPCBroker1.Param[4].Value := RETURN;
      frmMain.RPCBroker1.Param[4].PType := literal;
      frmMain.RPCBroker1.Param[5].Value := memoReturnRequestReason.Text;
      frmMain.RPCBroker1.Param[5].PType := literal;   
      try
        frmMain.RPCBrokerCall;
        frmMain.RPCBroker1.Call;
        If frmMain.RPCBroker1.Results.Count > 0 Then
        begin // RPC returned results - process them
          Status := Piece(frmMain.RPCBroker1.Results[0], '^', 1);
          StatusMessage := Piece(frmMain.RPCBroker1.Results[0], '^', 2);  // debug only, not displaying
          // FYI: An email is sent to a mail group (DVBA C 2507 REROUTE) at the originating site indicating a return
          if Status = SUCCESS then
          begin
             Result := TRUE;
             frmMain.VocRehab.SendMailManMessage('RETURNED');  
          end
          else // if Status = FAILURE then  // Originating VistA site was not updated
          begin
             buffer := RPC_DIRECT + ' : ' + RPCNAME + #13#10 + 
                       ' did not update the 2507 Request Status correctly.  ' +
                       frmVRMedicalRequest.edtReRoutedFrom.Text + ' has not been notified.';
             ShowMessageCAPRI(buffer);
          end;
        end; // if RPCBroker1.Results.Count > 0
      except
        On E: EBrokerError Do
        begin
          Screen.Cursor := crDefault;
          ShowMessageCAPRI(RPCNAME + ' could not be accessed at Station Number ' + StationNum + '!'+#13#10+E.Message);
        end;
      end; // try except
    end // Status = SUCCESS
    else // Status = FAILURE
    begin
      buffer := RPCNAME + ' did not update the 2507 Request Status correctly.' +
                frmVRMedicalRequest.edtReRoutedFrom.Text + ' has not been notified.';
      ShowMessageCAPRI(buffer);
    end;
  end // RPCBroker1.Results.Count > 0
  else // RPC call returned no results
  begin
    buffer := RPCNAME + ' failed. ' + frmVRMedicalRequest.edtReRoutedFrom.Text + ' has not been notified.';
    ShowMessageCAPRI(buffer);
  end;
end;


end.
