unit clsVrsnChckr;

interface

uses Windows, Forms, Classes, SysUtils, Dialogs, CCOWRPCBroker, Fmcmpnts,
     Fmcntrls, MFunStr, dialogsCAPRI;

type
  TVrsnChk = class(TObject)
    private
    { Private Declarations }
      class function GetGUIVersionInfo(): TStringList;
      class function GetVistAVersionInfo(AppNme: String; Brkr: TCCOWRPCBroker): String;
      class procedure DsplyErrorMsg(IEN, Vrsn: String; FMMemo: TFMMemo; Brkr: TCCOWRPCBroker);
    public
    { Public Declaraions }
      class function IsAppVersionOk(AppNme: String; FMMemo: TFMMemo; Brkr: TCCOWRPCBroker): Boolean;
  end;

implementation

{ Public }

class function TVrsnChk.IsAppVersionOk(AppNme: String; FMMemo: TFMMemo; Brkr: TCCOWRPCBroker): Boolean;
var
  gVrsnInfo: TStringList;
  gVrsn, vVrsnInfo: String;
begin
  Result := True;
  gVrsnInfo := TStringList.Create;
  try
    vVrsnInfo := GetVistAVersionInfo(AppNme,Brkr);
    if SameText(UpperCase(Piece(vVrsnInfo, '*', 3)), 'N') then
    begin
      gVrsnInfo := GetGUIVersionInfo;
      try
        gVrsn := MFunStr.Piece(gVrsnInfo.Values['FileVersion'], '.', 1, 2);
        { Ensure GUI Version is greater than or equal to VistA Minimum Version }
        if (StrToFloat(gVrsn) < StrToFloat(Piece(vVrsnInfo, '*', 2))) then
        begin
          Result := False;
          DsplyErrorMsg(MFunStr.Piece(vVrsnInfo, '*') + ',', gVrsn, FMMemo, Brkr);
        end;  { End If Older Version of SMART in use }
      except  { Do Nothing on float conversion errors }
      end;  { End Try/Except Block }
    end;  { End If Minimum Version Required - Don't allow previous versions }
  finally
    gVrsnInfo.Free;
  end;  { End Try/Finally Block }
end;

{ Private }

class procedure TVrsnChk.DsplyErrorMsg(IEN: String; Vrsn: String; FMMemo: TFMMemo;
     Brkr: TCCOWRPCBroker);
var
  i: Integer;
  halt: Boolean;
  fmDescGets, fmOldGets: TFMGets;
  dsplyTxt, txtVal: String;
begin
  fmDescGets := TFMGets.Create(nil);
  fmOldGets := TFMGets.Create(nil);
  try
    fmOldGets := FMMemo.FMGets;
    fmDescGets.RPCBroker := Brkr;
    fmDescGets.IENS := IEN;
    fmDescGets.FieldNumbers.Add('1');
    fmDescGets.FileNumber := '396.18';
    FMMemo.FMGets := fmDescGets;
    try
      fmDescGets.GetAndFill;
    except
      on e : Exception do
      begin
        Application.MessageBox(PChar(e.message), 'Error', 0);
      end;
    end;
    if Not (FMMemo.Lines.Count = 0) then
    begin
      i := -1;
      halt := False;
      { Message to display will proceed all other text in WP field and
        will start with an asterik (*) }
      repeat
        inc(i);
        txtVal := FMMemo.Lines[i];
        { Get message to display - all initial lines will start with * }
        if (Pos('*', txtVal) = 1) then
        begin
          dsplyTxt := dsplyTxt + Piece(txtVal, '*', 2) + #13#10;
          if (i = 0) then  { Display current GUI Version used }
            dsplyTxt := StringReplace(dsplyTxt, '#', Vrsn, []);
        end  { End If 1st Char of Line does not contain '*' }
        else
          halt := True;
      until (halt or ((FMMemo.Lines.Count - 1) = i)) ;
      if SameText(dsplyTxt, '') then  { Default message if needed }
        dsplyTxt := 'This version (' + Vrsn + ') of demTRAN is no longer supported.' + #13#10 +
             'Please contact IRM for assistance.';
      MessageDlgCAPRI(dsplyTxt, mtError, [mbOK], 0);  { Display error message }
    end;  { End If Lines Returned }
  finally
    FMMemo.FMGets := fmOldGets;  { Restore Original FMGETS to control }
    FMMemo.Lines.Clear;
    fmDescGets.Free;
    fmOldGets.Free;
  end;  { End Try/Finally Block }
end;

class function TVrsnChk.GetGUIVersionInfo(): TStringList;
const
  n_Info = 10;
  InfoStr: array [1..n_Info] of String =
    ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName',
     'LegalCopyright', 'LegalTradeMarks', 'OriginalFilename',
     'ProductName', 'ProductVersion', 'Comments');
var
  Info: String;
  BuffSize,Len: Cardinal;
  i: Integer;
  Buff, Value: PChar;
begin
  Result := TStringList.Create;
  Info := Application.ExeName;
  BuffSize := GetFileVersionInfoSize(PChar(Info),BuffSize);
  if BuffSize > 0 then
  begin
    Buff := AllocMem(BuffSize);
    GetFileVersionInfo(PChar(Info),0,BuffSize,Buff);
    for i := 1 to n_Info do
      if VerQueryValue(Buff,PChar('StringFileInfo\040904E4\'+
                       InfoStr[i]), Pointer(Value), Len) then
        Result.Values[InfoStr[i]] := Value;  { Hash Table of Version Info }
    FreeMem(Buff,BuffSize);
  end  { End If Version Info Found }
end;

{ Returns: Record IEN * Version Number (Major.Minor) * Allow Previous Versions }
class function TVrsnChk.GetVistAVersionInfo(AppNme: String; Brkr: TCCOWRPCBroker): String;
var
  fmLstrVrsnInfo: TFMLister;
begin
  Result := '';
  fmLstrVrsnInfo := TFMLister.Create(nil);
  try
    fmLstrVrsnInfo.RPCBroker := Brkr;
    fmLstrVrsnInfo.FileNumber := '396.18';
    fmLstrVrsnInfo.FMIndex := 'B';
    fmLstrVrsnInfo.DisplayFields.Clear;
    fmLstrVrsnInfo.FieldNumbers.Clear;
    fmLstrVrsnInfo.PartList.Clear;
    fmLstrVrsnInfo.ListerFlags := [];
    fmLstrVrsnInfo.IENS := '';
    fmLstrVrsnInfo.Number := '*';
    fmLstrVrsnInfo.FieldNumbers.Add('10');  { Version Info Contained in CRC Field }
    fmLstrVrsnInfo.PartList.Add(AppNme);
    fmLstrVrsnInfo.GetList(nil);  { Retrieve Template Version Info }
    { Only 1 Entry should ever be returned }
    if (fmLstrVrsnInfo.Results.Count = 1) then
    begin
      Result := fmLstrVrsnInfo.Results[0];
      Result := Result + '*' +
           fmLstrVrsnInfo.GetRecord(Result).GetField('10').FMDBExternal;
    end;  { End If Single Entry Found }
  finally
    fmLstrVrsnInfo.Free;
  end;  { End Try/Finally Block }
end;

end.
