unit dlgVndrExmRtrvl;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ShellAPI, Math, DateUtils,
   // DONE: // CodeCR457 - LMS removed AnsiStrings in conversion to D2006 from XE2
  { VistA Components }
  MFunStr, ImgList,
  { SFTP Component }
  clsSFTPConn,
  { Secure BlackBox Components }
  SBSftpCommon, ExtCtrls;

type
  TvndrExmsRtrvlDlg = class(TForm)
    lstVwVndrFiles: TListView;
    btnOk: TButton;
    btnRefresh: TButton;
    imgLst: TImageList;
    btnCancel: TButton;
    tmrWndwClose: TTimer;
    procedure lstVwVndrFilesSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure btnRefreshClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure tmrWndwCloseTimer(Sender: TObject);
    procedure lstVwVndrFilesColumnClick(Sender: TObject; Column: TListColumn);
    procedure lstVwVndrFilesCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
  private
    { Private declarations }
    _SFTPConn: TSFTPConn;
    _VVAFleRtrvl, _SrtAscndng: Boolean;
    _ImgHash: TStringList;
    _CurClmn: Integer;
    procedure GetSysIcon(fleExt: String; dirFldr: Boolean = False);
    procedure PopFileList;
    procedure ResetWndwTmr;
    function AnsiRightStr( const Source : AnsiString; const Count : Integer ) : String;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent; VndrNme: String; VVAFleRtrvl: Boolean;
         SFTPConn: TSFTPConn; TmrVal: Cardinal); reintroduce;
    destructor Destroy; override;
  end;

var
  vndrExmsRtrvlDlg: TvndrExmsRtrvlDlg;

implementation

{$R *.dfm}

function FileListStrSort(Item1, Item2: Pointer): Integer;
var
  Info1, Info2 : TElSftpFileInfo;
begin
  Info1 := Item1;
  Info2 := Item2;
  if Info1.Attributes.Directory = Info2.Attributes.Directory then
    Result := CompareText(Info1.Name, Info2.Name)
  else
  begin
    if Info1.Attributes.Directory then
      Result := -1
    else
      Result := 1;
  end;  { End If Item types don't match }
end;

function FileListDateSort(Item1, Item2: Pointer): Integer;
var
  Info1, Info2 : TElSftpFileInfo;
begin
  Info1 := Item1;
  Info2 := Item2;
  if Info1.Attributes.Directory = Info2.Attributes.Directory then
    Result := CompareDateTime(Info1.Attributes.MTime, Info2.Attributes.MTime)
  else
  begin
    if Info1.Attributes.Directory then
      Result := -1
    else
      Result := 1;
  end;  { End If Item types don't match }
end;

constructor TvndrExmsRtrvlDlg.Create(AOwner: TComponent; VndrNme: String;
     VVAFleRtrvl: Boolean; SFTPConn: TSFTPConn; TmrVal: Cardinal);
begin
  inherited Create(AOwner);
  Caption := 'Vendor (' + VndrNme + ') Completed Exams';
  _CurClmn := 0;  { Default to Name as Initial Column }
  _SrtAscndng := True;  { Default is displayed in Alphabetical Order (Ascending) }
  _SFTPConn := SFTPConn;
  _VVAFleRtrvl := VVAFleRtrvl;  { If True, Screen on PDF, else Screen on TXT (ASCII) }
  _ImgHash := TStringList.Create;
  GetSysIcon('', True);  { Add System Folder Icon to Image List }
  PopFileList;
  tmrWndwClose.Interval := TmrVal;
end;

destructor TvndrExmsRtrvlDlg.Destroy;
begin
  _ImgHash.Free;
  inherited;
end;

procedure TvndrExmsRtrvlDlg.FormShow(Sender: TObject);
begin
  ResetWndwTmr;
  BringToFront;
  Repaint;
end;

function TvndrExmsRtrvlDlg.AnsiRightStr(const Source: AnsiString; const Count: Integer): String;
begin
  // DONE: LMS does syncommon have better/faster pre-written functions for this?
  if Length(source) > 0 then
  begin
    Result := Copy(Source, Length(Source) - Count + 1, Count);
  end else begin
    Result := '';
  end;
end;

procedure TvndrExmsRtrvlDlg.btnRefreshClick(Sender: TObject);
begin
  tmrWndwClose.Enabled := False;
  Screen.Cursor := crHourGlass;
  try
    try
      PopFileList;
    except
      on E: Exception do
      begin
        Application.MessageBox(PChar(E.Message), 'SFTP Error', MB_OK);
      end;  { End Exception Handling }
    end;  { End Inner Try/Except Block }
  finally
    ResetWndwTmr;
    Screen.Cursor := crDefault;
  end;  { End Outer Try/Finally Block }
end;

{ Private }

procedure TvndrExmsRtrvlDlg.GetSysIcon(fleExt: String; dirFldr: Boolean = False);
var
 fleInfo: TSHFileInfo;
 fleIcon: TIcon;
begin
  fleIcon := TIcon.Create;
  if Not dirFldr then
    SHGetFileInfo(PChar(fleExt), FILE_ATTRIBUTE_NORMAL, fleInfo, SizeOf(fleInfo),
         SHGFI_ICON or SHGFI_SMALLICON or SHGFI_USEFILEATTRIBUTES)
  else
  begin
    SHGetFileInfo(PChar(fleExt), FILE_ATTRIBUTE_DIRECTORY, fleInfo, SizeOf(fleInfo),
         SHGFI_ICON or SHGFI_SMALLICON or SHGFI_USEFILEATTRIBUTES);
    fleExt := '.FOLDER';  { Period is needed to correspond with file icon retrieval }
  end;  { End Else If Get Folder Icon }
  fleIcon.Handle := fleInfo.hIcon;
  _ImgHash.Values[Piece(fleExt, '.', 2)] := IntToStr(imgLst.AddIcon(fleIcon));
end;

procedure TvndrExmsRtrvlDlg.lstVwVndrFilesColumnClick(Sender: TObject;
  Column: TListColumn);
var
  tmpLst: TListView;
begin
  tmrWndwClose.Enabled := False;
  tmpLst := Sender as TListView;
  { User Column won't be displayed in demTRAN }
  if (Column.Index <> 3) and (lstVwVndrFiles.Items.Count > 0) then
  begin
    if _CurClmn = Column.Index then
      _SrtAscndng := Not _SrtAscndng
    else
    begin
      tmpLst.Columns[_CurClmn].ImageIndex := -1;  { Remove Image from Column }
      _CurClmn := Column.Index;
      _SrtAscndng := True;
    end;  { End Else if NEW column clicked }
    { Set Appropriate Sort Image denoting direction of sort }
    tmpLst.Columns[_CurClmn].ImageIndex := Integer(_SrtAscndng);
    tmpLst.Items.BeginUpdate;
    try
      tmpLst.AlphaSort;
    finally
      tmpLst.Items.EndUpdate;
    end;  { End Try/Finally Block }
  end;  { End If Column Index not for User }
  ResetWndwTmr;
end;

procedure TvndrExmsRtrvlDlg.lstVwVndrFilesCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
begin
  case _CurClmn of
    0: Compare := FileListStrSort(TElSftpFileInfo(Item1.Data),
                       TElSftpFileInfo(Item2.Data));  { NAME }
    1: begin
         Compare := CompareValue(TElSftpFileInfo(Item1.Data).Attributes.Size,
                         TElSftpFileInfo(Item2.Data).Attributes.Size);
         if (Compare = 0) then
           Compare := FileListStrSort(TElSftpFileInfo(Item1.Data),
                           TElSftpFileInfo(Item2.Data));
       end;  { End Column Index 1 - SIZE }
    2: begin
         Compare := FileListDateSort(TElSftpFileInfo(Item1.Data),
                         TElSftpFileInfo(Item2.Data));
         if (Compare = 0) then
           Compare := FileListStrSort(TElSftpFileInfo(Item1.Data),
                           TElSftpFileInfo(Item2.Data));
       end;  { End Column Index 2 _ MODIFIED DATE }
  end;  { End Case Select on Column Index }
  if Not _SrtAscndng then
    Compare := -Compare;
end;

procedure TvndrExmsRtrvlDlg.lstVwVndrFilesSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  ResetWndwTmr;
  btnOk.Enabled := (lstVwVndrFiles.SelCount > 0);
end;

procedure TvndrExmsRtrvlDlg.PopFileList;
var
  dirList: TList;
  fleScrn: String;
  i, fleSze: Integer;
  info: TElSftpFileInfo;
  item: TListItem;
begin
  lstVwVndrFiles.Items.Clear;
  btnOk.Enabled := False;
  Refresh;
  fleScrn := 'TXT';  { ASCII for VistA }
  if _VVAFleRtrvl then
    fleScrn := 'PDF';  { BINARY for VVA }
  try
    dirList := _SFTPConn.GetFileDirContents();
    dirList.Sort(FileListStrSort);  { Sort List By Name }
    for i := 0 to dirList.Count - 1 do
    begin
      info := TElSftpFileInfo.Create;
      TElSftpFileInfo(dirList.Items[i]).CopyTo(info);
      { Note: The returned ElSftpFileInfo object must be freed. }
      TElSftpFileInfo(dirList.Items[i]).Free;
      if (not info.Attributes.Directory) and
         (UpperCase(AnsiRightStr(info.Name,3)) = fleScrn) then
        // gbh   (SameText(UpperCase(Piece(info.Name, '.', 2)), fleScrn)) then
      begin
        { Add File Name to TListView - Name Column }
        item := lstVwVndrFiles.Items.Add;
        item.Data := info;
        item.Caption := info.Name;
        { Add File Size to TListView - Size Column}
        fleSze := info.Attributes.Size;
        fleSze := fleSze div 1024;
        if ((info.Attributes.Size mod 1024) > 0) then
          fleSze := fleSze + 1;
        item.SubItems.Add(IntToStr(fleSze) + ' KB');
        { Add Date File Last Modified - Modified Column }
        item.SubItems.Add(DateTimeToStr(info.Attributes.MTime));
        { Add Icon associated with File Type }
        if _ImgHash.IndexOfName(Piece(item.Caption, '.', 2)) > -1 then
          item.ImageIndex := StrToInt(_ImgHash.Values[(Piece(item.Caption, '.', 2))])
        else
        begin
          GetSysIcon(item.Caption);
          item.ImageIndex := StrToInt(_ImgHash.Values[(Piece(item.Caption, '.', 2))]);
        end;  { End Else ICON not currently in list }
        { Additional SubItems may be requested later: Ie. Size, Modified Date, Etc... }
      end;  { End If File is NOT a Directory and Matches the File Screen Type }
    end;  { End For Loop through Directory List }
  except
    on E: Exception do
    begin
      raise Exception.Create('Unable to retrieve file list from Vendors SFTP Site.' +
           #13#10#13#10 + E.Message);
    end;  { End Exception Handling }
  end;  { End Try/Except Block for Retrieving File List from Vendor's SFTP Site }
end;

procedure TvndrExmsRtrvlDlg.ResetWndwTmr;
begin
  tmrWndwClose.Enabled := False;
  tmrWndwClose.Enabled := True;
end;

procedure TvndrExmsRtrvlDlg.tmrWndwCloseTimer(Sender: TObject);
begin
  { Shutdown Contracted window to allow app to close - tmr is set just short of
    Main Application's Shutdown Timer value }
  Close;
end;

end.
