{
VocRehabReRouteRequest

Added Patch 197 JRL 2/1/17

This unit is called when a Voc Rehab 8861 Medical Request is re-routed.

RPCs called:
  DVBA CAPRI REROUTE VAMC
  DVBA VR REROUTEREASONS
  DVBA VR SEND REROUTE
  
}
unit formtest;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms
  ,Dialogs
  ,StdCtrls
  ,Trpcb
  ,VA508AccessibilityManager
  ,CAPRISupport
  ,dialogsCAPRI
  ;

type
  TfrmTest = class(TForm)
    lblReRouteReason: TLabel;
    cbReRouteReason: TComboBox;
    labelVAMC: TLabel;
    cbVAMC: TComboBox;
    lblSortByState: TLabel;
    cbxSortByState: TCheckBox;
    lblDescription: TLabel;
    memoReRouteDescription: TMemo;
    btnRerouteRequest: TButton;
    btnCancelReRoute: TButton;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cbxSortByStateClick(Sender: TObject);
    procedure btnCancelReRouteClick(Sender: TObject);
    procedure btnRerouteRequestClick(Sender: TObject);
  private
    { Private declarations }
    VAMC_IEN : String;
    VAMC_IEN_List : TStringList;
    VAMC_Station_List : TStringList;
    VAMC_Name_List : TStringList;
    VAMC_State_List : TStringList;
    ReRouteReasonIENList : TStringList;
    SelectedStation : String;
    SaveStationNumber : String;
    procedure GetVAMCLocations;
    procedure GetRerouteReasons;
    function  CheckUserInputValues: Boolean;
    function  CreateReRouteRequest : Boolean;
  public
    { Public declarations }
  end;

var
  frmTest: TfrmTest;

implementation

uses VocRehabMedicalRequest, main;

const
  CLOSE_FORM = '1';      // Patch 197 JRL 12/21/16
  DONT_CLOSE_FORM = '2'; // Patch 197 JRL 12/21/16

{$R *.dfm}


//------------------------------------------------------------------------------
// Form Create
//------------------------------------------------------------------------------
procedure TfrmTest.FormCreate(Sender: TObject);
begin
  // Create string lists
  VAMC_IEN_List := TStringList.Create;
  VAMC_Station_List := TStringList.Create;
  VAMC_Name_List := TStringList.Create;
  VAMC_State_List := TStringList.Create;
  ReRouteReasonIENList := TStringList.Create;
  // Populate initial fields
  GetReRouteReasons; // Populate Re-Routing Reasons dropdown
  GetVAMCLocations;  // Populate VAMC Locations dropdown
  SaveStationNumber := '';
end;


//------------------------------------------------------------------------------
// Form Close
//------------------------------------------------------------------------------
procedure TfrmTest.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // Destroy string lists
  FreeAndNil(VAMC_IEN_List);
  FreeAndNil(VAMC_Station_List);
  FreeAndNil(VAMC_Name_List);
  FreeAndNil(VAMC_State_List);
  FreeAndNil(ReRouteReasonIENList);
end;


//------------------------------------------------------------------------------
// Cancel Re-Route button Clicked
//------------------------------------------------------------------------------
procedure TfrmTest.btnRerouteRequestClick(Sender: TObject);
var
  request_rerouted : Boolean;
  CloseForm : Boolean;
begin
  if CheckUserInputValues then // if all inputs are satisfied, allow re-routing
  begin
     request_rerouted := CreateReRouteRequest;     // send the re-routed request
     if request_rerouted = TRUE then // re-route was successful
        frmVRMedicalRequest.ReturnValue := CLOSE_FORM
     else
        frmVRMedicalRequest.ReturnValue := DONT_CLOSE_FORM;
     Close;
  end;  // CheckUserInputValues
end;



//------------------------------------------------------------------------------
// Cancel Re-Route button Clicked
//------------------------------------------------------------------------------
procedure TfrmTest.btnCancelReRouteClick(Sender: TObject);
begin
  // No action needed except to close the form
  Close;
end;


//------------------------------------------------------------------------------
// Sort By State checkbox clicked
//------------------------------------------------------------------------------
procedure TfrmTest.cbxSortByStateClick(Sender: TObject);
begin
  GetVAMCLocations;
end;


//------------------------------------------------------------------------------
// Create Re-Route Request
//------------------------------------------------------------------------------
// RPC Input:    
//   Request IEN
//   Patient IEN
//   VAMC IEN (Station Number)
//   Routing Location IEN
//   Re-Route Reason IEN
//   Re-Route Description
//
// RPC Output:  
//   1^Transmitted as message # 909761 from this site to DEVVCC.DNS      URL        
//   0^Message transmission error! Request WILL NOT be rerouted
// -----------------------------------------------------------------------------
// FROM THE AMIE TRANSFER DOCUMENTATION:
//   If the mail message is sent out successfully, the user will see:
//   "Transmitted as message # {number} from this site to {site name}.VA.GOV."
// 
//   If the message fails (and therefore the transfer fails) the user will be so informed:
//   "Message transmission error!   Request WILL NOT be sent !"
// -----------------------------------------------------------------------------
function TfrmTest.CreateReRouteRequest : Boolean;
CONST
  RPCNAME = 'DVBA VR SEND REROUTE';
var
  MailmanRPCStatus : Integer;
  MailmanMessage : String;
  buffer : string;
begin
  Result := FALSE; // default to re-route failed
  try
    Screen.Cursor := crHourGlass;
    // Setup RPC
    frmMain.RPCBroker1.Results.Clear;
    frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
    frmMain.RPCBroker1.Param[0].Value := frmVRMedicalRequest.SelectedMedicalRequestIEN;  // Request IEN
    frmMain.RPCBroker1.Param[0].PType := literal;
    frmMain.RPCBroker1.Param[1].Value := PatientIEN;                              // Patient IEN
    frmMain.RPCBroker1.Param[1].PType := literal;
    frmMain.RPCBroker1.Param[2].Value := VAMC_IEN;
    frmMain.RPCBroker1.Param[2].PType := literal;
    frmMain.RPCBroker1.Param[3].Value := ReRouteReasonIENList[cbReRouteReason.ItemIndex];
    frmMain.RPCBroker1.Param[3].PType := literal;
    frmMain.RPCBroker1.Param[4].Value := memoReRouteDescription.Text;
    frmMain.RPCBroker1.Param[4].Ptype := literal;
    
    frmMain.RPCBrokerCall;
    Try
      frmMain.RPCBroker1.Call;
    Except
      On e: EBrokerError Do
      begin
        buffer := RPCNAME + ' could not be accessed!  '+#10#13 + E.Message + '.';
        ShowMessageCAPRI(buffer);
      end;
    End;
  finally
    Screen.Cursor := crDefault;
  end;

  // Check RPC results  
  If frmMain.RPCBroker1.Results.Count > 0 Then
  begin
    MailmanRPCStatus := StrToIntDef(Piece(frmMain.RPCBroker1.Results[0], '^', 1),0);
    MailmanMessage := Piece(frmMain.RPCBroker1.Results[0], '^', 2);
  
    if MailmanRPCStatus > 0 then // Success, request was re-routed
    begin
      // FYI: An email is sent to the original requester upon successful re-routing (all M code)
      ShowMessageCAPRI(MailmanMessage);  
      Result := TRUE;
      Close;
    end
  end;
  // if Re-Route message failed
  if not Result then
    ShowMessageCAPRI(RPCNAME + ' returned a failure.'+#10#13+MailmanMessage);
end;


//------------------------------------------------------------------------------
// Get VAMC Locations
//------------------------------------------------------------------------------
procedure TfrmTest.GetVAMCLocations;
var
  i : Integer;
CONST
  RPCNAME = 'DVBA CAPRI REROUTE VAMC';
  STRINGFORMAT = '%-28s%-15s%-6s';
begin
  // setup and call RPC
  frmMain.RPCBroker1.Results.Clear;
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;
  if cbxSortByState.Checked then
     frmMain.RPCBroker1.Param[0].Value := 'STATE'
  else
     frmMain.RPCBroker1.Param[0].Value := 'NAME';
  frmMain.RPCBroker1.Param[0].PType := literal;
  frmMain.RPCBrokerCall;
  Try
    frmMain.RPCBroker1.Call;
  Except
    On E: EBrokerError Do
      ShowMessageCAPRI(RPCNAME + ' could not be accessed!'+#13#10+E.Message);
  End;
  // clear all lists
  cbVAMC.Items.Clear;
  VAMC_Name_List.Clear;
  VAMC_State_List.Clear;
  VAMC_IEN_List.Clear;
  VAMC_Station_List.Clear;
  // populate all lists
  If frmMain.RPCBroker1.Results.Count > 0 Then
  begin
    For i := 0 To frmMain.RPCBroker1.Results.Count - 1 Do
    begin
      VAMC_Name_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 2));
      VAMC_State_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 1));
      VAMC_IEN_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 3));
      VAMC_Station_List.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 4));
    end;
  end;
  // Add data to listbox and format it into columns
  for i := 0 to VAMC_Name_List.Count - 1 do
    cbVAMC.Items.Add(Format(STRINGFORMAT,[VAMC_Name_List[i],VAMC_State_List[i],VAMC_Station_List[i]]));

end;


//------------------------------------------------------------------------------
// Get Re-Route Reasons
//------------------------------------------------------------------------------
// RPC:  DVBA CAPRI REROUTEREASONS
//   Input:   None
//   Output:  Re-Route Code^IEN
//------------------------------------------------------------------------------
procedure TfrmTest.GetReRouteReasons;
const
  RPCNAME = 'DVBA VR REROUTEREASONS';
var
  i : Integer;
begin
  frmMain.RPCBroker1.Results.Clear;                                         
  frmMain.RPCBroker1.RemoteProcedure := RPCNAME;          
  frmMain.RPCBrokerCall;                                                    
  try                                                                       
    frmMain.RPCBroker1.Call;                                                
  except                                                                    
    On EBrokerError Do                                                      
       ShowMessageCAPRI('RPC: '+RPCNAME + ' could not be accessed!'); 
  end;                                                                      
  cbReRouteReason.Items.Clear;                                     
  ReRouteReasonIENList := TStringList.Create;                          
  if frmMain.RPCBroker1.Results.Count > 0 then                              
  begin                                                                     
    for i := 0 To frmMain.RPCBroker1.Results.Count - 1 do                   
    begin                                                                   
      cbReRouteReason.Items.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 2));
      ReRouteReasonIENList.Add(Piece(frmMain.RPCBroker1.Results[i], '^', 1));
    end;                                                                   
  end;
end;
 
//------------------------------------------------------------------------------
// Check User Input Values
//------------------------------------------------------------------------------
// Check all user input values.  If values are input correctly, return TRUE.
// If values are missing, routine short circuits back to input form.
//------------------------------------------------------------------------------
function TfrmTest.CheckUserInputValues: Boolean;
begin
  if cbReRouteReason.Text = '' then
  begin
     ShowMessageCAPRI('Please enter a Re-Routing Reason.');
     Result := FALSE;
     exit;  // make user complete form before sending
  end;
  if cbReRouteReason.Text = 'OTHER' then
  begin
    if Length(memoReRouteDescription.Lines.Text) = 0 then
    begin
       ShowMessageCAPRI('Please enter a Re-Routing Description.');
       Result := FALSE;
       exit;  // make user complete form before sending
    end;
  end;
  if cbVAMC.ItemIndex = -1 then
  begin
    ShowMessageCAPRI('Please enter a VAMC.');
    Result := FALSE;
    exit;  // make user complete form before sending
  end;
  Result := TRUE;
end;
 


end.
