unit frm508ReportRslts;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls, ComCtrls, ExtCtrls, MFunStr, untAddtnlMMthds,
  untMiscMthds, SQLite3Pages, CCOWRPCBroker, dlgCntrctdExmRpts, Printers,
  Trpcb, DateUtils, StrUtils, ShellAPI;

type
  TfrmRptRslts = class(TForm)
    rchEdtRptRslts: TRichEdit;
    mnuRprts: TMainMenu;
    mnuItmFle: TMenuItem;
    N1: TMenuItem;
    mnuItmFleExit: TMenuItem;
    tmrWndwClose: TTimer;
    mnuItmFlePrntPrvw: TMenuItem;
    mnuItmFlePrnt: TMenuItem;
    N2: TMenuItem;
    mnuItmFlePDFExprt: TMenuItem;
    mnuItmFleHTMLView: TMenuItem;
    rchEdtTmpRptRslts: TRichEdit;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure mnuItmFlePrntPrvwClick(Sender: TObject);
    procedure mnuItmFleExitClick(Sender: TObject);
    procedure mnuItmFlePrntClick(Sender: TObject);
    procedure mnuItmFlePDFExprtClick(Sender: TObject);
    procedure mnuItmFleHTMLViewClick(Sender: TObject);
  private
    { Private declarations }
    _rptType, _HTMLPage: String;
    _rptObj: TGDIPages;
    _cntrctrRpt, _Landscape: Boolean;
    _DtldHdrInfo, _RptParams: TStringList;
    _CCOWBrkr: TCCOWRPCBroker;
    function GetHTMLCaption(): String;
    function GetReportType(): Integer;
    procedure AddTextToHdr(txtVal: String);
    procedure AddLineToHdr;
    procedure AddTxtToBdy(txtVal: String);
    procedure AddColTxtToBdy(txtVal: String; hdrVals: Boolean = False);
    procedure CopyInRslts(Rslts: TStrings);
    // procedure CreateHTMLDir;
    procedure ExportPDF;
    procedure GenerateHTMLPage;
    procedure GenerateReport;
    procedure GetReportData;
    procedure ResetWndwTmr;
    procedure RptBdy(RptObj: TGDIPages; CntrctrSorted: Boolean);
    procedure RptHdrFtr(RptObj: TGDIPages);
    procedure SetRptColTabs;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent; RptType: String; RptParams: TStringList;
         CCOWBrkr: TCCOWRPCBroker; TmrVal: Cardinal); reintroduce;
    destructor Destroy; override;
  end;

var
  frmRptRslts: TfrmRptRslts;

implementation

{$R *.dfm}

{ RptType : Detailed / Summary or Timeliness }
constructor TfrmRptRslts.Create(AOwner: TComponent; RptType: String;
     RptParams: TStringList; CCOWBrkr: TCCOWRPCBroker; TmrVal: Cardinal);
begin
  inherited Create(AOwner);
  _rptType := RptType;
  _CCOWBrkr := CCOWBrkr;
  _HTMLPage := GetTempExmDir( _CCOWBrkr.User.DUZ) + '\DtldRpt.html';
  _DtldHdrInfo := TStringList.Create;
  _RptParams := TStringList.Create;
  _RptParams.Assign(RptParams);
  _Landscape := (SameText(UpperCase(_RptType), 'DETAILED') or
       (Not (_RptParams.IndexOfName('"CONTRACTOR"') <> -1)));
  _cntrctrRpt := (_RptParams.IndexOfName('"CONTRACTOR"') <> -1);
  tmrWndwClose.Interval := TmrVal;
  { Set width of landscape form }
  if _Landscape then
  begin
    width := 1000;
    if SameText(_rptType, 'DETAILED') then
      if ((Not (_RptParams.IndexOfName('"CONTRACTOR"') <> -1)) and
           (Not (_RptParams.IndexOfName('HDRSORT') <> -1))) then
        Width := 1020
      else
        Width := 830;
  end;  { End If Report to be landscaped }
  { Setup Formatted Report Container }
  with rchEdtRptRslts Do
  begin
    Paragraph.Alignment := taLeftJustify;
    SelAttributes.Size := 8;
    SelAttributes.Style := [];
  end;  { End Modify rchEdtRptRslts }
  SetRptColTabs;
  { Enable HTML DETAILED Report Option - for 508 Accessibility Requirement }
  mnuItmFleHTMLView.Enabled := SameText(_rptType, 'DETAILED');
  GenerateReport;
end;

destructor TfrmRptRslts.Destroy;
begin
  _DtldHdrInfo.Free;
  _RptParams.Clear;
  _RptParams.Free;
  { Delete temporary HTML Doc File (Detailed Report) from user's system if it exists }
  if FileExists(_HTMLPage) then
    DeleteFile(_HTMLPage);
  inherited;
end;

procedure TfrmRptRslts.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  tmrWndwClose.Enabled := False;
end;

procedure TfrmRptRslts.FormShow(Sender: TObject);
begin
  Caption := _rptType + ' Exam Report';
  ResetWndwTmr;
end;

procedure TfrmRptRslts.mnuItmFlePrntPrvwClick(Sender: TObject);
begin
  Hide;  { Hide 508 Report form while showing print preview }
  _rptObj.ShowPreviewForm;
  Show;  { Show 508 Report form }
end;

procedure TfrmRptRslts.mnuItmFlePrntClick(Sender: TObject);
begin
  ResetWndwTmr;
  _rptObj.PrintPages(-1,-1);  { Print from GDI NOT 508 Display }
end;

procedure TfrmRptRslts.mnuItmFlePDFExprtClick(Sender: TObject);
begin
  ResetWndwTmr;
  ExportPDF;
end;

procedure TfrmRptRslts.mnuItmFleHTMLViewClick(Sender: TObject);
var
  BrkrErr: Boolean;
  rslt: Cardinal;
begin
  BrkrErr := False;
  Screen.Cursor := crHourGlass;
  tmrWndwClose.Enabled := False;  { Disable Timer while retrieving Results from VistA }
  _RptParams.Add('"DELIMITTED"=1');  { Add new Filter to current RPC Parameters }
  try
    try
      GetReportData;  { Delimited Data to create HTML Table }
    except
      on E: Exception do
      begin
        Application.MessageBox(PChar(E.Message), 'Broker Error', MB_OK);
        BrkrErr := True;
      end;  { End Exception Handling }
    end;  { End Try/Except Block for Retrieving delimited report data }
    if Not BrkrErr then
    begin
      GenerateHTMLPage;  { 508 Accessibility for Detailed Tablular Report }
      if (FileExists(_HTMLPage)) then
      begin
        // TODO: LMS change PWideChar to PChar (PAnsiChar)..  rslt := ShellExecute(Handle, 'open', PWideChar(_HTMLPage), nil, nil, SW_SHOWNORMAL);
        rslt := ShellExecute(Handle, 'open', PAnsiChar(_HTMLPage), nil, nil, SW_SHOWNORMAL);
        if (rslt <= 32) then
          Application.MessageBox(PChar('Unable to open the HTML web page.' +
               #13#10#13#10 + SysErrorMessage(rslt)), 'HTML Display Error',
               MB_OK);  { Function GetLastError could be used instead of rslt }
      end;  { End If HTML File Exists }
    end;  { End If No Broker Errors }
  finally
    Screen.Cursor := crDefault;
    ResetWndwTmr;
  end;  { End Try/Finally Block }
end;

procedure TfrmRptRslts.mnuItmFleExitClick(Sender: TObject);
begin
  Close;
end;

{ Functions / Procedures }

{$REGION 'Write To RichEdit'}

procedure TfrmRPtRslts.AddTextToHdr(txtVal: string);
begin
  if SameText(UpperCase(_rptType), 'DETAILED') and
       (_CCOWBrkr.Results.Count > 0) then
    _DtldHdrInfo.Add(txtVal)
  else
    with rchEdtRptRslts Do
    begin
      Paragraph.Alignment := Classes.taCenter; // DONE: LMS TAlignment.taCenter; Has to be prefixed with Classes instead of refd with TAlignment. Otherwise also ends up referring sqllite3.tacenter. Only did it for taCenter but not tLeftJustify
      SelAttributes.Size := 12;
      SelAttributes.Style := [fsBold];
      Lines.Add(txtVal);
    end;  { End Modify rchEdtRptRslts }
end;

procedure TfrmRptRslts.AddLineToHdr;
var
  pLen: Integer;
begin
  pLen := 50;
  if _Landscape then
    pLen := 80;
  if SameText(UpperCase(_rptType), 'DETAILED') and
       (_CCOWBrkr.Results.Count > 0) then
  begin
    _DtldHdrInfo.Add(StringofChar('-', pLen));
    _DtldHdrInfo.Add('');
  end  { End If Detailed Report }
  else
    with rchEdtRptRslts Do
    begin
      Paragraph.Alignment := Classes.taCenter; // DONE: LMS TAlignment.taCenter; Has to be prefixed with Classes instead of refd with TAlignment. Otherwise also ends up referring sqllite3.tacenter. Only did it for taCenter but not tLeftJustify
      SelAttributes.Size := 8;
      SelAttributes.Style := [];
      Lines.Add(StringofChar('-', pLen));
      Lines.Add(#13#10);
    end;  { End Modify rchEdtRptRslts }
end;

procedure TfrmRptRslts.AddColTxtToBdy(txtVal: String; hdrVals: Boolean = False);
begin
  with rchEdtRptRslts Do
  begin
    SelStart := GetTextLen;
    Paragraph.Alignment := taLeftJustify;
    SelAttributes.Size := 8;
    SelAttributes.Style := [];
    if hdrVals then
      SelAttributes.Style := [fsBold];
    { Replace '^' delimitter with tabs for column display on report }
    SelText := StringReplace(txtVal, '^', #9, [rfReplaceAll]) + #13#10;
  end;  { End Modify rchEdtRptRslts }
end;

procedure TfrmRptRslts.AddTxtToBdy(txtVal: string);
begin
  with rchEdtRptRslts Do
  begin
    Paragraph.Alignment := taLeftJustify;
    Paragraph.FirstIndent := 5;
    SelAttributes.Size := 8;
    SelAttributes.Style := [];
    Lines.Add(txtVal);
  end;  { End Modify rchEdtRptRslts }
end;

procedure TfrmRptRslts.CopyInRslts(Rslts: TStrings);
var
  i: Integer;
begin
  { Insert Report Header into Results before copying in }
  if (_DtldHdrInfo.Count > 0) then
    for i := 0 to _DtldHdrInfo.Count - 1 do
      Rslts.Insert(i, _DtldHdrInfo[i]);
  QuickCopy(Rslts, rchEdtRptRslts);
  { Update Font Attributes for Inserted Header Info }
  with rchEdtRptRslts do
  begin
    for i := 0 to _DtldHdrInfo.Count - 1 do
    begin
      SelStart := Perform(EM_LINEINDEX,i,0);
      SelLength := Length(Lines[i]);
      Paragraph.Alignment := Classes.taCenter; // DONE: LMS TAlignment.taCenter; Has to be prefixed with Classes instead of refd with TAlignment. Otherwise also ends up referring sqllite3.tacenter. Only did it for taCenter but not tLeftJustify
      SelAttributes.Size := 12;
      SelAttributes.Style := [fsBold];
      if (i >= (_DtldHdrInfo.Count - 2)) then
      begin
        SelAttributes.Size := 8;
        SelAttributes.Style := [];
      end;
      SelText := Lines[i];
    end;
  end;
end;

procedure TfrmRptRslts.SetRptColTabs;
begin
  with rchEdtRptRslts Do
  begin
    { Detailed Report is setup in M and pasted directly into document.
      Tab Stops are set for Courier New Font @ 8 pitch on Landscape layout. }
    if SameText(UpperCase(_rptType), 'SUMMARY') and Not _cntrctrRpt then
    begin
      Paragraph.TabCount := 3;
      Paragraph.Tab[0] := 167;
      Paragraph.Tab[1] := 519;
      Paragraph.Tab[2] := 603;
    end  { End If Summary Report }
    else if SameText(UpperCase(_rptType), 'TIMELINESS') and Not _cntrctrRpt then
    begin
      Paragraph.TabCount := 2;
      Paragraph.Tab[0] := 167;
      Paragraph.Tab[1] := 583;
    end;  { End If Timeliness Report }
  end;  { End Modify rchEdtRptRslts }
end;

{$ENDREGION}

procedure TfrmRptRslts.GetReportData;
var
  i: Integer;
const
  RPC: String = 'DVBAD CONTRACTED EXAM REPORTS';
begin
  _CCOWBrkr.Results.Clear;
  _CCOWBrkr.RemoteProcedure := ShortString(RPC);
  _CCOWBrkr.Param[0].Value := UpperCase(Copy(_RptType, 0, 1));
  _CCOWBrkr.Param[0].PType := literal;
  { Add Applicable Parameter Array Values to RPC Call }
  for i := 0 to _RptParams.Count - 1 do
    if Not AnsiStartsStr('HDR', _RptParams.Names[i]) then
      _CCOWBrkr.Param[1].Mult[_RptParams.Names[i]] := _RptParams.ValueFromIndex[i];
  _CCOWBrkr.Param[1].PType := list;
  try
    _CCOWBrkr.Call;
  except
    on E: EBrokerError do
    begin
      raise Exception.Create('Broker Error: RPC ' + RPC + ' could NOT be accessed.' + #13#10#13#10 +
           E.Message);
    end;   { End OnEBrokerError }
  End;  { End Try/Except Block }
end;

procedure TfrmRptRslts.GenerateReport;
var
  CntrctrFltrd: Boolean;
begin
  GetReportData; { Setup RPC and Get data for report from VistA }
  _rptObj := TGDIPages.Create(self);
  CntrctrFltrd := (_RptParams.IndexOfName('"CONTRACTOR"') <> -1);
  with _rptObj do
  begin
    { Orientation is global for entire report - Currently No mixing allowed
      Display as Landscape if Detailed Report of if Contractor Filter Enabled }
    if SameText(UpperCase(_RptType), 'DETAILED') or (Not CntrctrFltrd) then
      _rptObj.Orientation := Printers.poLandscape;
    Caption := _RptType + ' Contracted Exam Report';
    BeginDoc;  { Open report document for editing }
    PageMargins := Rect(10,10,10,10);
    SaveLayout;
    { 508Rpt is for viewing, RptGDI is for printing/exporting }
    RptHdrFtr(_rptObj);  { Header and Footer }
    RptBdy(_rptObj, CntrctrFltrd);  { Main Body Content }
    EndDoc;  { Close report document }
  end;  { End With Report Object }
end;

procedure TfrmRptRslts.RptHdrFtr(RptObj: TGDIPages);
var
  hdrTxt: String;
begin
  with RptObj do
  begin
    Font.Name := 'Georgia';
    Font.Size := 14;
    Font.Style := [fsBold];
    TextAlign := taCenter;
    { Report Title }
    if (_RptParams.IndexOfName('"PENDING"') <> -1) then
      hdrTxt := _RptType + ' Pending Contracted Exam Report'
    else
      hdrTxt := _RptType + ' Referred Exam Report';
    AddTextToHeaderAt(hdrTxt, RightMarginPos shr 1);
    AddTextToHeader(#13#10);
    AddTextToHdr(hdrTxt);
    { Specific Contrator Report }
    if (_RptParams.IndexOfName('"CONTRACTOR"') <> -1) then
    begin
      hdrTxt := 'for ' + _RptParams.Values['HDRCONTRACTOR'];
      AddTextToHeaderAt(hdrTxt, RightMarginPos shr 1);
      AddTextToHeader(#13#10);
      AddTextToHdr(hdrTxt);
    end;  { End If Contractor Filter Enabled }
    { Pending X Number of days - Only shown if Pndg Exams Check-Box Checked }
    if (_RptParams.IndexOfName('"PENDING"') <> -1) and
         (_RptParams.IndexOfName('HDRPNDGDAYS') <> -1) then
    begin
      hdrTxt := 'exceeding ' + _RptParams.Values['HDRPNDGDAYS'] + ' days';
      AddTextToHeaderAt(hdrTxt, RightMarginPos shr 1);
      AddTextToHeader(#13#10);
      AddTextToHdr(hdrTxt);
    end;  { End If Pending Days Exceeded Filter Enabled }
    if (_RptParams.IndexOfName('HDRDATE') <> -1) then
    begin
      hdrTxt := _RptParams.Values['HDRDATE'];
      AddTextToHeaderAt(hdrTxt, RightMarginPos shr 1);
      AddTextToHdr(hdrTxt);
    end;  { End Date Filter Enabled }
    Font.Size := 30;  { To prevent strike though of header }
    Font.Style := [];
    AddLineToHeader(True);
    RestoreSavedLayout;
    TextAlign := taRight;
    Font.Size := 11;
    AddLineToFooter(False);
    AddPagesToFooterAt('Page %d/%d', RightMarginPos);
    TextAlign := taLeft;
    AddTextToFooter(DateTimeToStr(Now));
    AddLineToHdr;
  end;  { End Header & Footer Modifications for Report Object }
end;

procedure TfrmRptRslts.RptBdy(RptObj: TGDIPages; CntrctrSorted: Boolean);
var
  i, intRptType: Integer;
  val: String;
begin
  intRptType := GetReportType;
  with RptObj do
  begin
    RestoreSavedLayout;
    Font.Name := 'Courier New';  { Monospaced font }
    Font.Size := 12;
    NewHalfLine;
    WordWrapLeftCols := True;
    if (_CCOWBrkr.Results.Count <> 0) then   { Create document if results returned }
    begin
      { Create columns for report body content }
      if CntrctrSorted then
        case intRptType of
          { Detailed Report Columns displayed in header - Report is in Landscape }
          0 : RptObj.AddTextToHeader(#13#10 + _CCOWBrkr.Results[0]);
          { Note: Summary and Timeliness will be single line items on report
                  and will be in Potrait. }
        end  { End Case Selection }
      else
        case intRptType of  { Report will be configured for landscape here }
          { Detailed Report Columns }
          //0 : RptObj.AddTextToHeader(#13#10 + _CCOWBrkr.Results[0]);
          1 : begin  { Summary Report Columns }
                AddColumns([25,41,17,17]);  { % of area }
                SetColumnAlign(2, caCenter);
                SetColumnAlign(3, caCenter);
                AddColumnHeaders(['Contractor','Contractor Email','Exams Referred',
                     'Requests Referred'], True, True);
                AddColTxtToBdy('Contractor^Contractor Email^Exams Referred' +
                     '^Requests Referred', True);
              end;  { End If Summary Type Report }
          2 : begin  { Timeliness Report Columns }
                AddColumns([30,45,25]);  { % of area }
                SetColumnAlign(2, caCenter);
                AddColumnHeaders(['Contractor','Contractor Email',
                     'Avg Timeliness (Days)'], True, True);
                AddColTxtToBdy('Contractor^Contractor Email^Avg ' +
                     'Timeliness (Days)', True);
              end;  { End If Timeliness Type Report }
        end;  { End Case Selection }
      { Add RPC Retrieved Report Content }
      case intRptType of
        0 : begin  { Detailed Report }
              QuickCopy(_CCOWBrkr.Results, rchEdtTmpRptRslts);
              RptObj.AppendRichEdit(rchEdtTmpRptRslts.Handle);
              CopyInRslts(_CCOWBrkr.Results);
            end  { End Detailed Report Selection }
      else begin  { Summary and Timeliness Reports }
             if CntrctrSorted then
             begin
               for i := 0 to 4 do  { Line Spacing }
               begin
                 AddTxtToBdy('');
                 RptObj.DrawText('');
               end;
               case intRptType of
                 1 : begin  { Summary Report }
                       val := 'Total number of examinations referred to : ' +
                            Piece(_CCOWBrkr.Results[0], '^', 3);
                       AddTxtToBdy(val);
                       RptObj.DrawText(val);
                       val := 'Total number of 2507 requests referred to: ' +
                            Piece(_CCOWBrkr.Results[0], '^', 4);
                       AddTxtToBdy(val);
                       RptObj.DrawText(val);
                     end;  { End Case Summary Report Selection }
                 2 : begin  { Timeliness Report }
                       val := 'Average Calculated Timeliness Value: ' +
                            Piece(_CCOWBrkr.Results[0], '^', 3) + ' days';
                       AddTxtToBdy(val);
                       RptObj.DrawText(val);
                     end;  { End Case Timeliness Report Selection }
               end;  { End Case Selection }
             end  { End If Report IS for specific Contractor }
             else
             begin
               for i := 0 to _CCOWBrkr.Results.Count - 1 do
               begin
                 val := _CCOWBrkr.Results[i];
                 AddColTxtToBdy(val);  { Add to Rich Text Edit }
                 RptObj.DrawTextAcrossCols(StringToArray(val, '^'));  { Printable }
               end;  { End For Loop of RPC Results }
             end;  { End Else If Report NOT for specific Contractor }
           end;  { Summary and Timeliness Reports }
      end;  { End Case Selection }
    end  { End If Results passed back from RPC Call }
    else
    begin
      val := #13#10#13#10#13#10#13#10#13#10 + 'No results were returned.';
      AddTxtToBdy(val);
      RptObj.DrawText(val);
    end;  { End Else If No Results Passed Back }
  end;  { End Main Body Content Modification for Report Object }
end;

function TfrmRptRslts.GetReportType(): Integer;
begin
  Result :=0;  { Detailed Report }
  if SameText(UpperCase(_RptType), 'SUMMARY') then
    Result := 1;
  if SameText(UpperCase(_RptType), 'TIMELINESS') then
    Result := 2;
end;

{$REGION 'HTML Processes'}

procedure TfrmRptRslts.GenerateHTMLPage;
var
  i, r, itms: Integer;
  strVal: String;
  HTMLDoc: TStringList;
begin
  HTMLDoc := TStringList.Create;
  try
    HTMLDoc.Add('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" ' +
         '"http://www.w3.org/TR/html4/loose.dtd">');
    HTMLDoc.Add('<html>');
    HTMLDoc.Add('<head>');
    HTMLDoc.Add('<title>' + _rptType + ' Exam Report</title>');
    HTMLDoc.Add('</head>');
    HTMLDoc.Add('<body bgcolor="#FFFAF0">');
    if (_CCOWBrkr.Results.Count > 1) then  { Don't Count Header Row }
    begin
      HTMLDoc.Add('<table border="1" cellspacing="1" cellpadding="3" ' +
           'summary="External Contracted ' + _rptType + ' Exam Report">');
      HTMLDoc.Add('<caption><b>' + GetHTMLCaption() + '</b></caption>');
      HTMLDoc.Add('<thead>');
      HTMLDoc.Add('<tr>');
      { Add Column Headers }
      strVal := _CCOWBrkr.Results[0];
      itms := Occurrences('^', strVal);
      for i := 0 to itms - 1 do
        HTMLDoc.Add('<th scope="col">' + Piece(strVal, '^', (i + 1)) + '</th>');
      HTMLDoc.Add('</tr>');
      HTMLDoc.Add('</thead>');
      HTMLDoc.Add('<tbody>');
      { Add Table Row Data Contents }
      for i := 1 to _CCOWBrkr.Results.Count - 1 do
      begin
        strVal := _CCOWBrkr.Results[i];
        HTMLDoc.Add('<tr>');
        for r := 0 to itms - 1 do
          if SameText(Piece(strVal, '^', (r + 1)), '') then
            HTMLDoc.Add('<td>&nbsp;</td>')
          else
            HTMLDoc.Add('<td>' + Piece(strVal, '^', (r + 1)) + '</td>');
        HTMLDoc.Add('</tr>');
      end;  { End Outer For Loop through Broker Results }
      HTMLDoc.Add('</tbody>');
      HTMLDoc.Add('</table>');
    end  { End If Results Returned - Table Created }
    else
    begin
      HTMLDoc.Add('<h2 style="text-align: center"><b>' + GetHTMLCaption() + '</b></h2>');
      HTMLDoc.Add('<hr/>');
      HTMLDoc.Add('<h3><br/><br/><br/><br/><br/>No results were returned.</h3>');
    end;  { End If No Results Returned }
    HTMLDoc.Add('</body>');
    HTMLDoc.Add('</html>');
    try
      // CreateHTMLDir; // DONE: LMS - Refactored - GetTempExmDir does this.
      HTMLDoc.SaveToFile(_HTMLPage);  { Save to Temporary Directory for Viewing }
    except
      on E: Exception do
      begin
        Application.MessageBox(PChar('Unable to save HTML document.' + #13#10#13#10 +
             E.Message), 'HTML File Error', MB_OK);
      end;  { End Exception Handling }
    end;  { End Try/Except Block }
  finally
    HTMLDoc.Free;
  end;
end;

function TfrmRptRslts.GetHTMLCaption(): String;
const
  MRGN: String = '<p style="margin-top: 0px; margin-bottom: 4px;">';
begin
  { Report Title }
  if (_RptParams.IndexOfName('"PENDING"') <> -1) then
    Result := MRGN + _RptType + ' Pending Contracted Exam Report</p>'
  else
    Result := MRGN + _RptType + ' Referred Exam Report</p>';
  { Specific Contrator Report }
  if (_RptParams.IndexOfName('"CONTRACTOR"') <> -1) then
    Result := Result + MRGN + 'for ' + _RptParams.Values['HDRCONTRACTOR'] + '</p>';
  { Pending X Number of days - Only shown if Pndg Exams Check-Box Checked }
  if (_RptParams.IndexOfName('"PENDING"') <> -1) and
       (_RptParams.IndexOfName('HDRPNDGDAYS') <> -1) then
    Result := Result + MRGN + 'exceeding ' + _RptParams.Values['HDRPNDGDAYS'] + ' days</p>';
  if (_RptParams.IndexOfName('HDRDATE') <> -1) then
    Result := Result + MRGN + _RptParams.Values['HDRDATE'] + '</p>';
end;

(* // DONE: LMS refactored out
procedure TfrmRptRslts.CreateHTMLDir;
var
  tDir: String;
begin
  tDir := GetTempDir;
  if (Not DirectoryExists(tDir + 'demTranTemp\' + _CCOWBrkr.User.DUZ)) then
  begin
    { Create SubDirectory for File Attachements }
    CreateDir(tDir + 'demTranTemp\' + _CCOWBrkr.User.DUZ);
    if (Not DirectoryExists(tDir + 'demTranTemp\' + _CCOWBrkr.User.DUZ)) then
      ForceDirectories(tDir + 'demTranTemp\' + _CCOWBrkr.User.DUZ);
  end;  { End If Directory does not exists }
end;
*)

{$ENDREGION}

procedure TfrmRptRslts.ExportPDF;
begin
  with _rptObj do
  begin
    ExportPDFApplication := Caption;
    ExportPDFAuthor := _CCOWBrkr.User.Name;
    ExportPDFSubject := _rptType + ' Report';
    ExportPDFKeywords := _rptType + ', Report, Contracted, demTran';
    ExportPDFEmbeddedTTF := True;
    ExportPDF('', True, False);
  end;
end;

procedure TfrmRptRslts.ResetWndwTmr;
begin
  tmrWndwClose.Enabled := False;
  tmrWndwClose.Enabled := True;
end;

end.
