unit reportbld;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  DialogsCAPRI,
  Dialogs,
  StdCtrls,
  ORCtrls,
  ComCtrls,
  Buttons,
  GIFImage,
  Gauges,
  ExtCtrls,
  Trpcb,
  CCOWRPCBrokerCAPRI, ActnList, ToolWin, ActnMan, ActnCtrls, ActnMenus,
  XPStyleActnCtrls;

type
  TFormReportBuilder = class(TForm)
    ORListBoxItems: TORListBox;
    orBtndeleteReport: TORAlignButton;
    ORbtnPrintReport: TORAlignButton;
    ORbtnAddCurrent: TORAlignButton;
    ORbtnClear: TORAlignButton;
    memoDocs: TRichEdit;
    btnMoveUp: TBitBtn;
    btnmovedown: TBitBtn;
    Gauge1: TGauge;
    ORAlignButtonAddAll: TORAlignButton;
    ORAlignButtonAddStop: TORAlignButton;
    TimerAddAll: TTimer;
    BitBtnCancel: TBitBtn;
    ORbtnSend2VVA: TORAlignButton;
    procedure ORbtnAddCurrentClick(Sender: TObject);
    procedure ORbtnClearClick(Sender: TObject);
    procedure orBtndeleteReportClick(Sender: TObject);
    procedure ORbtnPrintReportClick(Sender: TObject);
    procedure ORListBoxItemsClick(Sender: TObject);
    procedure ORListBoxItemsDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure ORListBoxItemsEndDrag(Sender, Target: TObject; X,
      Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure btnmovedownClick(Sender: TObject);
    procedure btnMoveUpClick(Sender: TObject);
    procedure ORAlignButtonAddAllClick(Sender: TObject);
    procedure ORAlignButtonAddStopClick(Sender: TObject);
    procedure TimerAddAllTimer(Sender: TObject);
    procedure BitBtnCancelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ORbtnSend2VVAClick(Sender: TObject);
  private
    function  AddReportToList(ReportIndex: integer): boolean;
    procedure AfterAddCurrentClick;
    procedure AddBlankReport;
  public
    { Public declarations }
  end;

var
  FormReportBuilder: TFormReportBuilder;

implementation

uses main;

{$R *.DFM}

{==============================================================================
  AddReportToList
    AddReportToList is a function that will add a report or document selected in
      frmMain.LstDocs to the report builder listbox.
    First, it builds the item string.  If the string is valid (not empty), it
      will then check to see if the item is in the list already and set a flag.
      Assuming the flag is set to include, the itemIndex is set in LstDocs and
      the event handler is called which will get the document displayed in
      frmMain.memoDocs.  The report is added to the report builder list of
      reports and the report text is appended to this form's memoDocs.

  Parameters:
    ReportIndex is an integer representing the index into frmMain.LstDocs.Items
      of the report item to be added

  Return value:
    AddReportToList returns a boolean.  If a report is added to the list, it
      returns TRUE, otherwise it returns FALSE.

  Modification History:
    CodeCR126 - 7/2010 Method created -MER
    CodeCR167 - 3/2011 Change to a function returning a boolean -MER
 ==============================================================================}
function TFormReportBuilder.AddReportToList(ReportIndex: integer): boolean;
var
  x: integer;
  newitem: string;
  includeThisOne: boolean;
begin
  dvbaz := ReportIndex;
  result := FALSE; // CodeCR167 -MER
  // build the item string
  newitem := '';
  if dvbaz > -1 then begin
    newitem := IntToStr(frmMain.Tab95Control1.TabIndex) + '^' + Doctype + ', ' + frmMain.LstDocs.Items[dvbaz];
  end else if (frmMain.LstDocs.ItemIndex = -1) and (DocType <> 'MULTI') then begin
    // note: this branch should not occur, but to be safe it has been left intact
    newitem := IntToStr(frmMain.Tab95Control1.TabIndex) + '^' + Doctype;
  end;
  if newitem = '' then begin
    includeThisOne := FALSE;
  end else begin
    // Make sure it's not already in the list
    includeThisOne := TRUE;
    for x := 0 to ORListBoxItems.Items.Count - 1 do begin
      if newitem = ORListBoxItems.Items[x] then begin
        includeThisOne := FALSE;
        break;
      end;
    end;
  end; { if newitem = '' then }
  if (includeThisOne) then begin
    // select the item and call the click event handler to get the document
    // displayed in frmMain.memoDocs
    frmMain.LstDocs.ItemIndex := dvbaz;
    frmMain.lstDocsClick(Application);
    // Add it
    ORListBoxItems.Items.Add(newitem);
    //Copy text
    if frmMain.memoDocs.Lines.Count > 0 then begin
      memodocs.lines.add('*' + newitem);
      memodocs.lines.add('-------------------------------------------------------------------------------');
      for x := 0 to frmMain.memodocs.lines.count - 1 do
        memodocs.lines.add(frmMain.memodocs.lines[x]);
      memodocs.lines.add('');
      memodocs.lines.add('^*end');
    end;
    result := TRUE; // CodeCR167 -MER
  end; // if (IncludeThisOne) then
end; // AddReportToList

{==============================================================================
  AfterAddCurrentClick
    AfterAddCurrentClick is a method that will configure UI elements after
      processing the ORbtnAddCurrent OnClick event handler.  It was refactored
      into a procedure to avoid repitition of code

  Modification History:
    CodeCR126 - 7/2010 Method created -MER
 ==============================================================================}
procedure TFormReportBuilder.AfterAddCurrentClick;
begin
  ORListBoxItems.ItemIndex := ORListBoxItems.Items.Count - 1;
  ORListBoxItems.ItemIndex := -1;

  orBtndeleteReport.enabled := False;
  btnMoveDown.Enabled := False;
  btnMoveUp.Enabled := False;
  orbtnClear.Enabled := True;
  orbtnPrintReport.Enabled := True;
  orbtnSend2VVA.Enabled := False;  //CodeCR353 - rpm 6/26/12
  frmMain.AnimateLogo(False);

  // In order for the add current button to be re-enabled, dvbaz has to be set to 0
  dvbaz := 0;
end;

{==============================================================================
  AddBlankReport
    AddBlankReport is a method that will add a section indicating the lack of
      a particular report type to the ReportBuilder composite report

  Modification History:
    CodeCR126 - 7/2010 Method created -MER
 ==============================================================================}
procedure TFormReportBuilder.AddBlankReport;
var
  newItem: string;
  i: integer;
  includeThisOne: boolean;
begin
  orbtnPrintReport.Enabled := False;
  orbtnSend2VVA.Enabled := False;  //CodeCR353 - rpm 6/26/12
  screen.cursor := crHourglass;

  newitem := IntToStr(frmMain.Tab95Control1.TabIndex) + '^' + Doctype;
  // Make sure it's not already in the list
  includeThisOne := TRUE;
  for i := 0 to ORListBoxItems.Items.Count - 1 do begin
    if newitem = ORListBoxItems.Items[i] then begin
      includeThisOne := FALSE;
      break;
    end;
  end;
  if (includeThisOne) then begin
    // Add it to the reportBuilder list
    ORListBoxItems.Items.Add(newitem);
    //Copy text
    if frmMain.memoDocs.Lines.Count > 0 then begin
      memodocs.lines.add('*' + newitem);
      memodocs.lines.add('-------------------------------------------------------------------------------');
      for i := 0 to frmMain.memodocs.lines.count - 1 do
        memodocs.lines.add(frmMain.memodocs.lines[i]);
      memodocs.lines.add('');
      memodocs.lines.add('^*end');
    end;
  end;
  screen.cursor := crDefault;
  AfterAddCurrentClick;
end; // AddBlankReport

{==============================================================================
  ORbtnAddCurrentClick
    ORbtnAddCurrentClick is a method that will add reports in frmMain.LstDocs
      to the report builder listbox.  In patch 149 it was modified to handle
      multiple selected reports.

  Modification History:
    CodeCR126 - 07/2010 Method modified to handle multiple selected items -MER
    CodeCR167 - 03/2011 Method modified to restore selected items.  In addition
      changes were made to notify if no items were added -MER
 ==============================================================================}
procedure TFormReportBuilder.ORbtnAddCurrentClick(Sender: TObject);
var
  ndx: integer;
  lslSelected: TStringList;
  lOneAdded: boolean; // CodeCR167 -MER
begin
  // When showing the report after hitting the build button, the TabIndex is set
  // to -1.  There were originally several places in this procedure that checked
  // for tab index = -1 and they were consolidated to this one check.  -MER CodeCR126
  if (frmMain.Tab95Control1.TabIndex = -1) then
    exit;
  if (frmMain.LstDocs.SelCount = 0) then begin
    if (frmMain.LstDocs.Items.Count > 0) then begin
      // There are reports available, but none selected.  Likely a user error, so inform the user
      ShowMessageCAPRI('No reports are selected');
      exit;
    end else if (DocType <> 'MULTI') then begin // DocType is set to multi when building the composite report
      AddBlankReport; // CAPRI allows one to add an absence of available reports to the composite report
    end;
  end else begin
    lOneAdded := FALSE; // CodeCR167 -MER
    orbtnPrintReport.Enabled := False;
    orbtnSend2VVA.Enabled := False;  //CodeCR353 - rpm 6/26/12
    lslSelected := TStringList.Create;
    screen.cursor := crHourglass;
    try
      // save all the selected item indexes
      for ndx := 0 to (frmMain.LstDocs.Items.Count - 1) do begin
        if (frmMain.lstDocs.Selected[ndx]) then begin
          lslSelected.Add(IntToStr(ndx));
        end;
        // we set selected to false to allow the itemIndex property to work correctly when iterating through the selected items
        frmMain.lstDocs.Selected[ndx] := FALSE;
      end; // for ndx := 0 to (frmMain.LstDocs.Count - 1) do
      // iterate through the selected items and add them to reportBuilder
      for ndx := 0 to (lslSelected.Count - 1) do begin
         if AddReportToList(StrToInt(lslSelected[ndx])) then
           lOneAdded := TRUE; // CodeCR167 -MER
      end;
      // CodeCR167 -MER 03/2011 restore selected items
      for ndx := 0 to (lslSelected.Count - 1) do begin
        frmMain.lstDocs.Selected[StrToInt(lslSelected[ndx])] := TRUE;
      end;
      if lslSelected.Count > 1 then begin
        frmMain.memoDocs.Lines.Clear;
        frmMain.memoDocs.Lines.Add('Multiple documents are selected.');
      end;
    finally
      lslSelected.Free;
      screen.cursor := crDefault;
    end;
    AfterAddCurrentClick;
    if not(lOneAdded) then // CodeCR167 -MER
      ShowMessageCAPRI('All selected items are already contained in the report.');
  end;
end; // ORbtnAddCurrentClick

procedure TFormReportBuilder.ORbtnClearClick(Sender: TObject);
begin
  OrListBoxItems.Clear;
  MemoDocs.Lines.Clear;
  orBtndeleteReport.enabled := false;
  btnMoveDown.Enabled := False;
  btnMoveUp.Enabled := False;
  orbtnClear.Enabled := False;
  orbtnPrintReport.Enabled := False;
  orbtnSend2VVA.Enabled := False;  //CodeCR353 - rpm 6/26/12
end;

procedure TFormReportBuilder.orBtndeleteReportClick(Sender: TObject);
var
  x: integer;
begin
  for x := 0 to memodocs.lines.count - 1 do
    if memodocs.lines[x] = '*' + ORListBoxItems.Items[ORListBoxItems.ItemIndex] then begin
      repeat
        memodocs.lines.delete(x)
      until (memodocs.lines[x] = '^*end') or (x > memodocs.lines.count - 1);
      memodocs.lines.delete(x);
    end;
  if ORListBoxItems.ItemIndex > -1 then
    ORListBoxItems.Items.Delete(ORListBoxItems.ItemIndex);

  orBtndeleteReport.enabled := false;
  btnMoveDown.Enabled := False;
  btnMoveUp.Enabled := False;
  if memodocs.lines.count = 0 then begin
    orbtnClear.Enabled := False;
    orbtnPrintReport.Enabled := False;
    orbtnSend2VVA.Enabled := False;  //CodeCR353 - rpm 6/26/12
  end;

end;

procedure TFormReportBuilder.ORbtnPrintReportClick(Sender: TObject);
var
  x, y, z: integer;
  lasty, maxy: integer;
  linesprocessed: integer;
  lasttype: string;
  function getheader(tp: string): string;
  begin
    if tp = 'PN' then
      result := 'PROGRESS NOTES'
    else
      if tp = 'DS' then
        result := 'DISCHARGE SUMMARIES'
      else
        if tp = 'CN' then
          result := 'CONSULTS'
        else
          if tp = 'VS' then
            result := 'VITAL SIGNS'
          else
            if tp = 'MEDS' then
              result := 'MEDICATIONS'
            else
              if tp = 'LAB' then
                result := 'LAB'
              else
                if tp = 'IMAGING' then
                  result := 'IMAGING'
                else
                  if tp = 'DIET' then
                    result := 'DIET'
                  else
                    if tp = 'NUTASSESS' then
                      result := 'NUTRITIONAL ASSESSMENT'
                    else
                      if tp = 'ORDERS' then
                        result := 'ORDER SUMMARY'
                      else
                        if tp = 'PROCEDURES' then
                          result := 'PROCEDURES'
                        else
                          if tp = 'PROBLEM LIST' then
                            result := 'PROBLEM LIST'
                          else
                            result := 'OTHER DOCUMENTATION';

  end;
begin
  lasttype := '';
  frmMain.AnimateLogo(True);
  frmMain.StatusBarLoadPt.Caption := 'Building report...';
  screen.cursor := crHourglass;
  formReportBuilder.Caption := IntToStr(memodocs.lines.count - 1) + ' lines of data';
  Gauge1.Progress := 0;
  Gauge1.Visible := True;
  Gauge1.MaxValue := orListBoxItems.Items.Count - 1;
  frmMain.buttonClinDocDateRange.Enabled := False;
  frmMain.lstDocs.ItemIndex := -1;
  frmMain.memodocs.lines.clear;
  frmMain.MemoDocs.Visible := False;
  frmMain.Splitter2.Visible := False;
  lasty := 0;
  linesprocessed := 0;
  for x := 0 to orListBoxItems.Items.Count - 1 do begin
    gauge1.Progress := x;
    screen.cursor := crHourglass;
    maxy := memodocs.lines.count - 1;
    // Trick it into starting where it last stopped.  If it hits the
    // end, start over at the beginning and go to where it started.
    repeat begin
        y := lasty - 1;
        while y <= maxy do begin
          inc(y);
          //formReportBuilder.Caption:=inttostr(y);
          if memodocs.lines[y] = '*' + ORListBoxItems.Items[x] then begin
            if LastType <> frmMain.Piece(frmMain.Piece(ORListBoxItems.Items[x], '^', 2), ',', 1) then begin
              LastType := frmMain.Piece(frmMain.Piece(ORListBoxItems.Items[x], '^', 2), ',', 1);
              frmmain.MemoDocs.Lines.Add('');
              frmmain.MemoDocs.Lines.Add(Copy('** ' + GetHeader(LastType) + ' ******************************************************************************', 1, 78));
              frmmain.MemoDocs.Lines.Add('');
            end;
            inc(linesprocessed);
            z := y + 1;
            repeat begin
                if memodocs.lines[z] <> '^*end' then
                  frmmain.MemoDocs.lines.add(memodocs.lines[z]);
                application.processmessages;
                inc(z);
                inc(linesprocessed);
            end until memodocs.lines[z] = '^*end';
            lasty := -1 * z;
            y := maxy + 1;
          end;
          formReportBuilder.Caption := IntToStr(linesprocessed) + '/' + IntToStr(memodocs.lines.count - 1) + ' lines';
        end;
        if lasty >= 0 then begin
          maxy := lasty;
          lasty := 0;
        end;
    end until lasty < 0;
    lasty := -1 * lasty
  end;
  frmMain.Tab95Control1.TabIndex := -1;
  frmMain.lstDocs.Items.Clear;
  //frmMain.MemoDocs.SelStart:=999999999;  frmMain.MemoDocs.SelLength:=0;
  frmMain.MemoDocs.Visible := true;
  frmMain.MemoDocs.SelStart := 0;
  frmMain.MemoDocs.SelLength := 0;
  //CodeCR353 - rpm 6/26/12 - leave focus on report builder to allow Send button
  //                          accelerator to be accessed.
  //frmMain.MemoDocs.SetFocus;
  DocType := 'MULTI';
  frmMain.LabelDocsFound.Caption := '';
  (*
  frmMain.memoDocs.Clear;
  frmMain.lstDocs.ItemIndex:=-1;
  If ORListBoxItems.Items.Count>0 then begin;
    // Set up for all document types
    tempDivisionrun:=DivisionRun;
    tempClinDocDateRangeSelected:=ClinDocDateRangeSelected;
    ClinDocDateRangeSelected:=False;
    DivisionRun:=False;
    frmMain.Tab95Control1Change(Application);
    for x:=0 to ORListBoxItems.Items.Count-1 do begin;
      frmMain.Tab95Control1.TabIndex:=StrToInt(frmMain.Piece(ORListBoxItems.Items[x],'^',1));
      frmMain.Tab95Control1Change(Application);
      if frmMain.lstDocs.Items.Count>0 then
        for y:=0 to frmMain.lstDocs.Items.Count-1 do
         If orListBoxItems.items[x]=frmMain.Piece(orListBoxItems.Items[x],'^',1)+'^'+frmMain.lstDocs.Items[y] then begin;
           frmMain.lstDocs.ItemIndex:=y;
           frmMain.lstDocsClick(Application);
         end;
    end;
    // Set back to initial search range
    DivisionRun:=tempDivisionrun;
    ClinDocDateRangeSelected:=tempClinDocDateRangeSelected;
  end;
  *)
  ORbtnAddCurrent.Enabled := False;
  frmMain.PanelSearch.Visible := False;
  frmMain.PanelCount.Visible := False;
  frmMain.lstDocs.Visible := False;
  formReportBuilder.Caption := 'Report Builder';
  Gauge1.Visible := False;
  screen.cursor := crDefault;
  frmMain.StatusBarLoadPt.Caption := 'Ready.';
  frmMain.AnimateLogo(False);
  screen.cursor := crDefault;
  if (frmMain.VirtualVAURL <> '') and     //CodeCR353 - rpm 6/26/12
     (frmMain.VirtualVAToken <> '') then
    orbtnSend2VVA.Enabled := True;
end;

procedure TFormReportBuilder.ORbtnSend2VVAClick(Sender: TObject);
begin
  ORbtnSend2VVA.Enabled := False;  //CodeCR353 - rpm 6/26/12
  frmMain.actFileTransmitVirtualVAExecute(Self);  //CodeCR353 - rpm 6/26/12
  ORBtnSend2VVA.Enabled := True;  //CodeCR353 - rpm 6/26/12
end;

procedure TFormReportBuilder.ORListBoxItemsClick(Sender: TObject);
begin
  orBtnDeleteReport.Enabled := True;
  if ORListBoxItems.ItemIndex > 0 then
    btnMoveUp.Enabled := True
  else
    btnMoveUp.Enabled := False;
  if ORListBoxItems.ItemIndex < ORListBoxItems.Items.Count - 1 then
    btnMoveDown.Enabled := True
  else
    btnMoveDown.Enabled := False;
end;

procedure TFormReportBuilder.ORListBoxItemsDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if Sender = ORListBoxItems then
    Accept := true
  else
    Accept := False;

end;

procedure TFormReportBuilder.ORListBoxItemsEndDrag(Sender, Target: TObject;
  X, Y: Integer);
begin
  (*
    tempstring:=ORListBoxItems.Items[ORListBoxItems.ItemIndex];
    newlocation:=Inttostr(y div ORListBoxItems.Font.Height);
    If newLocation<0 then newlocation:=newlocation*-1;
    OldLocation:=
    ShowMessageCAPRI(newLocation);
    *)
end;

procedure TFormReportBuilder.FormResize(Sender: TObject);
begin
  btnMoveDown.Left := formReportBuilder.Width div 2;
  btnMoveUp.Width := btnMoveDown.Left - btnMoveUp.Left - 1;
  btnMoveDown.Width := ORListBoxItems.Width - btnMoveUp.Width - 1;
end;

procedure TFormReportBuilder.FormShow(Sender: TObject);
begin
  orbtnSend2VVA.Enabled := False;  //CodeCR353 - rpm 6/26/12
end;

procedure TFormReportBuilder.btnmovedownClick(Sender: TObject);
var
  oldspot, newspot: integer;
  tempstring: string;
begin
  oldspot := ORListBoxItems.ItemIndex;
  NewSpot := OldSpot + 1;
  TempString := ORListBoxItems.Items[OldSpot];
  ORListBoxItems.Items.Delete(OldSpot);
  if NewSpot < ORListBoxItems.Items.Count then
    ORListBoxItems.Items.Insert(NewSpot, TempString)
  else
    ORListBoxItems.Items.Add(TempString);
  //ORListBoxItems.Items.Delete(OldSpot);
  ORListBoxItems.ItemIndex := NewSpot;
  ORListBoxItemsClick(Application);
end;

procedure TFormReportBuilder.btnMoveUpClick(Sender: TObject);
var
  oldspot, newspot: integer;
  tempstring: string;
begin
  oldspot := ORListBoxItems.ItemIndex;
  NewSpot := OldSpot - 1;
  TempString := ORListBoxItems.Items[OldSpot];
  ORListBoxItems.Items.Delete(OldSpot);
  ORListBoxItems.Items.Insert(NewSpot, TempString);
  ORListBoxItems.ItemIndex := NewSpot;
  ORListBoxItemsClick(Application);
end;

procedure TFormReportBuilder.ORAlignButtonAddAllClick(Sender: TObject);
var
  i: integer;
begin
  dvbaz := 0;
  // since we have changed the listbox to multi-select, we need to clear the
  // selected items so that itemIndex will work properly
  for i := 0 to (frmMain.LstDocs.Items.Count - 1) do
    frmMain.lstDocs.Selected[i] := FALSE; // -MER CodeCR126
  TimerAddAll.Enabled := True;
end;

procedure TFormReportBuilder.ORAlignButtonAddStopClick(Sender: TObject);
begin
  dvbaz := frmMain.LstDocs.Items.Count + 1; // force stop in timer
  ShowMessageCAPRI('Add documents interrupted.');
  ORAlignButtonAddStop.Visible := False;
  if ORListBoxItems.Items.Count > 0 then begin
    ORbtnClear.Enabled := True;
    ORbtnPrintReport.Enabled := True;
    orbtnSend2VVA.Enabled := False;  //CodeCR353 - rpm 6/26/12
  end;
end;

procedure TFormReportBuilder.TimerAddAllTimer(Sender: TObject);
var
  x: integer;
  newitem: string;
begin
  if ANURemoteProcedureCallInProgress = True then
    exit;

  ORbtnAddCurrent.Enabled := False;
  timerAddAll.Enabled := False;
  for x := 1 to 300000 do
    application.processmessages;
  if frmMain.LstDocs.Items.Count > 0 then
    if dvbaz < frmMain.LstDocs.Items.Count then begin

      Gauge1.Visible := True;
      Gauge1.MaxValue := frmMain.lstDocs.Items.Count - 1;
      Gauge1.Progress := dvbaz;

      ORbtndeleteReport.Enabled := False;
      ORbtnAddCurrent.Enabled := False;
      ORbtnClear.Enabled := False;
      ORbtnPrintReport.Enabled := False;
      orbtnSend2VVA.Enabled := False;  //CodeCR353 - rpm 6/26/12
      btnMoveUp.Enabled := False;
      btnMoveDown.enabled := False;

      ORAlignButtonAddStop.Visible := True;
      ORAlignButtonAddStop.BringToFront;
      ORListBoxItems.BringToFront;

      frmMain.LstDocs.ItemIndex := dvbaz;
      frmMain.lstDocsClick(Application);

      newitem := '';

      if frmMain.LstDocs.ItemIndex > -1 then
        newitem := IntToStr(frmMain.Tab95Control1.TabIndex) + '^' + Doctype + ', ' + frmMain.LstDocs.Items[frmMain.LstDocs.ItemIndex];
      if frmMain.LstDocs.ItemIndex = -1 then
        if DocType <> 'MULTI' then
          newitem := IntToStr(frmMain.Tab95Control1.TabIndex) + '^' + Doctype;
      if newitem = '' then
        exit;

      // Make sure it's not in the list yet
      if frmMain.Tab95Control1.TabIndex > -1 then
        if OrListBoxItems.Items.Count > 0 then
          for x := 0 to ORListBoxItems.Items.Count - 1 do
            if newitem = ORListBoxItems.Items[x] then begin
              ShowMessageCAPRI('That item is already on the list.  Skipping...');
              inc(dvbaz);
              timerAddAll.Enabled := True;
              exit;
            end;

      // Add it
      if frmMain.Tab95Control1.TabIndex > -1 then
        ORListBoxItems.Items.Add(newitem);

      orBtndeleteReport.enabled := False;
      btnMoveDown.Enabled := False;
      btnMoveUp.Enabled := False;

      FormResize(Application);

      //Copy text
      screen.cursor := crHourglass;
      if frmMain.memoDocs.Lines.Count > 0 then begin
        memodocs.lines.add('*' + newitem);
        for x := 0 to frmMain.memodocs.lines.count - 1 do
          memodocs.lines.add(frmMain.memodocs.lines[x]);
        memodocs.lines.add('');
        memodocs.lines.add('^*end');
      end;
      screen.cursor := crDefault;

      ORListBoxItems.ItemIndex := ORListBoxItems.Items.Count - 1;
      ORListBoxItems.ItemIndex := -1;
      inc(dvbaz);
      TimerAddAll.Enabled := True;
    end
    else begin
      frmMain.lstDocs.ItemIndex := -1;
      dvbaz := 0;
      ORbtnAddCurrent.Enabled := False;
      ORAlignButtonAddStop.Visible := False;
      ORbtnClear.Enabled := True;
      ORbtnPrintReport.Enabled := True;
      orbtnSend2VVA.Enabled := False;  //CodeCR353 - rpm 6/26/12
      ORListBoxItems.ItemIndex := -1;
      Gauge1.Visible := False;
    end;
end;

procedure TFormReportBuilder.BitBtnCancelClick(Sender: TObject);
begin
  self.Close;
end;

end.
